// AocsData.cpp
//
// Version	1.0
// Date		10.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include required for use of sqrt function
#include <math.h>

// Include framework header files
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../AocsData/AocsData.h"
#include "../AocsEvent/EventTypeId.h"
#include "../ObjectMonitoring/Property.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsData/DataItemRead.h"
#include "../TelemetryManagement/TelemetryStream.h"

// -------------------------------------------------------------------
// Default constructor.
// NB:	Unlike other constructors in the framework, this constructor does 
//		not call the "reset" and "resetConfiguration" methods because 
//		these methods use the pure virtual function "getNumberOfItems" and cannot
//		therefore be used in the constructor. It is the responsibility of 
//		derived classes to call these methods

AocsData::AocsData()				
{
	setClassId(ID_AOCSDATA);			
}

// -------------------------------------------------------------------
// The code in this function performs some initialization actions. 
// Ideally this code, should be put in the constructor. This is not 
// possible because the code calls a pure virtual function (getNumberOfItems).
// Hence, method initialize should be called by the constructor of the
// classes derived from AocsData

void AocsData::initialize()
{
	setTmImageLength(shortTm, 0);	
	setTmImageLength(normalTm, 8*getNumberOfItems()*sizeof(float)) ;
	setTmImageLength(longTm, 8*getNumberOfItems()*sizeof(float)+
									8*getNumberOfItems()*sizeof(AocsTime));
	setTmImageLength(debugTm, 0);	// not implemented in fw prototype

	dataItem = new Real[getNumberOfItems()];
	timeTag = new AocsTime[getNumberOfItems()]; 
}

// -------------------------------------------------------------------

void AocsData::localReset()
{
	earliestTimeTag = 0;
	latestTimeTag = 0;

	for (int i=0; i<getNumberOfItems(); i++)
	{	dataItem[i]=0;
		timeTag[i]=0;
	}
}

// -------------------------------------------------------------------

void AocsData::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void AocsData::localResetConfiguration()
{
	normalizationThreshold = 1;
	closenessThreshold = 1;
	recoveryAction = NULL;
}
// -------------------------------------------------------------------

void AocsData::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool AocsData::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (recoveryAction == NULL)
		return false;
	
	return true;
}

// -------------------------------------------------------------------

Real AocsData::distance(AocsData* d)
{
	Real temp=0.0;

	if (d==(AocsData*)NULL)				// return modulus of AOCS datum
	{	for (int i=0; i<getNumberOfItems(); i++)
			temp += getDataItemValue(i)*getDataItemValue(i);
		return (Real)sqrt(temp);
	}

	if (d->getFormat() != getFormat())		
	{	reportConfigurationError(this,INCOMPATIBLE_DATA_ITEMS,this);
		return LARGE_REAL;
	}
	else
	{	for (int i=0; i<getNumberOfItems(); i++)
			temp += (getDataItemValue(i) - (d->getDataItemValue(i))) * 
						(getDataItemValue(i) - (d->getDataItemValue(i)));
		return (Real)sqrt(temp);
	}
}

// -------------------------------------------------------------------

bool AocsData::close(AocsData* d)
{
	return (distance(d)<closenessThreshold);
}

// -------------------------------------------------------------------

bool AocsData::close(AocsData* d, Real epsilon)
{
	return (distance(d)<epsilon);
}

// -------------------------------------------------------------------

Real AocsData::size()
{
	return distance((AocsData*)NULL);
}

// -------------------------------------------------------------------

Real AocsData::normalize()
{
	return 0;
}

// -------------------------------------------------------------------

AocsTime AocsData::getTimeTag(int i)
{
	if (i<getNumberOfItems())
		return timeTag[i];
	else
	{	reportConfigurationError(this,ILLEGAL_DATA_ITEM_ACCESS,this);
		return 0;
	}
}

// -------------------------------------------------------------------

AocsTime AocsData::getEarliestTimeTag()
{
	AocsTime min=LARGE_TIME;
	for (int i=0; i<getNumberOfItems(); i++)
		if (timeTag[i]<min)
			min = timeTag[i];

	return min;
}

// -------------------------------------------------------------------

AocsTime AocsData::getLatestTimeTag()
{
	AocsTime max=0;
	for (int i=0; i<getNumberOfItems(); i++)
		if (timeTag[i]>max)
			max = timeTag[i];

	return max;
}

// -------------------------------------------------------------------

DataItemRead AocsData::getDataItemRead(int i)
{
	if (i<getNumberOfItems())
		return DataItemRead(&dataItem[i]);
	else
	{	reportConfigurationError(this,ILLEGAL_DATA_ITEM_ACCESS,this);
		return DataItemRead((Real*)NULL);
	}
}

// -------------------------------------------------------------------

Property AocsData::getProperty(int i)
{
	if (i<getNumberOfItems())
		return Property(&dataItem[i],i*MAX_INSTANCE_ID+getInstanceId());
	else
	{	reportConfigurationError(this,ILLEGAL_PROPERTY_ACCESS,this);
		return Property((Real*)NULL,0);
	}
}

// -------------------------------------------------------------------

DataItemWrite AocsData::getDataItemWrite(int i)
{
	if (i<getNumberOfItems())
			return DataItemWrite(&timeTag[i], &dataItem[i]);
	else
	{	reportConfigurationError(this,ILLEGAL_DATA_ITEM_ACCESS,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}
}

// -------------------------------------------------------------------

void AocsData::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	tmStream->write((unsigned char)getNumberOfItems());

	for (int i=0; i<getNumberOfItems(); i++)
		switch(getTelemetryFormat()) 
		{
			case longTm:
					tmStream->write(timeTag[i]);
			case normalTm:
					tmStream->write((float)dataItem[i]);
		}
}

// -------------------------------------------------------------------

int AocsData::getTelemetryImageLength()
{
	return (getLocalTelemetryImageLength(getTelemetryFormat())
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------

bool AocsData::doConsistencyCheck()
{
	return (normalize()<normalizationThreshold);
}