// AocsData.h
//
// Version	1.0
// Date		10.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Base class for all AOCS data classes
//
// -------------------------------------------------------------------

#ifndef AocsDataH
#define AocsDataH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../ObjectMonitoring/Property.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsData/DataItemRead.h"

// -------------------------------------------------------------------

class AocsData : public AocsObject, public ConsistencyCheckable {

	Real* dataItem;			// array to hold data items
	AocsTime* timeTag;;		// array to hold data items time tags
	AocsTime earliestTimeTag;
	AocsTime latestTimeTag;

	Real normalizationThreshold;
	Real closenessThreshold;

	RecoveryAction* recoveryAction;		// associated to Consistency Check failure

	void localReset();		
	void localResetConfiguration();		

protected :

	AocsDataFormat format;

	void initialize();

	Real getDataItemValue(int i)
		{ return dataItem[i]; }

	virtual void setTmImageLength(TelemetryFormat, int i)=0;
	virtual int getLocalTelemetryImageLength(TelemetryFormat)=0;

public :

	AocsData();

	virtual int getNumberOfItems()=0;

	virtual Real distance(AocsData* d);			// Metrics methods. Default implementation
	bool close(AocsData* d);					// is based on Euclidean distance
	bool close(AocsData* d, Real epsilon);
	Real size();

	AocsTime getEarliestTimeTag();
	AocsTime getLatestTimeTag();
	AocsTime getTimeTag(int i);

	virtual Real normalize();

	DataItemRead getDataItemRead(int i);	
	DataItemWrite getDataItemWrite(int i);
	Property getProperty(int i);

	// pure virtual methods from interface Resettable
	virtual void reset();

	// pure virtual methods from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();
	
	// pure virtual methods from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// pure virtual methods from interface ConsistencyCheckable
	virtual bool doConsistencyCheck();

	// straightforward getter and setter methods
	AocsDataFormat getFormat()
		{ return format; }
	void setClosenessThreshold(Real epsilon)
		{ closenessThreshold = epsilon; }
	void setNormalizationThreshold(Real newThreshold)
		{ normalizationThreshold = newThreshold; }
	RecoveryAction* getRecoveryAction()
		{ return recoveryAction; }
	void setRecoveryAction(RecoveryAction* r)
		{ recoveryAction = r; }
};

// -------------------------------------------------------------------
#endif
