// AocsDataPool.cpp
//
// Version	1.0
// Date		14.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsData/AocsDataPool.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsData/AocsData.h"
#include "../TelemetryManagement/TelemetryStream.h"

// -------------------------------------------------------------------
// Default constructor.
// NB:	Unlike other constructors in the framework, this constructor does 
//		not call the "reset" and "resetConfiguration" methods because 
//		these methods rely on the aocsDataList having already been configured
//		and this is done only in the derived classes. It is the responsibility of 
//		derived classes to call the two reset methods

AocsDataPool::AocsDataPool()
{
	setClassId(ID_AOCSDATAPOOL);			// assign class identifier
}

// -------------------------------------------------------------------

void AocsDataPool::localReset()
{
	for (AocsData* obj=aocsDataList.first(); 
						!aocsDataList.isLast();
							obj=aocsDataList.next() )
		obj->reset();
}

// -------------------------------------------------------------------

void AocsDataPool::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void AocsDataPool::localResetConfiguration()
{
	recoveryAction = NULL;
	
	for (AocsData* obj=aocsDataList.first(); 
						!aocsDataList.isLast();
							obj=aocsDataList.next() )
		obj->resetConfiguration();
}

// -------------------------------------------------------------------

void AocsDataPool::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------
// A data pool is configured if all its members are configured

bool AocsDataPool::isConfigured()
{
	for (AocsData* obj=aocsDataList.first(); 
						!aocsDataList.isLast();
							obj=aocsDataList.next() )
		if ( !(obj->isConfigured()) )
			return false;

	return true;
}

// -------------------------------------------------------------------
// Only normalTm and longTm formats are supported by the prototype f/w

void AocsDataPool::writeToTelemetry(TelemetryStream* stream)
{
	AocsObject::writeToTelemetry(stream);

	// Compute the number of data items and of AocsData in the pools
	unsigned char nDataItems=0;
	unsigned char nAocsData=0;
	for (AocsData* d=aocsDataList.first(); 
					!aocsDataList.isLast();
					  d=aocsDataList.next() )
	{	nAocsData++;
		for (int i=0; i<d->getNumberOfItems(); i++)
			nDataItems++;
	}

	if (getTelemetryFormat()==normalTm)
	{	
		stream->write(nDataItems);

		for (AocsData* d=aocsDataList.first(); 
						!aocsDataList.isLast();
						  d=aocsDataList.next() )
			for (int i=0; i<d->getNumberOfItems(); i++)
				stream->write((float)d->getDataItemRead(i).get());
	}
	
	if (getTelemetryFormat()==longTm)
	{	
		stream->write(nAocsData);

		for (AocsData* d=aocsDataList.first(); 
						!aocsDataList.isLast();
						  d=aocsDataList.next() )
			d->writeToTelemetry(stream);
	}
}

// -------------------------------------------------------------------
// The telemetry format of data pool is imposed on all data in pool

void AocsDataPool::setTelemetryFormat(TelemetryFormat newFormat)
{
	AocsObject::setTelemetryFormat(newFormat);

	for (AocsData* obj=aocsDataList.first(); 
						!aocsDataList.isLast();
							obj=aocsDataList.next() )
		obj->setTelemetryFormat(newFormat);
}

// -------------------------------------------------------------------

int AocsDataPool::getTelemetryImageLength()
{
	int temp=8;	 // first byte holds n. of data items/AocsData in pool		

	if (getTelemetryFormat()==normalTm)
	{	
		for (AocsData* obj=aocsDataList.first(); 
							!aocsDataList.isLast();
								obj=aocsDataList.next() )
			temp+=8*4*obj->getNumberOfItems();

		return (temp+AocsObject::getTelemetryImageLength());
	}

	if (getTelemetryFormat()==normalTm)
	{	
		for (AocsData* obj=aocsDataList.first(); 
							!aocsDataList.isLast();
								obj=aocsDataList.next() )
			temp+=obj->getTelemetryImageLength();

		return (temp+AocsObject::getTelemetryImageLength());
	}

	return 0;
}

// -------------------------------------------------------------------
// A data pool is consistent if all its members are consisten

bool AocsDataPool::doConsistencyCheck()
{
	for (AocsData* obj=aocsDataList.first(); 
						!aocsDataList.isLast();
							obj=aocsDataList.next() )
		if ( !(obj->doConsistencyCheck()) )
			return false;

	return true;
}
