// AocsUnitFactory.cpp 
//
// Version	1.0
// Date		15.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// AOCS Unit Factory Class 
// AOCS units are created, configured and linked to their destination
// and source buffers. 
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsUnit/AocsUnit.h"
#include "../AocsUnit/AocsUnitHardware.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../SequentialDataProcessing/BiasScalingCompensator.h"
#include "../SequentialDataProcessing/TorqueToThrusterPrototype.h"
#include "../AocsUnit/FssPrototype.h"
#include "../AocsUnit/GyrPrototype.h"
#include "../AocsUnit/SapPrototype.h"
#include "../AocsUnit/RwPrototype.h"
#include "../AocsUnit/TorquingThrusters.h"
#include "../ReconfigurationManagement/RwSet.h"
#include "../AocsUnit/TriggerList.h"
#include "../AocsUnit/FollowerUnitTriggerModeManager.h"
#include "../AocsUnit/UnitTrigger.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsData/AocsData.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// Include the AOCS factory files
#include "../AocsFactory/AocsUnitFactory.h"

// -------------------------------------------------------------------

AocsUnitFactory::AocsUnitFactory()
{
	// Initialize the unit and trigger list references
	fss = NULL;
	fssA = NULL;
	fssB = NULL;
	gyrA = NULL;
	gyrB = NULL;
	gyr = NULL;
	sapA = NULL;
	sapB = NULL;
	sap = NULL;
	thu = NULL;
	rwSet = NULL;

	for (int i=0; i<4; i++)
		rw[i] = NULL;

	triggerListAcquisition = NULL;
	cspTriggerListSend = NULL;
	fspTriggerListSend = NULL;

	unitTriggerModeMngAcquisition = NULL;
	unitTriggerModeMngSend = NULL;

	unitTriggerAcquisition = NULL;
	unitTriggerSend = NULL;

	unitHw = NULL;
}

// -------------------------------------------------------------------
// Make a MACS telecom controller. The methods parameters are the 
// addresses of the MACS registers. If NULL references are passed, 
// then it is assumed that there is no external MACS interface and
// that the MACS registers must be simulated.

void AocsUnitFactory::makeAocsUnitHardware(short* macsCsReg, 
											short* macsInstrReg,
											 short* macsDataReg)
{
	// Only one copy should be created
	if (unitHw != NULL)			
		return;

	if ((macsCsReg==NULL)||(macsInstrReg==NULL)||(macsDataReg==NULL))
	{	macsCsReg = new short;
		macsInstrReg = new short;
		macsDataReg = new short;
	}

	unitHw = new MacsTcController(macsCsReg,macsInstrReg,macsDataReg);
	unitHw->setMaxPolling(2);
}

// -------------------------------------------------------------------
// Two identical redundant FSS are built and embedded in a 
// reconfiguration manager.

void AocsUnitFactory::makeFss(AocsData* fssData, 
								RecoveryAction* hwFailureRecActA,
									RecoveryAction* hwFailureRecActB,
										RecoveryAction* reconfigErrorRecAct)
{
	// Only one copy should be created
	if (fssA != NULL)			
		return;

	// Create FSS units
	fssA = new FssPrototype(MACS_FSS_A,FSS_SA_X,FSS_SA_Y);
	fssB = new FssPrototype(MACS_FSS_B,FSS_SA_X,FSS_SA_Y);

	// Plug hw unit into FSS units
	fssA->setAocsUnitHardware(unitHw);
	fssB->setAocsUnitHardware(unitHw);

	// Create the converter from raw to AOCS level data 
	RawBusDataConverterIn* convIn_a = new RawBusDataConverterIn(2,FSS_Q);
	RawBusDataConverterIn* convIn_b = new RawBusDataConverterIn(2,FSS_Q);

	// Create the compensator for the bias and scaling factors
	BiasScalingCompensator* bsc_x_a = new BiasScalingCompensator
												(BIAS_FSS_X_A,SCALING_FSS_X_A);
	BiasScalingCompensator* bsc_y_a = new BiasScalingCompensator
												(BIAS_FSS_Y_A,SCALING_FSS_Y_A);

	BiasScalingCompensator* bsc_x_b = new BiasScalingCompensator
												(BIAS_FSS_X_B,SCALING_FSS_X_B);
	BiasScalingCompensator* bsc_y_b = new BiasScalingCompensator
												(BIAS_FSS_Y_B,SCALING_FSS_Y_B);

	// Link the raw data converter with the bias/scaling compensator
	bsc_x_a->linkInput(convIn_a,0,0);
	bsc_y_a->linkInput(convIn_a,1,0);

	bsc_x_b->linkInput(convIn_b,0,0);
	bsc_y_b->linkInput(convIn_b,1,0);

	// Create the super block to act as converter for incoming FSS data
	ControlChannelSuperBlock* fssConv_a = new ControlChannelSuperBlock(2,2);
	ControlChannelSuperBlock* fssConv_b = new ControlChannelSuperBlock(2,2);

	// Embed the raw data converter and bias/scaling compensator in the 
	// FSS in-data converters
	fssConv_a->linkInputInternal(convIn_a,0,0);
	fssConv_a->linkInputInternal(convIn_a,1,1);
	fssConv_a->linkOutputInternal(bsc_x_a,0,0);
	fssConv_a->linkOutputInternal(bsc_y_a,0,1);

	fssConv_b->linkInputInternal(convIn_b,0,0);
	fssConv_b->linkInputInternal(convIn_b,1,1);
	fssConv_b->linkOutputInternal(bsc_x_b,0,0);
	fssConv_b->linkOutputInternal(bsc_y_b,0,1);

	// Plug the data converters into the FSS units
	fssA->setFunctionalInConverter(fssConv_a);
	fssB->setFunctionalInConverter(fssConv_b);

	// Link the FSS to its data destination buffer 
	fssA->setFunctionalInLink(fssData);
	fssB->setFunctionalInLink(fssData);

	// Plug the recovery action into the fss
	fssA->setHwFailureRecoveryAction(hwFailureRecActA);
	fssB->setHwFailureRecoveryAction(hwFailureRecActB);

	// Create the reconfiguration manager and plug the FSS units into it
	fss = new BasicUnitReconfigurer(2,0,0);
	fss->setUnit(fssA,0);
	fss->setUnit(fssB,1);

	// Plug the recovery action into the FSS reconfiguration manager
	fss->setReconfigurationErrorRecoveryAction(reconfigErrorRecAct);
}

// -------------------------------------------------------------------
// Two identical redundant gyros are built and embedded in a 
// reconfiguration manager.

void AocsUnitFactory::makeGyr(AocsData* gyrData, 
								RecoveryAction* hwFailureA,
									RecoveryAction* hwFailureB,
										RecoveryAction* reconfigError)
{
	// Only one copy should be created
	if (gyrA != NULL)			
		return;

	// Create GYR units
	gyrA = new GyrPrototype(MACS_GYR_A,GYR_SA_RATE);
	gyrB = new GyrPrototype(MACS_GYR_B,GYR_SA_RATE);

	// Plug hw unit into GYR units
	gyrA->setAocsUnitHardware(unitHw);
	gyrB->setAocsUnitHardware(unitHw);

	// Create the converter from raw to AOCS level data 
	RawBusDataConverterIn* convIn_a = new RawBusDataConverterIn(1,GYR_RAT_Q);
	RawBusDataConverterIn* convIn_b = new RawBusDataConverterIn(1,GYR_RAT_Q);

	// Create the compensator for the bias and scaling factors
	BiasScalingCompensator* bsc_a = new BiasScalingCompensator
												(BIAS_GYR_A,SCALING_GYR_A);
	BiasScalingCompensator* bsc_b = new BiasScalingCompensator
												(BIAS_GYR_B,SCALING_GYR_B);

	// Link the raw data converter with the bias/scaling compensator
	bsc_a->linkInput(convIn_a,0,0);
	bsc_b->linkInput(convIn_b,0,0);

	// Create the super block to act as converter for incoming GYR data
	ControlChannelSuperBlock* gyrConv_a = new ControlChannelSuperBlock(1,1);
	ControlChannelSuperBlock* gyrConv_b = new ControlChannelSuperBlock(1,1);

	// Embed the raw data converter and bias/scaling compensator in the 
	// GYR in-data converters
	gyrConv_a->linkInputInternal(convIn_a,0,0);
	gyrConv_a->linkOutputInternal(bsc_a,0,0);

	gyrConv_b->linkInputInternal(convIn_b,0,0);
	gyrConv_b->linkOutputInternal(bsc_b,0,0);

	// Plug the data converters into the GYR units
	gyrA->setFunctionalInConverter(gyrConv_a);
	gyrB->setFunctionalInConverter(gyrConv_b);

	// Link the GYR to its data destination buffer 
	gyrA->setFunctionalInLink(gyrData);
	gyrB->setFunctionalInLink(gyrData);

	// Plug the recovery action into the gyro 
	gyrA->setHwFailureRecoveryAction(hwFailureA);
	gyrB->setHwFailureRecoveryAction(hwFailureB);

	// Create the reconfiguration manager and plug the GYR units into it
	gyr = new BasicUnitReconfigurer(2,0,0);
	gyr->setUnit(gyrA,0);
	gyr->setUnit(gyrB,1);

	// Plug the recovery action into the FSS reconfiguration manager
	gyr->setReconfigurationErrorRecoveryAction(reconfigError);
}

// -------------------------------------------------------------------
// Construct and configure the i-th reaction wheel

void AocsUnitFactory::makeRw(int wheelNumber,
								DataItemWrite rwSpeed, 
									DataItemWrite rwTorque, 
										RecoveryAction* hwFailure)
{
	// Only one copy should be created
	if (rw[wheelNumber] != NULL)			
		return;
	
	// Check legality of wheel number
	if ((wheelNumber<0) || (wheelNumber>3))
		AocsObject::getConfigurationEventRepository()->
			create(NULL,ILLEGAL_FACTORY_PAR,NULL);
	
	// Create RW unit
	rw[wheelNumber] = new RwPrototype(MACS_RW,RW_SA_SPEED,RW_SA_TOR);
	
	// Plug hw unit into RW unit
	rw[wheelNumber]->setAocsUnitHardware(unitHw);

	// Create the converter from raw to AOCS level for speed read-out
	// NB No bias/scaling compensation is used for speed read-outs
	RawBusDataConverterIn* convIn = new RawBusDataConverterIn(1,RW_SPD_Q);

	// Create the converter from AOCS to raw level for torque demand
	// NB No bias/scaling compensation is used for torque demands
	RawBusDataConverterOut* convOut = new RawBusDataConverterOut(1,RW_TOR_Q);

	// Plug the data converters into the RW unit
	rw[wheelNumber]->setFunctionalInConverter(convIn);
	rw[wheelNumber]->setFunctionalOutConverter(convOut);

	// Link the RW unit to its data destination and data source buffers
	rw[wheelNumber]->setFunctionalOutLink(rwTorque,0);
	rw[wheelNumber]->setFunctionalInLink(rwSpeed,0);

	// Plug the recovery action into the RW
	rw[wheelNumber]->setHwFailureRecoveryAction(hwFailure);
}

// -------------------------------------------------------------------
// Construct and configure the reaction wheel set

void AocsUnitFactory::makeRwSet(AocsData* dataDestBuf, 
									AocsData* dataSourceBuf, 
										RecoveryAction* reconfigError)
{
	// Only one copy should be created
	if (rwSet != NULL)			
		return;

	// Construct the RW configuration matrix
	Real rwMatrix[3][4] = {
		{1, 0, 0, 0.58},
		{0, 1, 0, 0.58},
		{0, 0, 1, 0.58}	  };

	// Create the wheel reconfiguration manager 
	rwSet = new RwSet(0,rwMatrix,RW_INERTIA);

	// Check that the reaction wheels have already been configured
	// (they can only be plugged into the rwSet if they are 
	// configured)
	for (int i=0; i<4; i++)
		if (!rw[i]->isConfigured())
			AocsObject::getConfigurationEventRepository()->
				create(NULL,UNCONFIGURED_RW,NULL);

	// Plug the reaction wheels into the RwSet reconfiguration manager
	for (i=0; i<4; i++)
		rwSet->setRw(rw[i],i);

	// Link the RwSet unit to its data source and data destination buffers
	rwSet->setFunctionalOutLink(dataSourceBuf);
	rwSet->setFunctionalInLink(dataDestBuf);

	// Plug the recovery action into the RwSet reconfiguration manager
	rwSet->setReconfigurationErrorRecoveryAction(reconfigError);
}

// -------------------------------------------------------------------
// Two identical redundant sap units are built and embedded in a 
// reconfiguration manager.

void AocsUnitFactory::makeSap(AocsData* sas1, AocsData* sas2, AocsData* sas3,
									AocsData* thuOnTime, AocsData* thuDelayTime,
											RecoveryAction* hwFailureRecActA,
												RecoveryAction* hwFailureRecActB,
													RecoveryAction* reconfigErrorRecAct)
{
	// Only one copy should be created
	if (sapA != NULL)			
		return;

	// Create SAP units
	sapA = new SapPrototype(MACS_SAP_A,SAS_SA_X_1,THU_SA_DT_1);
	sapB = new SapPrototype(MACS_SAP_B,SAS_SA_X_1,THU_SA_DT_1);

	// Plug hw unit into SAP units
	sapA->setAocsUnitHardware(unitHw);
	sapB->setAocsUnitHardware(unitHw);

	// Create the converter from raw to AOCS level data for the SAS signals
	RawBusDataConverterIn* convIn_a = new RawBusDataConverterIn(6,SAS_Q);
	RawBusDataConverterIn* convIn_b = new RawBusDataConverterIn(6,SAS_Q);

	// Create the compensator for the SAS bias and scaling factors
	BiasScalingCompensator* bsc_a_0 = new BiasScalingCompensator
												(BIAS_SAS_1_A_X,SCALING_SAS_1_A_X);
	BiasScalingCompensator* bsc_a_1 = new BiasScalingCompensator
												(BIAS_SAS_2_A_X,SCALING_SAS_2_A_X);
	BiasScalingCompensator* bsc_a_2 = new BiasScalingCompensator
												(BIAS_SAS_3_A_X,SCALING_SAS_3_A_X);
	BiasScalingCompensator* bsc_a_3 = new BiasScalingCompensator
												(BIAS_SAS_1_A_Y,SCALING_SAS_1_A_Y);
	BiasScalingCompensator* bsc_a_4 = new BiasScalingCompensator
												(BIAS_SAS_2_A_Y,SCALING_SAS_2_A_Y);
	BiasScalingCompensator* bsc_a_5 = new BiasScalingCompensator
												(BIAS_SAS_3_A_Y,SCALING_SAS_3_A_Y);

	BiasScalingCompensator* bsc_b_0 = new BiasScalingCompensator
												(BIAS_SAS_1_B_X,SCALING_SAS_1_B_X);
	BiasScalingCompensator* bsc_b_1 = new BiasScalingCompensator
												(BIAS_SAS_2_B_X,SCALING_SAS_2_B_X);
	BiasScalingCompensator* bsc_b_2 = new BiasScalingCompensator
												(BIAS_SAS_3_B_X,SCALING_SAS_3_B_X);
	BiasScalingCompensator* bsc_b_3 = new BiasScalingCompensator
												(BIAS_SAS_1_B_Y,SCALING_SAS_1_B_Y);
	BiasScalingCompensator* bsc_b_4 = new BiasScalingCompensator
												(BIAS_SAS_2_B_Y,SCALING_SAS_2_B_Y);
	BiasScalingCompensator* bsc_b_5 = new BiasScalingCompensator
												(BIAS_SAS_3_B_Y,SCALING_SAS_3_B_Y);
	
	// Link the raw data converter with the bias/scaling compensator for the SAS
	bsc_a_0->linkInput(convIn_a,0,0);
	bsc_a_1->linkInput(convIn_a,1,0);
	bsc_a_2->linkInput(convIn_a,2,0);
	bsc_a_3->linkInput(convIn_a,3,0);
	bsc_a_4->linkInput(convIn_a,4,0);
	bsc_a_5->linkInput(convIn_a,5,0);

	bsc_b_0->linkInput(convIn_b,0,0);
	bsc_b_1->linkInput(convIn_b,1,0);
	bsc_b_2->linkInput(convIn_b,2,0);
	bsc_b_3->linkInput(convIn_b,3,0);
	bsc_b_4->linkInput(convIn_b,4,0);
	bsc_b_5->linkInput(convIn_b,5,0);

	// Create and configure the converter superblock for the SAS outputs
	ControlChannelSuperBlock* sasConv_a = new ControlChannelSuperBlock(6,6);
	for (int i=0; i<6; i++)
		sasConv_a->linkInputInternal(convIn_a,i,i);

	sasConv_a->linkOutputInternal(bsc_a_0,0,0);
	sasConv_a->linkOutputInternal(bsc_a_1,0,1);
	sasConv_a->linkOutputInternal(bsc_a_2,0,2);
	sasConv_a->linkOutputInternal(bsc_a_3,0,3);
	sasConv_a->linkOutputInternal(bsc_a_4,0,4);
	sasConv_a->linkOutputInternal(bsc_a_5,0,5);
	
	ControlChannelSuperBlock* sasConv_b = new ControlChannelSuperBlock(6,6);
	for (i=0; i<6; i++)
		sasConv_b->linkInputInternal(convIn_b,i,i);

	sasConv_b->linkOutputInternal(bsc_b_0,0,0);
	sasConv_b->linkOutputInternal(bsc_b_1,0,1);
	sasConv_b->linkOutputInternal(bsc_b_2,0,2);
	sasConv_b->linkOutputInternal(bsc_b_3,0,3);
	sasConv_b->linkOutputInternal(bsc_b_4,0,4);
	sasConv_b->linkOutputInternal(bsc_b_5,0,5);

	// Create the raw data converter for the THU commands.
	// The first six inputs are reserved to the delay-time commands and 
	// the last six inputs are reserved to the on-time commands.
	// It is assumed that there is no bias/scaling_factor compensation 
	// for the THU commands.
	RawBusDataConverterOut* thuConv = new RawBusDataConverterOut(12,1);

	// Plug the data converters into the SAP units
	sapA->setFunctionalInConverter(sasConv_a);
	sapB->setFunctionalInConverter(sasConv_b);
	sapA->setFunctionalOutConverter(thuConv);
	sapB->setFunctionalOutConverter(thuConv);

	// Link the SAP data destination buffers
	sapA->setFunctionalInLink(sas1->getDataItemWrite(0),0);
	sapA->setFunctionalInLink(sas1->getDataItemWrite(1),1);
	sapA->setFunctionalInLink(sas2->getDataItemWrite(0),2);
	sapA->setFunctionalInLink(sas2->getDataItemWrite(1),3);
	sapA->setFunctionalInLink(sas3->getDataItemWrite(0),4);
	sapA->setFunctionalInLink(sas3->getDataItemWrite(1),5);

	sapB->setFunctionalInLink(sas1->getDataItemWrite(0),0);
	sapB->setFunctionalInLink(sas1->getDataItemWrite(1),1);
	sapB->setFunctionalInLink(sas2->getDataItemWrite(0),2);
	sapB->setFunctionalInLink(sas2->getDataItemWrite(1),3);
	sapB->setFunctionalInLink(sas3->getDataItemWrite(0),4);
	sapB->setFunctionalInLink(sas3->getDataItemWrite(1),5);

	// Link the SAP data source buffers
	for (i=0; i<6; i++)
	{	sapA->setFunctionalOutLink(thuDelayTime->getDataItemWrite(i),i);
		sapB->setFunctionalOutLink(thuDelayTime->getDataItemWrite(i),i);
	}

	for (i=0; i<6; i++)
	{	sapA->setFunctionalOutLink(thuOnTime->getDataItemWrite(i),i+6);
		sapB->setFunctionalOutLink(thuOnTime->getDataItemWrite(i),i+6);
	}

	// Link the SAP hardware failure reconfiguration manager
	sapA->setHwFailureRecoveryAction(hwFailureRecActA);
	sapB->setHwFailureRecoveryAction(hwFailureRecActB);

	// Create the reconfiguration manager and plug the SAP units into it
	sap = new BasicUnitReconfigurer(2,0,0);
	sap->setUnit(sapA,0);
	sap->setUnit(sapB,1);

	// Link the SAP reconfiguration manager to its recovery action
	sap->setReconfigurationErrorRecoveryAction(reconfigErrorRecAct);
}

// -------------------------------------------------------------------
// Create a fictitious unit taking torques in sc reference frame as 
// inputs to be transformed into on-time requests for the thrusters

void AocsUnitFactory::makeThu(AocsData* dataSourceBuf)
{
	// Only one copy should be created
	if (thu != NULL)			
		return;

	thu = new TorquingThrusters(6);

	// Create the torque-to-thruster converter
	TorqueToThrusterPrototype* torToThuConv = 
		new TorqueToThrusterPrototype(T_THU,THU_PERIOD);

	// Plug the torque-to-thruster converter in the THU
	thu->setFunctionalOutConverter(torToThuConv);

	// Link the THU unit to its data source buffers
	thu->setFunctionalOutLink(dataSourceBuf);

	// Link the THU unit to the SAP (this should only be done if the SAP
	// is configured)
	if (sap->isConfigured())
		thu->setLowLevelThrusterUnit(sap);
	else
		AocsObject::getConfigurationEventRepository()->create(
			NULL,UNCONFIGURED_SAP,NULL);
}

// -------------------------------------------------------------------
// Create the unit trigger lists .
//
// Three trigger lists are created: 
//	- triggerListAcquisition: units from which data have to be acquired.
//			This is the same for both CSP and FSP mode. 
//	- cspTriggerListSend: units to which data have to be send in CSP mode
//	- fspTriggerListSend: units to which data have to be send in FSP mode

void AocsUnitFactory::makeUnitTriggerLists()
{
	// Only one copy should be created
	if (triggerListAcquisition != NULL)			
		return;

	// Create the trigger lists
	triggerListAcquisition = new FunctionalTriggerList(1,4,0,0);		
	cspTriggerListSend = new FunctionalTriggerList(0,0,1,0);		
	fspTriggerListSend = new FunctionalTriggerList(0,0,1,0);		
}

// -------------------------------------------------------------------
// Create the mode managers for the unit triggers and load them with  
// the trigger lists.
// Follower mode managers are used. The master mode manager is the
// the method argument.

void AocsUnitFactory::makeUnitTriggerModeManagers(
				ModeManager* masterModeManager, ModeChangeAction* mca)
{
	// Only one copy should be created
	if (unitTriggerModeMngAcquisition != NULL)			
		return;

	// Check that the trigger lists have been created
	if ((triggerListAcquisition==NULL) || (cspTriggerListSend==NULL) ||
		(fspTriggerListSend==NULL))
		AocsObject::getConfigurationEventRepository()->
			create(NULL,MISSING_TRIGGER_LIST,NULL);
	
	// Create and load the unit trigger mode managers
	unitTriggerModeMngAcquisition = new FollowerUnitTriggerModeManager(
					N_OF_MODES,0,masterModeManager);
	unitTriggerModeMngSend = new FollowerUnitTriggerModeManager(
					N_OF_MODES,0,masterModeManager);

	unitTriggerModeMngAcquisition->loadTriggerList(CSP,triggerListAcquisition);
	unitTriggerModeMngAcquisition->loadTriggerList(FSP,triggerListAcquisition);
	unitTriggerModeMngSend->loadTriggerList(CSP,cspTriggerListSend);
	unitTriggerModeMngSend->loadTriggerList(FSP,fspTriggerListSend);

	unitTriggerModeMngAcquisition->setModeChangeAction(mca);
	unitTriggerModeMngSend->setModeChangeAction(mca);
}

// -------------------------------------------------------------------
// Create and configure the unit triggers

void AocsUnitFactory::makeUnitTriggers()
{
	// Only one copy should be created
	if (unitTriggerAcquisition != NULL)			
		return;

	// Check that the mode managers have been created
	if ((unitTriggerModeMngAcquisition==NULL) || (unitTriggerModeMngSend==NULL))
		AocsObject::getConfigurationEventRepository()->
			create(NULL,MISSING_TRIGGER_MM,NULL);

	// Create the unit triggers	
	unitTriggerAcquisition = new UnitTrigger();
	unitTriggerSend = new UnitTrigger();

	// Plug the mode managers into the unit triggers
	unitTriggerAcquisition->setUnitTriggerModeManager(unitTriggerModeMngAcquisition);
	unitTriggerSend->setUnitTriggerModeManager(unitTriggerModeMngSend);
}