// FailureDetectionFactory.cpp 
//
// Version	1.0
// Date		14.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Failure Detection Framelet Factory Class 
// -------------------------------------------------------------------

// Include framework header files
#include "../FailureDetection/FailureDetectionManager.h"
#include "../FailureDetection/FailureDetectionModeManager.h"
#include "../FailureDetection/FollowerFailureDetectionModeManager.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../FailureDetection/MonitoringCheck.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// Include the AOCS factory files
#include "../AocsFactory/FailureDetectionFactory.h"

// -------------------------------------------------------------------

FailureDetectionFactory::FailureDetectionFactory()
{
	fdManager = NULL;
	fdModeManager = NULL;
	ccListArray= NULL;
	mcListArray= NULL;

	makeFailureDetectionLists();
}
	
// -------------------------------------------------------------------
// Create and configure the failure detection lists

void FailureDetectionFactory::makeFailureDetectionLists()
{
	if (ccListArray==NULL)				// do only once
	{	ccListArray = new 
				ObjectListTemplate<ConsistencyCheckable>*[N_FD_MODES];
		for (int i=0; i<N_FD_MODES; i++)
		{	ObjectListTemplate<ConsistencyCheckable>* ccList = 
					new ObjectListTemplate<ConsistencyCheckable>();
			ccList->allocateMemory(CC_LIST_SIZE);
			ccListArray[i] = ccList;
		}
	}

	if (mcListArray==NULL)				// do only once
	{	mcListArray = new 
				ObjectListTemplate<MonitoringCheck>*[N_FD_MODES];
		for (int i=0; i<N_FD_MODES; i++)
		{	ObjectListTemplate<MonitoringCheck>* mcList = 
					new ObjectListTemplate<MonitoringCheck>();
			mcList->allocateMemory(MC_LIST_SIZE);
			mcListArray[i] = mcList;
		}
	}
}

// -------------------------------------------------------------------
// Create and configure the failure detection mode manager

void FailureDetectionFactory::makeFailureDetectionModeManager(
						ModeManager* master, ModeChangeAction* mca)
{
	if (fdModeManager == NULL)				// create only once
	{	FollowerFailureDetectionModeManager* fdmm = 
			new FollowerFailureDetectionModeManager(
					N_FD_MODES,MAX_FD_MONITORS,master);
		
		// Plug in the mode change action
		fdmm->setModeChangeAction(mca);

		// Plug in failure detection lists
		for (int i=0; i<N_FD_MODES; i++)
		{	fdmm->loadConsistencyCheckableList(i,ccListArray[i]);
			fdmm->loadMonitoringCheckList(i,mcListArray[i]);
		}

		// Assign internal variable
		fdModeManager = fdmm;
	}
}

// -------------------------------------------------------------------
// Create and configure the failure detection manager (should only
// be called after the failure detection mode manager has been
// constructed).

void FailureDetectionFactory::makeFailureDetectionManager(
											ChangeEventRepository* rep)
{
	if (fdManager == NULL)					// create only once
	{	fdManager = new FailureDetectionManager();

		if (fdModeManager == NULL)
			AocsObject::getConfigurationEventRepository()->
				create(NULL,MISSING_FD_MM,NULL);
		else
		{	fdManager->setFailureDetectionModeManager(fdModeManager);
			fdManager->setChangeEventRepository(rep);
		}
	}
}

// -------------------------------------------------------------------
// Return a consistency checkable list

ObjectListTemplate<ConsistencyCheckable>* FailureDetectionFactory::
								getConsistencyCheckableList(int mode)
{
	if (mode >= N_FD_MODES)
	{	AocsObject::getConfigurationEventRepository()->
			create(NULL,ILLEGAL_FACTORY_PAR,NULL);
		return NULL;
	}
	else
		return ccListArray[mode];
}

// -------------------------------------------------------------------
// Return a monitoring check list

ObjectListTemplate<MonitoringCheck>* FailureDetectionFactory::
									getMonitoringCheckList(int mode)
{
	if (mode >= N_FD_MODES)
	{	AocsObject::getConfigurationEventRepository()->
			create(NULL,ILLEGAL_FACTORY_PAR,NULL);
		return NULL;
	}
	else
		return mcListArray[mode];
}

// -------------------------------------------------------------------
// Return a monitoring check object

MonitoringCheck* FailureDetectionFactory::makeMonitoringCheck(
		Property prop, ChangeObject* chngObj,RecoveryAction* recAct)
{
	return new MonitoringCheck(prop,chngObj,recAct);
}
