// FailureRecoveryFactory.cpp 
//
// Version	1.0
// Date		27.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory for failure recovery framelet
// -------------------------------------------------------------------

// Include framework header files
#include "../FailureRecovery/FailureRecoveryManager.h"
#include "../FailureRecovery/FailureRecoveryModeManager.h"
#include "../FailureRecovery/FollowerFailureRecoveryModeManager.h"
#include "../FailureRecovery/RecoveryStrategy.h"
#include "../FailureRecovery/RecoveryAction.h"
#include "../FailureRecovery/LocalRecoveryActions.h"
#include "../FailureRecovery/ModeChange.h"
#include "../FailureRecovery/ObjectReset.h"
#include "../FailureRecovery/SystemReset.h"
#include "../FailureRecovery/Reconfiguration.h"
#include "../FailureRecovery/SystemResetOnTooManyFailures.h"
#include "../FailureRecovery/SystemResetOnConfigurationError.h"
#include "../FailureRecovery/NullRecoveryAction.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// Include the factory declaration files
#include "../AocsFactory/FailureRecoveryFactory.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// -------------------------------------------------------------------

FailureRecoveryFactory::FailureRecoveryFactory()
{
	// Initialize all internal variables
	recoveryManager		= NULL;
	recoveryModeManager	= NULL;
	nullRecoveryAction	= NULL;	
	systemReset			= NULL;
	cspRecoveryStrategy	= NULL;
	fspRecoveryStrategy	= NULL;

	// Construct the failure recovery strategies
	makeCspRecoveryStrategy();
	makeFspRecoveryStrategy();
}

// -------------------------------------------------------------------
// Construct and configure the failure recovery manager

void FailureRecoveryFactory::makeFailureRecoveryManager()
{
	if (recoveryManager == NULL)		// only do once!
	{	recoveryManager = new FailureRecoveryManager();
		if (recoveryModeManager == NULL)
			AocsObject::getConfigurationEventRepository()->
				create(NULL,MISSING_FR_MM,NULL);
		else
			recoveryManager->setFailureRecoveryModeManager(
				recoveryModeManager);
	}
}

// -------------------------------------------------------------------
// Creates the mode manager, load the failure recovery strategies into
// it and plugs it into the recovery manager

void FailureRecoveryFactory::makeFailureRecoveryModeManager(
				ModeManager* masterModeManager,ModeChangeAction* mca)
{
	if (recoveryModeManager == NULL)	// only do once!
	{	FollowerFailureRecoveryModeManager* rmm= 
			new FollowerFailureRecoveryModeManager(
					N_FR_MODES,MAX_FR_MONITORS,masterModeManager);

		rmm->loadRecoveryStrategy(0,cspRecoveryStrategy);
		rmm->loadRecoveryStrategy(1,fspRecoveryStrategy);

		rmm->setModeChangeAction(mca);

		recoveryModeManager = rmm;
	}
}

// -------------------------------------------------------------------
// The CSP recovery strategy consists of three recovery strategies linked
// together. 
// The first recovery strategy triggers a software reset if a 
// configuration error has been detected. 
// The second recovery strategy triggers a software reset if more 
// than MAX_FAIL_EVT failure events have been generated since the last
// activation of the recovery manager.
// The third recovery strategy executes the recovery action associated 
// to each failure event generated since the last activation of the 
// recovery manager. 

void FailureRecoveryFactory::makeCspRecoveryStrategy()
{
	if (cspRecoveryStrategy == NULL)			// only do once!
	{	
		cspRecoveryStrategy = new SystemResetOnConfigurationError();
		SystemResetOnTooManyFailures* s2 = new SystemResetOnTooManyFailures();
		LocalRecoveryActions* s3 = new LocalRecoveryActions();
		
		s2->setFailureEventThreshold(MAX_FAIL_EVT);

		cspRecoveryStrategy->setNextRecoveryStrategy(s2);
		s2->setNextRecoveryStrategy(s3);
	}
}

// -------------------------------------------------------------------
// The FSP recovery strategy is the same as the CSP recovery strategy

void FailureRecoveryFactory::makeFspRecoveryStrategy()
{
	if (fspRecoveryStrategy == NULL)			// only do once!
	{	if (cspRecoveryStrategy == NULL)
			makeCspRecoveryStrategy();
		fspRecoveryStrategy = cspRecoveryStrategy;
	}
}

// -------------------------------------------------------------------

RecoveryAction* FailureRecoveryFactory::makeNullRecoveryAction()
{
	if (nullRecoveryAction == NULL)			// construct only one copy!			
		return (new NullRecoveryAction());
	else
		return nullRecoveryAction;
}

// -------------------------------------------------------------------

RecoveryAction* FailureRecoveryFactory::makeSystemResetRecoveryAction()
{
	if (systemReset == NULL)		// construct only one copy!			
		return (new SystemReset());
	else
		return systemReset;
}

// -------------------------------------------------------------------

RecoveryAction* FailureRecoveryFactory::
			makeReconfigurationRecoveryAction(Reconfigurable* r)
{
	return (new Reconfiguration(r));
}

// -------------------------------------------------------------------

RecoveryAction* FailureRecoveryFactory::
			makeModeChangeRecoveryAction(ModeManager* m, int newMode)
{
	return (new ModeChange(m,newMode));
}






