// FailureRecoveryFactory.h
//
// Version	1.0
// Date		27.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for the Recovery Management Framelet.
//
// This class should be customized (by inheritance) for each AOCS
// application.
//
// The factory produces the following items:
//	- the failure recovery manager
//	- the failure recovery strategies
//	- the failure recovery actions
//
// Recovery actions that have no internal state are created only once 
// and all its users receive references to the same object.
//
// Two recovery strategies are manufactured corresponding to the two
// operational modes - CSP and FSP - of the prototype AOCS.
// -------------------------------------------------------------------

#ifndef FailureRecoveryFactoryH
#define FailureRecoveryFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"

// -------------------------------------------------------------------

class FailureRecoveryFactory {

	FailureRecoveryManager* recoveryManager;
	FailureRecoveryModeManager* recoveryModeManager;

	// Recovery actions
	RecoveryAction* nullRecoveryAction;
	RecoveryAction* systemReset;

	// Recovery strategies
	RecoveryStrategy* cspRecoveryStrategy;
	RecoveryStrategy* fspRecoveryStrategy;

protected :

	void makeCspRecoveryStrategy();
	void makeFspRecoveryStrategy();

public:

	FailureRecoveryFactory();	

	// Factory method for the failure recovery mode manager
	void makeFailureRecoveryModeManager(
				ModeManager* masterModeManager, ModeChangeAction* mca);

	// Make and configure the failure recovery manager
	void makeFailureRecoveryManager();

	// Factory methods for the recovery actions
	RecoveryAction* makeNullRecoveryAction();
	RecoveryAction* makeSystemResetRecoveryAction();
	RecoveryAction* makeReconfigurationRecoveryAction(Reconfigurable* r);
	RecoveryAction* makeModeChangeRecoveryAction(ModeManager* m, int newMode);


	// Retrieve the recovery strategies
	RecoveryStrategy* getCspRecoveryStrategy()
		{ return cspRecoveryStrategy; }
	RecoveryStrategy* getFspRecoveryStrategy()
		{ return fspRecoveryStrategy; }

	// Retrieve the failure recovery manager and its mode manager
	FailureRecoveryManager* getFailureRecoveryManager()
		{ return recoveryManager; }

	FailureRecoveryModeManager* getFailureRecoveryModeManager()
		{ return recoveryModeManager; }

};

#endif
