// initializeAocs.h 
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Routine to initialize the AOCS software 
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsData/AocsData.h"
#include "../AocsData/DataItemRead.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsData/AttitudeDataPool.h"
#include "../SystemManagement/SystemManager.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/ReconfigurationEventRepository.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/TelecommandEventRepository.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/SystemEventRepository.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../ControllerManagement/Controller.h"
#include "../TelecommandManagement/TelecommandLoader.h"
#include "../TelecommandManagement/TelecommandManager.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsConfiguration.h"
#include "../AocsFactory/SystemFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/TelecommandFactory.h"
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/FailureRecoveryFactory.h"
#include "../AocsFactory/FailureDetectionFactory.h"
#include "../AocsFactory/MonitoringFactory.h"
#include "../AocsFactory/ModeFactory.h"
#include "../AocsFactory/ManoeuvreFactory.h"
#include "../AocsFactory/ControllerFactory.h"

// Include test header files
#include "../GeneralUtilities/TestUtilities.h"
#include "../AocsPrototypeTest/AocsPrototypeTest.h"
#include "../AocsPrototypeTest/AocsTestHarness.h"

// -------------------------------------------------------------------

void instantiateAocs(FILE* out, FrameworkFactories fwFact)
{
	SystemFactory* systemFactory = fwFact.systemFactory;

	makeSystemComponents(fwFact);
		
	makeTelemetryComponents(fwFact);

	makeControllerComponents(fwFact);
	
	makeFailureRecoveryComponents(fwFact);

	makeFailureDetectionComponents(fwFact);

	makeUnits(fwFact);

	makeUnitTriggers(fwFact);

	makeManoeuvreComponents(fwFact);

	loadTelemetryLists(fwFact);

	loadFailureDetectionLists(fwFact);

	loadUnitTriggerLists(fwFact);

	loadControllerLists(fwFact);

	makeTelecommandComponents(fwFact);

	// Check that all AOCS objects are configured
	if (!systemFactory->getSystemManager()->isSystemConfigured())
		printTestOutcome(out,"initializeAocs",false);

	// Check that no configuration errors have been generated
	if (systemFactory->getConfigurationEventRepository()->getCounter() > 0)
		printTestOutcome(out,"initializeAocs",false);
}

// -------------------------------------------------------------------
// Construct the system components (but note that the system manager
// and the event repositories are created in the constructor of the
// SystemManager because they are needed by the constructors of other
// factories).

void makeSystemComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	ModeFactory* modeFactory = fwFact.modeFactory;
	SystemFactory* systemFactory = fwFact.systemFactory;
	FailureRecoveryFactory* frFactory = fwFact.failureRecoveryFactory;

	// Plug recovery action associated to calls of AocsObject destructor
	AocsObject::setDestructionRecoveryAction(
								frFactory->makeNullRecoveryAction());

	// Create the AOCS clock and plug it into AocsObject and
	// DataItemWrite
	systemFactory->makeAocsClock();
	
	// Create the mission mode manager
	systemFactory->makeMissionModeManager(
							modeFactory->makeNullModeChangeAction());

	// Make the attitude data pool
	systemFactory->makeDataPools(frFactory->makeNullRecoveryAction());
}

// -------------------------------------------------------------------
// Construct the telemetry components.

void makeTelemetryComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	TelemetryFactory* tmFactory = fwFact.telemetryFactory;
	ModeFactory* modeFactory = fwFact.modeFactory;

	// Make the telemetry stream. By passing a 0 as the second 
	// parameter the factory method is instructed to create a 
	// TM DMA buffer internally (simulated DMA buffer)
	tmFactory->makeTelemetryStream(TM_BUFFER_SIZE, (char*)0);

	// Construct telemetry mode manager
	tmFactory->makeTelemetryModeManager(
							modeFactory->makeNullModeChangeAction());

	// Construct telemetry manager
	tmFactory->makeTelemetryManager();

	return;
}

// -------------------------------------------------------------------
// Construct the controller manager and its mode managers

void makeControllerComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	ModeFactory* modeFactory = fwFact.modeFactory;
	ControllerFactory* controllerFactory = fwFact.controllerFactory;

	// Create the controller mode manager
	controllerFactory->makeControllerModeManager(
						systemFactory->getMissionModeManager(),
							modeFactory->makeNullModeChangeAction());

	// Create the controller manager
	controllerFactory->makeControllerManager();
}

// -------------------------------------------------------------------
// Construct the failure recovery manager and its mode managers

void makeFailureRecoveryComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	ModeFactory* modeFactory = fwFact.modeFactory;
	FailureRecoveryFactory* failureRecoveryFactory = 
										fwFact.failureRecoveryFactory;

	// Create the failure recovery mode manager
	failureRecoveryFactory->makeFailureRecoveryModeManager(
					systemFactory->getMissionModeManager(),
						modeFactory->makeNullModeChangeAction());

	// Create the failure recovery manager
	failureRecoveryFactory->makeFailureRecoveryManager();
}

// -------------------------------------------------------------------
// Construct the failure detection manager and its mode managers

void makeFailureDetectionComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	ModeFactory* modeFactory = fwFact.modeFactory;
	FailureDetectionFactory* failureDetectionFactory = 
										fwFact.failureDetectionFactory;

	// Create the failure detection mode manager
	failureDetectionFactory->makeFailureDetectionModeManager(
						systemFactory->getMissionModeManager(),
							modeFactory->makeNullModeChangeAction());

	// Create the failure detection manager
	failureDetectionFactory->makeFailureDetectionManager(
							systemFactory->getChangeEventRepository());
}

// -------------------------------------------------------------------
// Construct the manoeuvre manager and the concrete manoeuvres

void makeManoeuvreComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	ManoeuvreFactory* manoeuvreFactory = fwFact.manoeuvreFactory;

	// Create the manoeuvre manager
	manoeuvreFactory->makeManoeuvreManager();

	// Create the concrete manoeuvre components
	manoeuvreFactory->makeConcreteManoeuvres();
}

// -------------------------------------------------------------------
// Construct the AOCS units and their reconfiguration managers

void makeUnits(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	FailureRecoveryFactory* failureRecoveryFactory = 
										fwFact.failureRecoveryFactory;

	// Make the interface to the external units. By passing NULL
	// parameters, the factory method is informed that there is no
	// external MACS hardware and that the MACS registers should be
	// created internally for purposes of simulation
	aocsUnitFactory->makeAocsUnitHardware(NULL,NULL,NULL);
	
	// Construct the FSS units and their reconfiguration manager
	AocsData* fss = systemFactory->getAttitudeDataPool()->getFssData();
	RecoveryAction* raFssA = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raFssB = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raFss = failureRecoveryFactory->makeNullRecoveryAction();
	aocsUnitFactory->makeFss(fss,raFssA,raFssB,raFss);

	// Construct the GYR units and their reconfiguration manager
	AocsData* gyr = systemFactory->getAttitudeDataPool()->getGyrRate();
	RecoveryAction* raGyrA = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raGyrB = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raGyr = failureRecoveryFactory->makeNullRecoveryAction();
	aocsUnitFactory->makeGyr(gyr,raGyrA,raGyrB,raGyr);

	// Construct the SAP units and their reconfiguration manager
	AocsData* sas1 = systemFactory->getAttitudeDataPool()->getSas1Data();
	AocsData* sas2 = systemFactory->getAttitudeDataPool()->getSas2Data();
	AocsData* sas3 = systemFactory->getAttitudeDataPool()->getSas3Data();
	AocsData* thuDelayTime = systemFactory->getAttitudeDataPool()->getThuDelayTime();
	AocsData* thuOnTime = systemFactory->getAttitudeDataPool()->getThuOnTime();
	RecoveryAction* raSapA = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raSapB = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* raSap = failureRecoveryFactory->makeNullRecoveryAction();
	aocsUnitFactory->makeSap(sas1,sas2,sas3,thuOnTime,thuDelayTime,raSapA,raSapB,raSap);

	// Construct the reaction wheel units
	AocsData* rwSpd = systemFactory->getAttitudeDataPool()->getRwSpeed();	
	AocsData* rwTor = systemFactory->getAttitudeDataPool()->getRwTorque();
	for (int i=0; i<4; i++)
		aocsUnitFactory->makeRw(i,rwSpd->getDataItemWrite(i),
									rwTor->getDataItemWrite(i),
										failureRecoveryFactory->makeNullRecoveryAction());

	// Construct the reaction wheel set (RW reconfiguration manager)
	AocsData* scTor = systemFactory->getAttitudeDataPool()->getScTorque();
	AocsData* rwAngMom = systemFactory->getAttitudeDataPool()->getRwAngularMomentum();
	RecoveryAction* raRwSet = failureRecoveryFactory->makeNullRecoveryAction();
	aocsUnitFactory->makeRwSet(rwAngMom,scTor,raRwSet);

	// Construct the torquing thruster fictitious unit
	aocsUnitFactory->makeThu(scTor);
}

// -------------------------------------------------------------------
// Construct the triggers lists, the unit trigger mode managers and
// the unit trigger objects

void makeUnitTriggers(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	ModeFactory* modeFactory = fwFact.modeFactory;
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	FailureRecoveryFactory* failureRecoveryFactory = 
										fwFact.failureRecoveryFactory;

	aocsUnitFactory->makeUnitTriggerLists();

	aocsUnitFactory->makeUnitTriggerModeManagers(
						systemFactory->getMissionModeManager(),
							modeFactory->makeNullModeChangeAction());

	aocsUnitFactory->makeUnitTriggers();
}

// -------------------------------------------------------------------
// Load all the telemetry lists. Two telemetry lists are defined, 
// one for CSP mode and one for FSP mode.
// The TM list for CSP mode contains the following:
//	- same content as TM list 2
//	- mode event repository
//	- system event repository
//	- reconfiguration event repository
//	- telecommand event repository
//	- recovery event repository
//	- manoeuvre event repository
// The TM list for FSP contains the following:
//	- failure event repository
//	- configuration event repository
//	- data pool

void loadTelemetryLists(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	TelemetryFactory* telemetryFactory = fwFact.telemetryFactory;

	// Load the first TM list
	ObjectListTemplate<Telemeterable>* tmList1=telemetryFactory->getTelemetryList(CSP);
	tmList1->addObject(systemFactory->getFailureEventRepository());
	tmList1->addObject(systemFactory->getConfigurationEventRepository());
	tmList1->addObject(systemFactory->getAttitudeDataPool());
	tmList1->addObject(systemFactory->getModeEventRepository());
	tmList1->addObject(systemFactory->getSystemEventRepository());
	tmList1->addObject(systemFactory->getReconfigurationEventRepository());
	tmList1->addObject(systemFactory->getTelecommandEventRepository());
	tmList1->addObject(systemFactory->getRecoveryEventRepository());
	tmList1->addObject(systemFactory->getManoeuvreEventRepository());

	// Load the second TM list
	ObjectListTemplate<Telemeterable>* tmList2=telemetryFactory->getTelemetryList(FSP);
	tmList2->addObject(systemFactory->getFailureEventRepository());
	tmList2->addObject(systemFactory->getConfigurationEventRepository());
	tmList2->addObject(systemFactory->getAttitudeDataPool());
}

// -------------------------------------------------------------------
// Load all the controller lists. Two controller lists are defined, 
// one for CSP mode and one for FSP mode.
// Each list contains three controllers: one for each spacecraft axis.
// All controllers are 'minimal' controllers with a zero set-point,
// no measurement filtering, no feedforward compensation. The 
// compensator is always a PID.
// The X- and Y-axis controller take as inputs the SAS measurements in
// CSP and the FSS measurements in FSP. The Z-axis controller takes
// as input the gyro measurement. The controller output is always a
// torque request.

void loadControllerLists(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	FailureRecoveryFactory* frFactory = fwFact.failureRecoveryFactory;
	ControllerFactory* controllerFactory = fwFact.controllerFactory;

	// Construct the CSP X-axis controller
	Controller* cspX = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getSas1Data()->getDataItemRead(0),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(0),
		controllerFactory->makePDController(CSP_XY_KP,CSP_XY_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Construct the CSP Y-axis controller
	Controller* cspY = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getSas1Data()->getDataItemRead(1),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(1),
		controllerFactory->makePDController(CSP_XY_KP,CSP_XY_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Construct the CSP Z-axis controller
	Controller* cspZ = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getGyrRate()->getDataItemRead(0),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(2),
		controllerFactory->makePDController(CSP_Z_KP,CSP_Z_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Construct the FSP X-axis controller
	Controller* fspX = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getFssData()->getDataItemRead(0),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(0),
		controllerFactory->makePDController(FSP_XY_KP,FSP_XY_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Construct the FSP Y-axis controller
	Controller* fspY = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getFssData()->getDataItemRead(1),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(1),
		controllerFactory->makePDController(FSP_XY_KP,FSP_XY_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Construct the FSP Z-axis controller
	Controller* fspZ = controllerFactory->makeController(
		systemFactory->getAttitudeDataPool()->getGyrRate()->getDataItemRead(0),
		systemFactory->getAttitudeDataPool()->getScTorque()->getDataItemWrite(2),
		controllerFactory->makePDController(FSP_Z_KP,FSP_Z_KD,DER_MAX),
		frFactory->makeNullRecoveryAction());

	// Load the CSP controller list
	ObjectListTemplate<Controller>* cspList = controllerFactory->getControllerList(CSP);
	cspList->addObject(cspX);
	cspList->addObject(cspY);
	cspList->addObject(cspZ);

	// Load the FSP controller list
	ObjectListTemplate<Controller>* fspList = controllerFactory->getControllerList(FSP);
	fspList->addObject(fspX);
	fspList->addObject(fspY);
	fspList->addObject(fspZ);
}
	
// -------------------------------------------------------------------
// Load the failure detection lists.
// The following FD lists with the following contents and associated
// recovery actions are defined:
//	ConsistencyCheckable list to be used for CSP:
//		- attitude data pool	-->	software reset
//	ConsistencyCheckable list to be used for FSP:
//		- attitude data pool	-->	software reset
//	MonitoringCheck list to be used for CSP:
//		- delta change check on FSS X output --> FSS reconfiguration
//		- delta change check on FSS Y output --> FSS reconfiguration
//		- out-of-range check on FSS X output --> FSS reconfiguration
//		- out-of-range check on FSS Y output --> FSS reconfiguration
//		- delta change check on GYR rate output --> GYR reconfiguration
//		- out-of-range check on GYR rate output --> GYR reconfiguration
//	MonitoringCheck list to be used for FSP
//		- same content as MonitoringCheck list for FSP
//		- out-of-range check on X-axis attitude error --> fallback to CSP
//		- out-of-range check on Y-axis attitude error --> fallback to CSP

void loadFailureDetectionLists(FrameworkFactories fwFact)
{
	// Recover fw factories
	SystemFactory* systemFactory = fwFact.systemFactory;
	FailureDetectionFactory* fdFactory = fwFact.failureDetectionFactory;
	FailureRecoveryFactory* frFactory = fwFact.failureRecoveryFactory;
	MonitoringFactory* monFactory = fwFact.monitoringFactory;
	AocsUnitFactory* unitFactory = fwFact.aocsUnitFactory;

	// Declare local variables (to make code more readable)
	ConsistencyCheckable* cc;
	MonitoringCheck* mc;
	ChangeObject* co;

	// Load the ConsistencyCheckable lists
	ObjectListTemplate<ConsistencyCheckable>* ccListCsp = 
							fdFactory->getConsistencyCheckableList(CSP);
	ObjectListTemplate<ConsistencyCheckable>* ccListFsp = 
							fdFactory->getConsistencyCheckableList(FSP);

	// Load the attitude data pool and associate recovery action to it
	cc = systemFactory->getAttitudeDataPool();
	ccListCsp->addObject(cc);
	ccListFsp->addObject(cc);
	systemFactory->getAttitudeDataPool()->setRecoveryAction(
						frFactory->makeSystemResetRecoveryAction());

	// Load the monitoring check lists 
	ObjectListTemplate<MonitoringCheck>* mcListCsp = 
							fdFactory->getMonitoringCheckList(CSP);
	ObjectListTemplate<MonitoringCheck>* mcListFsp = 
							fdFactory->getMonitoringCheckList(FSP);

	// Recover the datum representing the FSS read-out and the FSS
	// reconfiguration manager and construct a recovery action to
	// reconfigure the FSS
	AocsData* fss = systemFactory->getAttitudeDataPool()->getFssData();
	Reconfigurable* fssReconfigurer = unitFactory->getFssReconfigurer();
	RecoveryAction* reconfigFss = frFactory->
					makeReconfigurationRecoveryAction(fssReconfigurer);

	// Load the delta change check on the FSS X
	co = monFactory->makeDeltaChange(MAX_DELTA_FSS);
	mc = fdFactory->makeMonitoringCheck(fss->getProperty(0),co,reconfigFss);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Load the delta change check on the FSS Y
	co = monFactory->makeDeltaChange(MAX_DELTA_FSS);
	mc = fdFactory->makeMonitoringCheck(fss->getProperty(1),co,reconfigFss);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Load the out-of-range check on the FSS X
	co = monFactory->makeOutOfRangeChange(LOWER_RANGE_FSS,UPPER_RANGE_FSS);
	mc = fdFactory->makeMonitoringCheck(fss->getProperty(0),co,reconfigFss);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Load the out-of-range check on the FSS Y
	co = monFactory->makeOutOfRangeChange(LOWER_RANGE_FSS,UPPER_RANGE_FSS);
	mc = fdFactory->makeMonitoringCheck(fss->getProperty(1),co,reconfigFss);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Recover the datum representing the GYR read-out and the GYR
	// reconfiguration manager and construct a recovery action to
	// reconfigure the GYR
	AocsData* gyr = systemFactory->getAttitudeDataPool()->getGyrRate();
	Reconfigurable* gyrReconfigurer = unitFactory->getGyrReconfigurer();
	RecoveryAction* reconfigGyr = frFactory->
					makeReconfigurationRecoveryAction(gyrReconfigurer);


	// Load the delta change check on the GYR rate output
	co = monFactory->makeDeltaChange(MAX_DELTA_GYR);
	mc = fdFactory->makeMonitoringCheck(gyr->getProperty(0),co,reconfigGyr);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Load the out-of-range check on the GYR rate output
	co = monFactory->makeOutOfRangeChange(LOWER_RANGE_GYR,UPPER_RANGE_GYR);
	mc = fdFactory->makeMonitoringCheck(gyr->getProperty(0),co,reconfigGyr);
	mcListCsp->addObject(mc);
	mcListFsp->addObject(mc);

	// Load the out-of-range check on the attitude error (FSP only)
	AocsData* attErr = systemFactory->getAttitudeDataPool()->getAttError();
	RecoveryAction* fallback = frFactory->
		makeModeChangeRecoveryAction(systemFactory->getMissionModeManager(),CSP);
	co = monFactory->makeOutOfRangeChange(0,MAX_ATT_ERR_FSP);
	mc = fdFactory->makeMonitoringCheck(attErr->getProperty(0),co,fallback);
	mcListFsp->addObject(mc);
}

// -------------------------------------------------------------------
// Load the unit trigger lists .
//
// Three trigger lists are supplied by the AocsUnitFactory: 
//	- triggerListAcquisition: units from which data have to be acquired.
//			This is the same for both CSP and FSP mode. Synchronization
//			messages are only sent to the FSS.
//	- cspTriggerListSend: units to which data have to be send in CSP mode
//	- fspTriggerListSend: units to which data have to be send in FSP mode

void loadUnitTriggerLists(FrameworkFactories fwFact)
{
	// Recover fw factories
	AocsUnitFactory* unitFactory = fwFact.aocsUnitFactory;

	// Load the list of units to be triggered at the beginning of 
	// a cycle (sensor acquisition)
	unitFactory->getTriggerListAcquisition()->
		addFunctionalSynchronize(unitFactory->getFss());

	unitFactory->getTriggerListAcquisition()->
		addFunctionalAcquire(unitFactory->getFss());
	unitFactory->getTriggerListAcquisition()->
		addFunctionalAcquire(unitFactory->getGyr());
	unitFactory->getTriggerListAcquisition()->
		addFunctionalAcquire(unitFactory->getSap());
	unitFactory->getTriggerListAcquisition()->
		addFunctionalAcquire(unitFactory->getRwSet());

	// Load the list of units to be triggered at the end of the
	// cycle in CSP mode
	unitFactory->getCspTriggerListSend()->
		addFunctionalSend(unitFactory->getThu());

	// Load the list of units to be triggered at the end of the
	// cycle in FSP mode
	unitFactory->getFspTriggerListSend()->
		addFunctionalSend(unitFactory->getRwSet());
}

// -------------------------------------------------------------------
// Make the TC manager and the TC loader

void makeTelecommandComponents(FrameworkFactories fwFact)
{
	// Recover fw factories
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	FailureRecoveryFactory* failureRecoveryFactory = 
										fwFact.failureRecoveryFactory;

	// Construct recovery actions
	RecoveryAction* tcListFull = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* tcStackError = failureRecoveryFactory->makeNullRecoveryAction();
	RecoveryAction* tcLoaderError = failureRecoveryFactory->makeNullRecoveryAction();

	// Make TC manager 
	tcFactory->makeTelecommandManager(tcListFull,tcStackError);

	// Make TC loader. By passing a NULL argument, the factory method
	// is informed that there is no real TC interface and that the
	// TC buffer has to be generated internally
	tcFactory->makeTelecommandLoader(NULL,tcLoaderError);

	// TC manager and TC loader must be plugged into each other
	tcFactory->getTelecommandLoader()->setTelecommandManager(
		tcFactory->getTelecommandManager());

	tcFactory->getTelecommandManager()->setTelecommandLoader(
		tcFactory->getTelecommandLoader());

	// Clear telecommand buffer
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();
	tcBuffer[0]=0;		// number of bytes in TC buffer
}