// TelemetryFactory.cpp 
//
// Version	1.0
// Date		15.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Telemetry Framelet Factory Class 
// -------------------------------------------------------------------

// Include framework header files
#include "../TelemetryManagement/TelemetryManager.h"
#include "../TelemetryManagement/TelemetryModeManager.h"
#include "../TelemetryManagement/CyclingTelemetryModeManager.h"
#include "../TelemetryManagement/DmaTelemetryStream.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// Include the AOCS factory files
#include "../AocsFactory/TelemetryFactory.h"

// -------------------------------------------------------------------

TelemetryFactory::TelemetryFactory()
{
	tmManager = NULL;
	tmModeManager = NULL;
	tmStream = NULL;
	tmListArray = NULL;
	tmDmaBufferStart = NULL;
	tmBufferSize = 0;

	makeTelemetryLists();
}
	
// -------------------------------------------------------------------
// Create and configure the telemetry lists

void TelemetryFactory::makeTelemetryLists()
{
	if (tmListArray==NULL)					// do only once
	{	tmListArray = new ObjectListTemplate<Telemeterable>*[N_TM_MODES];
		for (int i=0; i<N_TM_MODES; i++)
		{	ObjectListTemplate<Telemeterable>* tmList = 
					new ObjectListTemplate<Telemeterable>();
			tmList->allocateMemory(TM_LIST_SIZE);
			tmListArray[i] = tmList;
		}
	}
}

// -------------------------------------------------------------------
// Create and configure the telemetry mode manager

void TelemetryFactory::makeTelemetryModeManager(ModeChangeAction* mca)
{
	if (tmModeManager == NULL)					// create only once
	{	CyclingTelemetryModeManager* tmmm = 
			new CyclingTelemetryModeManager(N_TM_MODES,CSP,MAX_TM_MONITORS);
		
		// Plug in telemetry lists
		for (int i=0; i<N_TM_MODES; i++)
			tmmm->loadTelemetryList(i,tmListArray[i]);

		// Plug in the mode change action
		tmmm->setModeChangeAction(mca);

		// Assign internal variable
		tmModeManager = tmmm;
	}
}

// -------------------------------------------------------------------
// Create and configure the telemetry stream. A DMA TM stream is 
// assumed. If a real DMA device is present, the start address of the
// DMA buffer should be passed as a second parameter. If there is no
// real DMA device, the second parameter should be passed as 0 and
// the factory method will create an internal (simulated) DMA buffer.
// The TM buffer size is expressed in bytes.

void TelemetryFactory::makeTelemetryStream(int tmSize, char* tmDmaStart)
{
	if (tmStream == NULL)					// create only once
	{	tmStream = new DmaTelemetryStream();

		tmBufferSize = tmSize;

		if (tmDmaStart!=NULL)	// real DMA device: this is its address
			tmDmaBufferStart = tmDmaStart;
		else					// simulated DMA device
			tmDmaBufferStart = new char[tmBufferSize];

		tmStream->setBufferSize(tmBufferSize);
		tmStream->setDmaBufferStartAddress((char*)tmDmaBufferStart);
	}
}

// -------------------------------------------------------------------
// Create and configure the telemetry manager (should only be called
// after the mode manager has been constructed).

void TelemetryFactory::makeTelemetryManager()
{
	if (tmManager == NULL)					// create only once
	{	tmManager = new TelemetryManager();
		if (tmModeManager == NULL)
			AocsObject::getConfigurationEventRepository()->
				create(NULL,MISSING_TM_MM,NULL);
		else
			tmManager->setTelemetryModeManager(tmModeManager);

		tmManager->setTelemetryStream(tmStream);
	}
}

// -------------------------------------------------------------------
// Return the telemetry lists

ObjectListTemplate<Telemeterable>* TelemetryFactory::getTelemetryList(int mode)
{
	if (mode >= N_TM_MODES)
	{	AocsObject::getConfigurationEventRepository()->
			create(NULL,ILLEGAL_FACTORY_PAR,NULL);
		return NULL;
	}
	else
		return tmListArray[mode];
}

