// AocsTestHarness.h
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Base class for test harnesses for testing AOCS prototype sw.
// A test harness simulates the external environment of the AOCS sw
// and buffers up its outputs to make them available for inspection
// at the end of a test run. 
// The test harness is activated at every AOCS cycle when it performs
// the following actions:
//	- send telecommands to the AOCS sw
//	- load MACS hw input buffers with data simulating sensor read-outs
//	- generate a new set of actuator commands
//	- save current telemetry frame
//	- save current unit data source buffers
//
// This is an abstract class. Derived classes define specific test 
// runs. Each derived class defines a particular set and sequence of
// telecommands, of simulated sensor outputs and actuator inputs.
// -------------------------------------------------------------------

#ifndef AocsTestHarnessH
#define AocsTestHarnessH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../AocsPrototypeTest/AocsPrototypeTest.h"
#include "../AocsFactory/AocsConfiguration.h"
#include <stdio.h>  

// -------------------------------------------------------------------

class AocsTestHarness : public Runnable {

	// Output device (for error messages)
	FILE* out;

	// Buffer to hold the TM_SAVE most recent tm frames
	char* tmBuffer;

	// Buffer to hold the MACS hw buffers with outgoing data from
	// the last MACS_SAVE cycles
	unsigned short* macsBuffer;

	// Counters
	int tmCounter;
	int macsCounter;

protected:

	// Structure holding the fw factories
	FrameworkFactories fwFact;

	// Send a telecommand to the AOCS 
	virtual void loadTelecommands()=0;

	// Load the MACS hw buffer with simulated incoming data
	virtual void loadMacs()=0;

	// Generate a set of actuator commands
	virtual void generateActuatorCommands()=0;

	// Save the current tm frame
	void saveTelemetry();

	// Save the current MACS data source buffers
	void saveMacs();

public:

	AocsTestHarness(FrameworkFactories fwFact, FILE* out);

	// Inherited from interface Runnable
	virtual void run(AocsTime deadline);

	// Getter methods
	int getTmBufferSize()
		{ return TM_BUFFER_SIZE; }

	int getNumberOfTmBuffers()
		{ return TM_SAVE; }

	char* getTmBuffer()
		{ return tmBuffer; }

	int getTmCounter()
		{ return tmCounter; }

	int getNumberOfMacsWords()
		{ return N_MACS_OUT_HW_BUF; }

	unsigned short* getMacsBuffer()
		{ return macsBuffer; }

	int getNumberOfMacsBuffers()
		{ return MACS_SAVE; }

	int getTmMacsCounter()
		{ return macsCounter; }
};

// -------------------------------------------------------------------
#endif