// AocsTestHarness_2.cpp
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/CompilerSwitches.h"
#include "../TelemetryManagement/DmaTelemetryStream.h"
#include "../AocsUnit/AocsUnit.h"
#include "../AocsData/AttitudeDataPool.h"
#include "../TelecommandManagement/ModeChangeTransactionTelecommand.h"
#include "../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../TelecommandManagement/ReconfigureTelecommand.h"
#include "../TelecommandManagement/AttitudeSlewTelecommand.h"
#include "../OperatingSystemObjects/AocsClock.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/AocsConfiguration.h"

// Include test header files
#include "../AocsPrototypeTest/AocsTestHarness_2.h"
#include "../GeneralUtilities/TestUtilities.h"

// -------------------------------------------------------------------

AocsTestHarness_2::AocsTestHarness_2(FrameworkFactories f, FILE* out):
					AocsTestHarness(f,out)
{
	w_x = 0.0002;
	tcCycleCounter = 0;
	macsCycleCounter = 0;
}

// -------------------------------------------------------------------
// Load the hardware buffers for incoming MACS data.
// Unit data are computed to ensure that, after correction by the 
// AOCS software for bias and scaling, they give error-free 
// attitude measurements.

void AocsTestHarness_2::loadMacs()
{
	// Recover the AOCS units that send data to the AOCS computer
	AocsUnitFactory* unitFactory = fwFact.aocsUnitFactory;

	AocsUnit* fssA = unitFactory->getFssA();
	AocsUnit* fssB = unitFactory->getFssB();

	AocsUnit* gyrA = unitFactory->getGyrA();
	AocsUnit* gyrB = unitFactory->getGyrB();

	AocsUnit* sapA = unitFactory->getSapA();
	AocsUnit* sapB = unitFactory->getSapB();

	AocsUnit* rw[4];
	for (int i=0; i<4; i++)
		rw[i] = unitFactory->getRw(i);

	// Compute angles by which s/c has rotated around three axes
	Real theta_x = w_x*macsCycleCounter;
	Real theta_y = 0;
	Real theta_z = 0;

	// Declara vairables useful below
	short v;
	unsigned short fss_x_a;
	unsigned short fss_x_b;
	unsigned short fss_y_a;
	unsigned short fss_y_b;
	unsigned short sas_1_x_a;
	unsigned short sas_1_x_b;
	unsigned short sas_1_y_a;
	unsigned short sas_1_y_b;
	unsigned short sas_2_x_a;
	unsigned short sas_2_x_b;
	unsigned short sas_2_y_a;
	unsigned short sas_2_y_b;
	unsigned short sas_3_x_a;
	unsigned short sas_3_x_b;
	unsigned short sas_3_y_a;
	unsigned short sas_3_y_b;
	unsigned short gyr_a;
	unsigned short gyr_b;

	// Compute the FSS channel outputs (recall that FSS B has
	// a 10% scaling factor error)
	v = (theta_y+BIAS_FSS_X_A)/(FSS_Q*SCALING_FSS_X_A);
	if (v>0)	fss_x_a = v;
	else 		fss_x_a = 0x8000-v;

	v = (theta_y+BIAS_FSS_X_B)/(FSS_Q*SCALING_FSS_X_B*1.1);
	if (v>0)	fss_x_b = v;
	else 		fss_x_b = 0x8000-v;

	v = (theta_x+BIAS_FSS_Y_A)/(FSS_Q*SCALING_FSS_Y_A);
	if (v>0)	fss_y_a = v;
	else 		fss_y_a = 0x8000-v;

	v = (theta_x+BIAS_FSS_Y_B)/(FSS_Q*SCALING_FSS_Y_B*1.1);
	if (v>0)	fss_y_b = v;
	else 		fss_y_b = 0x8000-v;

	// Compute the SAS channel outputs
	v = (theta_y+BIAS_SAS_1_A_X)/(SAS_Q*SCALING_SAS_1_A_X);
	if (v>0)	sas_1_x_a = v;
	else 		sas_1_x_a = 0x8000-v;

	v = (theta_y+BIAS_SAS_1_B_X)/(SAS_Q*SCALING_SAS_1_B_X);
	if (v>0)	sas_1_x_b = v;
	else 		sas_1_x_b = 0x8000-v;

	v = (theta_x+BIAS_SAS_1_A_Y)/(SAS_Q*SCALING_SAS_1_A_Y);
	if (v>0)	sas_1_y_a = v;
	else 		sas_1_y_a = 0x8000-v;

	v = (theta_x+BIAS_SAS_1_B_Y)/(SAS_Q*SCALING_SAS_1_B_Y);
	if (v>0)	sas_1_y_b = v;
	else 		sas_1_y_b = 0x8000-v;

	v = (BIAS_SAS_2_A_X)/(SAS_Q*SCALING_SAS_2_A_X);
	if (v>0)	sas_2_x_a = v;
	else 		sas_2_x_a = 0x8000-v;

	v = (BIAS_SAS_2_B_X)/(SAS_Q*SCALING_SAS_2_B_X);
	if (v>0)	sas_2_x_b = v;
	else 		sas_2_x_b = 0x8000-v;

	v = (BIAS_SAS_2_A_Y)/(SAS_Q*SCALING_SAS_2_A_Y);
	if (v>0)	sas_2_y_a = v;
	else 		sas_2_y_a = 0x8000-v;

	v = (BIAS_SAS_2_B_Y)/(SAS_Q*SCALING_SAS_2_B_Y);
	if (v>0)	sas_2_y_b = v;
	else 		sas_2_y_b = 0x8000-v;
	
	v = (BIAS_SAS_3_A_X)/(SAS_Q*SCALING_SAS_3_A_X);
	if (v>0)	sas_3_x_a = v;
	else 		sas_3_x_a = 0x8000-v;
	
	v = (BIAS_SAS_3_B_X)/(SAS_Q*SCALING_SAS_3_B_X);
	if (v>0)	sas_3_x_b = v;
	else 		sas_3_x_b = 0x8000-v;

	v = (BIAS_SAS_3_A_Y)/(SAS_Q*SCALING_SAS_3_A_Y);
	if (v>0)	sas_3_y_a = v;
	else 		sas_3_y_a = 0x8000-v;

	v = (BIAS_SAS_3_B_Y)/(SAS_Q*SCALING_SAS_3_B_Y);
	if (v>0)	sas_3_y_b = v;
	else 		sas_3_y_b = 0x8000-v;

	// Compute the GYR channel outputs (zero rate is assumed)

	v = (BIAS_GYR_A)/(GYR_RAT_Q*SCALING_GYR_A);
	if (v>0)	gyr_a = v;
	else 		gyr_a = 0x8000-v;
	
	v = (BIAS_GYR_B)/(GYR_RAT_Q*SCALING_GYR_B);
	if (v>0)	gyr_b = v;
	else 		gyr_b = 0x8000-v;
	
	// Simulate a GYR A failure in the second cycle
	if (macsCycleCounter==2)
		gyr_a = (MAX_DELTA_GYR*2+BIAS_GYR_A)/(GYR_RAT_Q*SCALING_GYR_A); 

	// Load the hardware buffers for incoming FSS, SAP and GYR MACS data
	fssA->setFcHwInpBuff(fss_x_a,0);
	fssA->setFcHwInpBuff(fss_y_a,1);
	fssB->setFcHwInpBuff(fss_x_b,0);
	fssB->setFcHwInpBuff(fss_y_b,1);

	gyrA->setFcHwInpBuff(gyr_a,0);
	gyrB->setFcHwInpBuff(gyr_b,0);

	sapA->setFcHwInpBuff(sas_1_x_a,0);
	sapA->setFcHwInpBuff(sas_1_y_a,1);
	sapA->setFcHwInpBuff(sas_2_x_a,2);
	sapA->setFcHwInpBuff(sas_2_y_a,3);
	sapA->setFcHwInpBuff(sas_3_x_a,4);
	sapA->setFcHwInpBuff(sas_3_y_a,5);
	sapB->setFcHwInpBuff(sas_1_x_b,0);
	sapB->setFcHwInpBuff(sas_1_y_b,1);
	sapB->setFcHwInpBuff(sas_2_x_b,2);
	sapB->setFcHwInpBuff(sas_2_y_b,3);
	sapB->setFcHwInpBuff(sas_3_x_b,4);
	sapB->setFcHwInpBuff(sas_3_y_b,5);

	// Set to zero the RW speed
	for (i=0; i<4; i++)
		rw[i]->setFcHwInpBuff(0,0);

	// Update cycle counter
	macsCycleCounter++;
}

// -------------------------------------------------------------------
// Assemble TC buffer representing TC send to the AOCS sw. 
// Telecommands are sent at the fourth and sixth cycle.

void AocsTestHarness_2::loadTelecommands()
{
	if (tcCycleCounter==3)		
		loadSlewManoeuvreTelecommand();

	if (tcCycleCounter==4)		
		loadTelecommandTransaction();

	if (tcCycleCounter==6)		
		loadFssReconfigureTelecommand();

	tcCycleCounter++;
}

// -------------------------------------------------------------------
// Assemble TC transaction. 
// The transaction contains with two transaction TC's.
// TC1: mode change for the AOCS mission mode manager
// TC2: reconfiguration for FSS
// The TC carries a time tag that should cause it to be executed in 
// the cycle in which it is loaded

void AocsTestHarness_2::loadTelecommandTransaction()
{
	// Recovery the TC buffer
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();

	// Load the AOCS mission mode manager address in a 4-byte array
	SystemFactory* systemFactory = fwFact.systemFactory;
	int mmAddressInt = (int)(systemFactory->getMissionModeManager());
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// Create a dummy mode change transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ModeChangeTransactionTelecommand mct(0,NULL,1);

	// Load the VTable pointer for class ModeChangeTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Load the FSS reconfiguration manager address in a 4-byte array
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	int frAddressInt = (int)(aocsUnitFactory->getFssReconfigurer());
	int* frAddressPtr = &frAddressInt;
	char* frAddress = (char*)frAddressPtr;

	// Create a dummy reconfigure transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ReconfigureTransactionTelecommand rtt(0,NULL);

	// Load the VTable pointer for class ReconfigureTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int rttVTableAddress = *((int*)&rtt);
#else
	int rttVTableAddress = *(((int*)&rtt+3));
#endif
	int* rttVTablePtrInt = &rttVTableAddress;
	char* rttVTablePtr = (char*)rttVTablePtrInt;

	// Load a 4-byte array with the time-tag for the TC
	AocsTime timeTag = systemFactory->getAocsClock()->getTime()+1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a reconfigure transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=2;	// start of TC packet -- 2 blocks in TC packet

	tcBuffer[2]=14;	// start of first TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=FSP;				// new mode

	tcBuffer[18]=9;		// start of second TC block -- 9 bytes in TC data section
	tcBuffer[19]=0;		// no VTable section 

	tcBuffer[20]=rttVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[21]=rttVTablePtr[1];
	tcBuffer[22]=rttVTablePtr[2];
	tcBuffer[23]=rttVTablePtr[3];

	tcBuffer[24]=rtt.getClassId();	// class identifier

	tcBuffer[25]=frAddress[0];		// FSS reconfigurer address
	tcBuffer[26]=frAddress[1];
	tcBuffer[27]=frAddress[2];
	tcBuffer[28]=frAddress[3];

}

// -------------------------------------------------------------------
// Assemble a simple TC. 
// The TC consists of a simple TC to reconfigure the FSS reconfigurer.
// The TC carries a time tag that should cause it to be executed in 
// the cycle in which it is loaded

void AocsTestHarness_2::loadFssReconfigureTelecommand()
{
	// Recovery the TC buffer
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();

	// Load the FSS reconfiguration manager address in a 4-byte array
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	int frAddressInt = (int)(aocsUnitFactory->getFssReconfigurer());
	int* frAddressPtr = &frAddressInt;
	char* frAddress = (char*)frAddressPtr;

	// Create a dummy reconfigure transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ReconfigureTelecommand rtt(0,NULL);

	// Load the VTable pointer for class ReconfigureTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int rttVTableAddress = *((int*)&rtt);
#else
	int rttVTableAddress = *(((int*)&rtt+3));
#endif
	int* rttVTablePtrInt = &rttVTableAddress;
	char* rttVTablePtr = (char*)rttVTablePtrInt;

	// Load a 4-byte array with the time-tag for the TC
	SystemFactory* systemFactory = fwFact.systemFactory;
	AocsTime timeTag = systemFactory->getAocsClock()->getTime()+1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a reconfigure transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet

	tcBuffer[2]=13;	// start of first TC block -- 13 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=rttVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=rttVTablePtr[1];
	tcBuffer[6]=rttVTablePtr[2];
	tcBuffer[7]=rttVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=rtt.getClassId();	// class identifier

	tcBuffer[13]=frAddress[0];		// FSS reconfigurer address
	tcBuffer[14]=frAddress[1];
	tcBuffer[15]=frAddress[2];
	tcBuffer[16]=frAddress[3];

}

// -------------------------------------------------------------------
// Assemble a simple TC. 
// The TC consists of a simple TC to start an attitude slew manoeuvre.
// The TC carries a time tag that should cause it to be executed in 
// the cycle in which it is loaded

void AocsTestHarness_2::loadSlewManoeuvreTelecommand()
{
	// Recovery the TC buffer
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();

	// Recover the manoeuvre manager and the attitude slew
	ManoeuvreFactory* manFactory = fwFact.manoeuvreFactory;
	ManoeuvreManager* mnManager = manFactory->getManoeuvreManager();
	AttitudeSlew* as = manFactory->getAttitudeSlew(0);

	// Recover the current time
	SystemFactory* systemFactory = fwFact.systemFactory;
	AocsClock* aocsClock = systemFactory->getAocsClock();
	AocsTime currentTime = aocsClock->getTime();

	// Create the attitude slew manoeuvre that the TC will load
	// The attitude slew updates the X-axis attitude set point in the
	// data pool. The attitude slew manoeuvre needs its actual
	// address and the address of its time tag.
	AocsTime startTime = currentTime;			
	AocsTime endTime = currentTime*10;			
	float errorThreshold = 100;
	float att_end = 10;				// slew target attitude
	DataItemWrite diw = systemFactory->getAttitudeDataPool()
						->getAttSetPoint()->getDataItemWrite(0);
	
	int* diwAddr = (int*)&diw;
	Real* itemToSet = (Real*)diwAddr[1];
	AocsTime* itemTimeTag = (AocsTime*)diwAddr[2];

	// Configure the static part of the attitude slew telecommand
	AttitudeSlewTelecommand::setManoeuvreManager(mnManager);

	// Load the attitude slew manoeuvre address in a 4-byte array
	int asAddressInt = (int)as;
	int* asAddressPtr = &asAddressInt;
	char* asAddress = (char*)asAddressPtr;

	// Create a dummy attitude slew TC. It is needed to recover the VTable 
	// pointer for class AttitudeSlewTelecommand.
	AttitudeSlewTelecommand astc(0,as, errorThreshold, att_end, startTime,
									endTime, itemToSet, itemTimeTag);

	// Load the VTable pointer for class AttitudeSlewTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int astcVTableAddress = *((int*)&astc);
#else
	int astcVTableAddress = *(((int*)&astc+3));
#endif
	int* astcVTablePtrInt = &astcVTableAddress;
	char* astcVTablePtr = (char*)astcVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = 0;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Load a 4-byte array with the error threshold
	int* errorThresholdPtrInt = (int*)&errorThreshold;
	char* errorThresholdPtr = (char*)errorThresholdPtrInt;

	// Load a 4-byte array with the attitude end value
	int* att_endPtrInt = (int*)&att_end;
	char* att_endPtr = (char*)att_endPtrInt;

	// Load a 4-byte array with the start time for the attitude slew
	int* startTimePtrInt = (int*)&startTime;
	char* startTimePtr = (char*)startTimePtrInt;

	// Load a 4-byte array with the end time for the attitude slew
	int* endTimePtrInt = (int*)&endTime;
	char* endTimePtr = (char*)endTimePtrInt;

	// Load a 4-byte array with pointer to the item to set
//Real* itemToSetAddr = &itemToSet;
	int* itemToSetPtrInt = (int*)&itemToSet;
	char* itemToSetPtr = (char*)itemToSetPtrInt;

	// Load a 4-byte array with pointer to the item to set
//	AocsTime* itemTimeTagAddr = &itemTimeTag;
	int* itemTimeTagPtrInt = (int*)&itemTimeTag;
	char* itemTimeTagPtr = (char*)itemTimeTagPtrInt;

	// Prepare the TC buffer. The TC buffer consists of a single 
	// TC packet containing the manoeuvre TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=37;	// start of TC block -- 33 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=astcVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=astcVTablePtr[1];
	tcBuffer[6]=astcVTablePtr[2];
	tcBuffer[7]=astcVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=astc.getClassId();	// class identifier

	tcBuffer[13]=asAddress[0];		// attitude slew address
	tcBuffer[14]=asAddress[1];
	tcBuffer[15]=asAddress[2];
	tcBuffer[16]=asAddress[3];

	tcBuffer[17]=errorThresholdPtr[0];		// error threshold
	tcBuffer[18]=errorThresholdPtr[1];
	tcBuffer[19]=errorThresholdPtr[2];
	tcBuffer[20]=errorThresholdPtr[3];

	tcBuffer[21]=att_endPtr[0];				// attitude end point
	tcBuffer[22]=att_endPtr[1];
	tcBuffer[23]=att_endPtr[2];
	tcBuffer[24]=att_endPtr[3];

	tcBuffer[25]=startTimePtr[0];			// slew starting time
	tcBuffer[26]=startTimePtr[1];
	tcBuffer[27]=startTimePtr[2];
	tcBuffer[28]=startTimePtr[3];

	tcBuffer[29]=endTimePtr[0];				// slew end time
	tcBuffer[30]=endTimePtr[1];
	tcBuffer[31]=endTimePtr[2];
	tcBuffer[32]=endTimePtr[3];

	tcBuffer[33]=itemToSetPtr[0];			// item to set
	tcBuffer[34]=itemToSetPtr[1];
	tcBuffer[35]=itemToSetPtr[2];
	tcBuffer[36]=itemToSetPtr[3];

	tcBuffer[37]=itemTimeTagPtr[0];			// time tag of item to set
	tcBuffer[38]=itemTimeTagPtr[1];
	tcBuffer[39]=itemTimeTagPtr[2];
	tcBuffer[40]=itemTimeTagPtr[3];
}

// -------------------------------------------------------------------
// Set the spacecraft torque. 

void AocsTestHarness_2::generateActuatorCommands()
{
	return;
}