// TimingTestErc32.cpp 
//
// Version	1.0
// Date		27.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Test program for the AOCS prototype - ERC32 Version
//
// The program is designed to compute the time required to run the
// AOCS active objects. Because of poor clock resolution, the timing
// measurement is performed on a sequence of a large number of 
// AOCS cycles. The AOCS cycles are run in an RTEMS task.
// The initialization task (Init) creates and initializes the AOCS
// framework and creates the RTEMS task where the timing measurement
// is performed.
//
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../FailureDetection/FailureDetectionManager.h"
#include "../FailureRecovery/FailureRecoveryManager.h"
#include "../AocsUnit/UnitTrigger.h"
#include "../TelemetryManagement/TelemetryManager.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelecommandManagement/TelecommandLoader.h"
#include "../ManoeuvreManagement/ManoeuvreManager.h"
#include "../ControllerManagement/ControllerManager.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsConfiguration.h"
#include "../AocsFactory/SystemFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/TelecommandFactory.h"
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/FailureRecoveryFactory.h"
#include "../AocsFactory/FailureDetectionFactory.h"
#include "../AocsFactory/MonitoringFactory.h"
#include "../AocsFactory/ModeFactory.h"
#include "../AocsFactory/ManoeuvreFactory.h"
#include "../AocsFactory/ControllerFactory.h"

// Include test header files
#include "../AocsPrototypeTest/AocsPrototypeTest.h"
#include "../AocsPrototypeTest/AocsTestHarness_2.h"
#include "../GeneralUtilities/FlushTelemetryBuffers.h"
#include "../GeneralUtilities/FlushMacsBuffers.h"
#include "../GeneralUtilities/TestUtilities.h"

// Define useful macros and symbols (from sample RTEMS tests)

#include <tmacros.h>

// Specify C linkage for initialization/task routines (called by rtems exec) 

extern "C" {
	rtems_task Init( rtems_task_argument argument);
	rtems_task AocsTask(rtems_task_argument unused);
}

// Define RTEMS configuration (the #define variables control switches 
// in file /erc32/rtemsnp/rtems/erc32/lib/include/confdefs.h)							
#define TEST_INIT
#define CONFIGURE_SPTEST

#define CONFIGURE_TEST_NEEDS_CONSOLE_DRIVER
#define CONFIGURE_TEST_NEEDS_CLOCK_DRIVER

#define CONFIGURE_RTEMS_INIT_TASKS_TABLE
#define CONFIGURE_INIT

#define CONFIGURE_INIT_TASK_STACK_SIZE 2*RTEMS_MINIMUM_STACK_SIZE

#define CONFIGURE_INIT_TASK_ATTRIBUTES RTEMS_FLOATING_POINT

#include <confdefs.h>

// Define task global variables 

Runnable* aocsTasks[N_AOCS_TASKS];    // array of AOCS active objects
rtems_id   Rtems_Task_id[ 4 ];        // array of task ids 	
rtems_name Rtems_Task_name[ 4 ];      // array of task names
struct FrameworkFactories fwFact;     // AOCS factories
AocsTestHarness* aocsTestHarness;     	

//----------------------------------------------------------------------
// Entry point for initialization task (called by rtems executive)

rtems_task Init ( rtems_task_argument argument )
{
  rtems_status_code status;
  rtems_time_of_day time;

  // Print header message to output stream
  FILE* out=NULL;
  printTestHeader(out,"AocsPrototypeTest");

  // Print the tick duration
  printString(out,"Ticks per second: ");
  printInt(out, TICKS_PER_SECOND);
  printString(out,"\n");

  // Initialize RTEMS time
  build_time( &time, 01, 01, 1999, 9, 0, 0, 0 );
  status = rtems_clock_set( &time );

  // Construct framework factories and initialize factory structure
  SystemFactory* systemFactory = new SystemFactory;
  TelemetryFactory* telemetryFactory = new TelemetryFactory;
  TelecommandFactory* telecommandFactory = new TelecommandFactory;
  AocsUnitFactory* aocsUnitFactory = new AocsUnitFactory;
  FailureRecoveryFactory* failureRecoveryFactory = new FailureRecoveryFactory;
  FailureDetectionFactory* failureDetectionFactory = new FailureDetectionFactory;
  MonitoringFactory* monitoringFactory = new MonitoringFactory;
  ModeFactory* modeFactory = new ModeFactory;
  ManoeuvreFactory* manoeuvreFactory = new ManoeuvreFactory;
  ControllerFactory* controllerFactory = new ControllerFactory;
  fwFact.systemFactory = systemFactory;
  fwFact.telemetryFactory = telemetryFactory;
  fwFact.telecommandFactory = telecommandFactory;
  fwFact.aocsUnitFactory = aocsUnitFactory;
  fwFact.failureRecoveryFactory = failureRecoveryFactory;
  fwFact.failureDetectionFactory = failureDetectionFactory;
  fwFact.monitoringFactory = monitoringFactory;
  fwFact.modeFactory = modeFactory;
  fwFact.manoeuvreFactory = manoeuvreFactory;
  fwFact.controllerFactory = controllerFactory;

  // Initialize AOCS software
  instantiateAocs(out, fwFact);

  // Create the test harness task
  aocsTestHarness = new AocsTestHarness_2(fwFact, out);

  // Define and load the structure holding the AOCS tasks
  for (int k=0; k<N_AOCS_TASKS; k++)
	aocsTasks[k] = NULL;

  aocsTasks[0] = aocsUnitFactory->getUnitTriggerAcquisition();
  aocsTasks[1] = telemetryFactory->getTelemetryManager();
  aocsTasks[2] = telecommandFactory->getTelecommandLoader();
  aocsTasks[3] = telecommandFactory->getTelecommandManager();
  aocsTasks[4] = failureDetectionFactory->getFailureDetectionManager();
  aocsTasks[5] = failureRecoveryFactory->getFailureRecoveryManager();
  aocsTasks[6] = manoeuvreFactory->getManoeuvreManager();
  aocsTasks[7] = controllerFactory->getControllerManager();
  aocsTasks[N_AOCS_TASKS-2] = aocsUnitFactory->getUnitTriggerSend();
//  aocsTasks[N_AOCS_TASKS-1] = aocsTestHarness;	no test harness in timing test

  // Create the RTEMS task to execute the AOCS tasks
  Rtems_Task_name[ 1 ] = rtems_build_name( 'A', 'O', 'C', 'S' );

  // The fifth parameter is the attribute set for the task
  status = rtems_task_create(
    Rtems_Task_name[ 1 ], 1, 2*RTEMS_MINIMUM_STACK_SIZE, RTEMS_DEFAULT_MODES,
    RTEMS_FLOATING_POINT, &Rtems_Task_id[ 1 ] );

  // Start the RTEMS task to execute the AOCS tasks
  status = rtems_task_start( Rtems_Task_id[ 1 ], AocsTask, 1 );

  // Delete the initialization task
  status = rtems_task_delete( RTEMS_SELF );
}

//----------------------------------------------------------------------
// Entry point for the AOCS task (called by rtems executive). After 
// N_AOCS_CYCLES cycles the task exits and the test terminates.

rtems_task AocsTask(rtems_task_argument unused)
{
  rtems_id          tid;
  rtems_time_of_day time;
  rtems_unsigned32  task_index;
  rtems_status_code status;

  static int cycleCounter=0;
  int n_cycles=100;		
  rtems_interval ticksAtBeginningOfTest;
  rtems_interval ticksAtEndOfTest;

  status = rtems_task_ident( RTEMS_SELF, RTEMS_SEARCH_ALL_NODES, &tid );
  task_index = task_number( tid );

  while( FOREVER ) {

    // Recover current time as ticks since boot
    status = rtems_clock_get( RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &ticksAtBeginningOfTest );

    // Run n_cycle AOCS cycles
    for (int j=0; j<n_cycles; j++)
    	// Run one AOCS cycle
    	for (int j=0; j<N_AOCS_TASKS; j++)
	  if (aocsTasks[j]!=NULL)
	    aocsTasks[j]->run(0);

    // Recovery current time
    status = rtems_clock_get( RTEMS_CLOCK_GET_TICKS_SINCE_BOOT, &ticksAtEndOfTest );

    // Print time and beginning and end of test
    printf("\n%d cycles began at %d ticks and ended at %d ticks",
                  n_cycles,ticksAtBeginningOfTest,ticksAtEndOfTest);
    fflush(stdout);

    cycleCounter++;

    // If test finished, delete task
    if (cycleCounter >= N_AOCS_CYCLES)
    {	FILE* out=NULL;	
	status = rtems_task_delete( RTEMS_SELF );
    }

    status = rtems_clock_get( RTEMS_CLOCK_GET_TOD, &time );

    // Print time and task name
    put_name( Rtems_Task_name[ task_index ], FALSE );
    print_time( " - rtems_clock_get - ", &time, "\n" );

    // Suspend task for 1 second
    status = rtems_task_wake_after( TICKS_PER_SECOND );
  }
}

