// TestCaseAocsUnit_6.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_6.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../SequentialDataProcessing/BiasScalingCompensator.h"
#include "../../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/SapPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_6::TestCaseAocsUnit_6():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+6,
									"TestCaseAocsUnit_6") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_6::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create the raw data converter for the SAS measurements
	RawBusDataConverterIn convIn(6,0.2);

	// Create the bias and scaling factor converter for the SAS sun angle measurements
	BiasScalingCompensator bsc1(1.0,2.0);
	BiasScalingCompensator bsc2(2.0,2.0);
	BiasScalingCompensator bsc3(3.0,2.0);
	BiasScalingCompensator bsc4(4.0,2.0);
	BiasScalingCompensator bsc5(5.0,2.0);
	BiasScalingCompensator bsc6(6.0,2.0);

	// Link the raw data converter and the bias/scaling compensator
	bsc1.linkInput(&convIn,0,0);
	bsc2.linkInput(&convIn,1,0);
	bsc3.linkInput(&convIn,2,0);
	bsc4.linkInput(&convIn,3,0);
	bsc5.linkInput(&convIn,4,0);
	bsc6.linkInput(&convIn,5,0);

	// Create the raw data converter for the THU commands.
	// The first six inputs are reserved to the delay-time commands and 
	// the last six inputs are reserved to the on-time commands.
	// It is assumed that there is no bias/scaling_factor compensation 
	// for the THU commands.
	RawBusDataConverterOut convOut(12,10);

	// Create and configure the converter superblock for the SAS outputs
	ControlChannelSuperBlock sasConv(6,6);
	for (int i=0; i<6; i++)
		sasConv.linkInputInternal(&convIn,i,i);
	sasConv.linkOutputInternal(&bsc1,0,0);
	sasConv.linkOutputInternal(&bsc2,0,1);
	sasConv.linkOutputInternal(&bsc3,0,2);
	sasConv.linkOutputInternal(&bsc4,0,3);
	sasConv.linkOutputInternal(&bsc5,0,4);
	sasConv.linkOutputInternal(&bsc6,0,5);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffers for SAS measurements and their time tag
	Real sas[6];
	AocsTime sasTTag[6];
	DataItemWrite sasDiw[6];
	for (i=0; i<6; i++)
		sasDiw[i].initialize(&sasTTag[i],&sas[i]);
		
	// Create the source buffers for the THU on-time
	Real onTime[6];
	AocsTime onTimeTTag[6];
	DataItemWrite onTimeDir[6];
	for (i=0; i<6; i++)
		onTimeDir[i].initialize(&onTimeTTag[6], &onTime[i]);

	// Create the source buffers for the THU delay-time
	Real delayTime[6];
	AocsTime delayTimeTTag[6];
	DataItemWrite delayTimeDir[6];
	for (i=0; i<6; i++)
		delayTimeDir[i].initialize(&delayTimeTTag[6], &delayTime[i]);
	
	// Create and configure the SAP object
	SapPrototype sap(10,1,1);
	sap.setAocsUnitHardware(&macsTcCont);
	sap.setFunctionalInConverter(&sasConv);
	sap.setFunctionalOutConverter(&convOut);
	for (i=0; i<6; i++)
		sap.setFunctionalInLink(sasDiw[i],i);
	for (i=0; i<6; i++)
		sap.setFunctionalOutLink(delayTimeDir[i],i);
	for (i=0; i<6; i++)
		sap.setFunctionalOutLink(onTimeDir[i],i+6);
	sap.setHwFailureRecoveryAction(&recAct);

	// Check that the SAP object is configured
	if (!sap.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
		return;
	}

	// Acquire the SAP measurement. Since there is no real external unit, six
	// bus errors should be generated. The MACS control register should reflect 
	// the last instruction put on the bus
	sap.acquireFunctionalData();
	if (macsInstr!=-31146)			// -... = 0x8656		
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+6)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
		return;
	}

	// Simulate acquisition of sun position measurements from the SAS
	unsigned short rawSas[6];
	for (i=0; i<6; i++)
	{	rawSas[i] = 0x8030 + i;
		sap.setFcHwInpBuff(rawSas[i],i);
	}

	sap.refreshFunctionalIn();

	for (i=0; i<6; i++)
	{	if ( (sas[i] < (1+i+2*0.2*(48+i)-0.0001)) || (sas[i] > (1+i+2*0.2*(48+i)+0.0001)) )
		{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
			return;
		}
	}

	// Simulate a refresh of the output data for the thruster. 
	for (i=0; i<6; i++)
	{	delayTime[i]=-2*i;
		onTime[i]=-i;
	}
	sap.refreshFunctionalOut();
	for (i=0; i<6; i++)
	{	if (sap.getFcHwOutBuff(i)!=20*i)
		{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
			return;
		}
		if (sap.getFcHwOutBuff(i+6)!=10*i)
		{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
			return;
		}
	}

	// Send the on-time and delay-time commands to the thrusters.
	// Since there is no real external unit, 12 bus errors should have been 
	// generated. The MACS control register should reflect  the last 
	// instruction put on the bus
	sap.sendFunctionalData();
	if (macsInstr!=0x4C56)			
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+18)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 6 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}




