// TestCaseAocsUnit_8.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_8.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../SequentialDataProcessing/BiasScalingCompensator.h"
#include "../../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../../SequentialDataProcessing/TorqueToThrusterPrototype.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/RwPrototype.h"
#include "../../AocsUnit/FssPrototype.h"
#include "../../AocsUnit/FunctionalTriggerList.h"
#include "../../AocsUnit/FollowerUnitTriggerModeManager.h"
#include "../../AocsUnit/UnitTrigger.h"
#include "../../AocsUnit/TorquingThrusters.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_8::TestCaseAocsUnit_8():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+8,
									"TestCaseAocsUnit_8") {
		return;
}

// -------------------------------------------------------------------
// Test the unit trigger object
//
// Two functional trigger lists are created. Trigger list 1 contains 
// the following:
//	- functional synchronization message to the FSS
//	- functional data acquisition from FSS
//	- functional data send to the RW
//	- initialization/shutdown for FSS and RW
// Trigger list 2 contains the following
//	- functional data acquisition from the RW
//	- functional data acquisition from FSS
//	- initialization/shutdown for FSS and RW
//
// The mode manager for the unit trigger follows the AOCS mission manager.

void TestCaseAocsUnit_8::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create the data converter for the FSS and RW units
	RawBusDataConverterIn fssConv(2,0.1);
	RawBusDataConverterIn rwConvIn(1,0.1);
	RawBusDataConverterOut rwConvOut(1,10);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffer for the FSS measurements and
	// their time tag
	Real fss_x, fss_y;
	AocsTime fss_x_tag, fss_y_tag;
	DataItemWrite fss_x_diw(&fss_x_tag, &fss_x);
	DataItemWrite fss_y_diw(&fss_y_tag, &fss_y);

	// Create and configure the FSS object
	FssPrototype fss(1,11,12);
	fss.setAocsUnitHardware(&macsTcCont);
	fss.setFunctionalInConverter(&fssConv);
	fss.setFunctionalInLink(fss_x_diw,0);
	fss.setFunctionalInLink(fss_y_diw,1);
	fss.setHwFailureRecoveryAction(&recAct);

	// Create the destination buffer for the RW speed measurements and
	// their time tag
	Real rwSpeed;
	AocsTime rwSpeedTag;
	DataItemWrite rwSpeedDiw(&rwSpeedTag, &rwSpeed);

	// Create the source buffer for the RW torque request
	Real rwTorque;
	AocsTime rwTorqueTag;
	DataItemWrite rwTorqueDir(&rwTorqueTag,&rwTorque);

	// Create and configure the RW object
	RwPrototype rw(2,11,12);
	rw.setAocsUnitHardware(&macsTcCont);
	rw.setFunctionalInConverter(&rwConvIn);
	rw.setFunctionalInLink(rwSpeedDiw,0);
	rw.setFunctionalOutConverter(&rwConvOut);
	rw.setFunctionalOutLink(rwTorqueDir,0);
	rw.setHwFailureRecoveryAction(&recAct);
 
	// Create and configure trigger list 1
	FunctionalTriggerList list_1(4,4,4,4);
	list_1.addFunctionalAcquire(&fss);
	list_1.addFunctionalSynchronize(&fss);
	list_1.addFunctionalSend(&rw);
	list_1.addInitShutdownUnit(&fss);
	list_1.addInitShutdownUnit(&rw);

	// Create and configure trigger list 2
	FunctionalTriggerList list_2(2,2,2,2);
	list_2.addFunctionalAcquire(&rw);
	list_2.addFunctionalAcquire(&fss);
	list_2.addInitShutdownUnit(&fss);
	list_2.addInitShutdownUnit(&rw);

	// Create and configure an AOCS mission manager
	AocsMissionModeManager aocsMissionModeManager(2,0,5);
	NullModeChangeAction mca;
	aocsMissionModeManager.setModeChangeAction(&mca);

	// Create and configure the unit trigger mode manager
	FollowerUnitTriggerModeManager futmm(2,5,&aocsMissionModeManager);
	futmm.loadTriggerList(0,&list_1);
	futmm.loadTriggerList(1,&list_2);
	futmm.setModeChangeAction(&mca);

	// Create and configure the unit trigger object
	UnitTrigger unitTrigger;
	unitTrigger.setUnitTriggerModeManager(&futmm);

	// Check object configuration
	if (!fss.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	if (!rw.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	if (!list_1.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	if (!list_2.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	if (!futmm.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	if (!unitTrigger.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	// Load the fss hardware buffers to simulate a data acquisition
	fss.setFcHwInpBuff(400,0);
	fss.setFcHwInpBuff(200,1);

	// Load the RW source buffer
	rwTorque = 0.2;

	// Run the unit trigger
	unitTrigger.run(0);

	// A total of 6 MACS bus transactions should have been started:
	//	- 1 initialize transaction to FSS
	//	- 1 initialize transaction to RW
	//	- 1 synchronize transaction to the FSS
	//	- 2 data acquisition to the FSS (1 for each channel)
	//	- 1 data send transaction to the RW
	// Since there is no real bus connection, they should have resulted in 
	// failure events being generated
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+6)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	// Check that the data were correctly transferred to/from the hw buffers
	if (fss_x!=-40)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}
	if (fss_y!=-20)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}
	if (rw.getFcHwOutBuff(0) != 32768+2)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	// Change the mission manager mode (this should cause a change in the 
	// unit trigger mode and should cause initialize transaction being 
	// sent to the FSS and RW)
	aocsMissionModeManager.setMode(1);

	// Load the fss and rw hardware buffers to simulate a data acquisition
	fss.setFcHwInpBuff(800,0);
	fss.setFcHwInpBuff(300,1);
	rw.setFcHwInpBuff(100,0);

	// Advance the clock and run the unit trigger
	aocsClock.run(0);
	unitTrigger.run(0);

	// A total of 5 MACS bus transactions should have been started. Since there
	// is no real bus connection, this should have resulted in 5 failure event
	// being generated
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+6+5)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	// Check that the data were correctly transferred to/from the hw buffers
	if (fss_x!=-80)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}
	if (fss_y!=-30)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}
	if (rwSpeed!=-10)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 8 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}

