// TorquingThruster.cpp
//
// Version	1.0
// Date		06.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsUnit/TorquingThrusters.h"			
#include "../AocsUnit/AocsUnitHardware.h"
#include "../AocsUnit/MacsConstants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsData/AocsData.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsUnit/AocsUnitHardware.h"
#include "../SequentialDataProcessing/AbstractControlChannel.h"

// -------------------------------------------------------------------
// Define static variables
int TorquingThrusters::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

TorquingThrusters::TorquingThrusters(int n)
{
	setClassId(ID_TORQUINGTHRUSTERS);	

	tmImageLength[shortTm]  = 0;	// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	nOfThrusters = n;		// number of thrusters

	onTime = new DataItemWrite[nOfThrusters];
	delayTime = new DataItemWrite[nOfThrusters];

	localResetConfiguration();
	localReset();
}

// -------------------------------------------------------------------

void TorquingThrusters::localReset()
{
	if (torqueToThuConv != NULL)
		torqueToThuConv->reset();
}

// -------------------------------------------------------------------

void TorquingThrusters::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void TorquingThrusters::localResetConfiguration()
{
	torqueToThuConv = NULL;
	lowLevelThuUnit = NULL;
}

// -------------------------------------------------------------------

void TorquingThrusters::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool TorquingThrusters::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if ( (torqueToThuConv==NULL) || (lowLevelThuUnit==NULL) )
		return false;

	if (!torqueToThuConv->isConfigured())
		return false;

	for (int i=0; i<nOfThrusters; i++)
		if ( (!onTime[i].isDataItemWriteConfigured()) || 
			 (!delayTime[i].isDataItemWriteConfigured()) )
			 return false;

	return true;
}

// -------------------------------------------------------------------

void TorquingThrusters::refreshFunctionalOut()
{	
	// Convert the torque requests to thruster on-times
	torqueToThuConv->propagate(getTime());

	// Write the on- and delay-times to the thruster inputs
	for (int i=0; i<nOfThrusters; i++)
	{	onTime[i].set(torqueToThuConv->getOutput(i+nOfThrusters).get());
		delayTime[i].set(torqueToThuConv->getOutput(i).get());
	}

	// Propagate the refresh request to the low level thruster unit
	lowLevelThuUnit->refreshFunctionalOut();
}

// -------------------------------------------------------------------

void TorquingThrusters::setFunctionalOutConverter(AbstractControlChannel* c)
{
	// The converter should have three inputs (the three torque requests) and
	// 2*nOfThrusters outputs (one on-time and one delay-time request for each 
	// thruster)
	if ( (c->getOutputOrder()!=2*nOfThrusters) || (c->getInputOrder()!=3) )
	{	reportConfigurationError(this,ILLEGAL_FC_CONVERTER,this);
		return;
	}

	torqueToThuConv = c;
}	

// -------------------------------------------------------------------

void TorquingThrusters::setFunctionalOutLink(DataItemWrite d, int i)
{
	// Check legality of parameters
	if ( (i<0) || (i>=3) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}
	
	// Link the data source buffer to the inputs of the converter
	if (torqueToThuConv!=NULL)
	{	torque[i]=d;
		torqueToThuConv->linkInput(d,i);
	}
	else
		reportConfigurationError(this,MISSING_FC_OUT_CONV,this);
}

// -------------------------------------------------------------------

void TorquingThrusters::setFunctionalOutLink(AocsData* d)
{
	// Check legality of parameter
	if ( d->getNumberOfItems()!=3 )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}
	
	for (int i=0; i<3; i++)
		setFunctionalOutLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

DataItemWrite TorquingThrusters::getFunctionalOutLink(int i)
{
	// Check legality of parameters
	if ( (i<0) || (i>=3) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);;
	}
	
	return torque[i];
}

// -------------------------------------------------------------------

DataItemWrite TorquingThrusters::getFunctionalInLink(int i)
{
	reportConfigurationError(this,NO_IN_CONVERTER,this); 
	return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
}

// -------------------------------------------------------------------
// Set the low-level thruster unit and retrieve and store the links to 
// the on- and delay-times

void TorquingThrusters::setLowLevelThrusterUnit(AocsUnitFunctional* u)
{
	lowLevelThuUnit = u; 

	for (int i=0; i<nOfThrusters; i++)
	{	onTime[i] = lowLevelThuUnit->getFunctionalOutLink(i+nOfThrusters);
		delayTime[i] = lowLevelThuUnit->getFunctionalOutLink(i);
	}
}

// -------------------------------------------------------------------
// From Telemeterable interface

int TorquingThrusters::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void TorquingThrusters::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
