// Controller.h
//
// Version  1.4
// Date     21.11.2000
// Author   T. Brown
//
// -------------------------------------------------------------------
// Controller is an abstract class providing default implementations
// of its methods.  Concrete controllers are instances of subclasses
// of Controller.
// -------------------------------------------------------------------

#ifndef ControllerH
#define ControllerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsData/DataItemRead.h"
#include "../AocsData/DataItemWrite.h"
#include "../SequentialDataProcessing/P_Block.h"
#include "../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../SequentialDataProcessing/PassThruBlock.h"
#include "../SequentialDataProcessing/DifferenceBlock.h"
#include "../SequentialDataProcessing/AdderBlock.h"
#include "../SequentialDataProcessing/LimitBlock.h"

// -------------------------------------------------------------------
// The Controller class represents a Single-Input-Single-Output (SISO)
// controller.
// -------------------------------------------------------------------

class Controller : public AocsObject {

private :

    // Telemetry Image Lengths for various telemetry formats
    static int tmImageLength[NTMFORMATS];

    // Default input values for unspecified input links to
    // the controller
    static Real zero;

    // Controller Input: measurement from sensor
    // as linked by the client
    DataItemRead measurementInput;

    // Indication of whether or not the measurement
    // input has been linked by the client
    bool measurementInputLinked;

    // DataItemRead to use as measurement input if the
    // input has not been linked by the client
    // Note: It is currently not valid for the client not to link
    // a measurement input signal.  This default measurement input
    // signal is a convenience used to simplify the internal configuration.
    // It is the client's responsibility to determine if the Controller
    // is properly configured by calling the isConfigured() member function.
    DataItemRead defaultMeasurementInput;

    // Controller Input: reference signal to be tracked
    DataItemRead referenceInput;

    // Indication of whether or not the reference signal input has been linked
    bool referenceInputLinked;

    // DataItemRead to use as reference input if the
    // input has not been linked by the client
    DataItemRead defaultReferenceInput;

    // Controller Input: feedforward signal
    DataItemRead feedforwardInput;

    // Indication of whether or not the feedforward signal has been linked
    bool feedforwardInputLinked;

    // DataItemRead to use as feedforward input if the
    // input has not been linked by the client
    DataItemRead defaultFeedForwardInput;

    // Controller Output: actuator command
    DataItemWrite actuatorCommand;

    // Indication of whether or not the actuator command output has been linked
    bool actuatorCommandLinked;

    // Controller Output: compensator output
    DataItemWrite compensatorOutput;

    // Indication of whether or not the compensator output has been linked
    bool compensatorOutputLinked;

    // Controller Output: control error
    DataItemWrite controlError;

    // Indication of whether or not the control error output has been linked
    bool controlErrorLinked;

    // Controller Transfer Function Block: filter for measurement signal
    // as specified by the client
    AbstractControlChannel* measurementFilter;

    // Control block to use as the filter for measurement signal if
    // it has not otherwise been specified by the client
    PassThruBlock defaultMeasurementFilter;

    // Actual filter for measurement signal currently being used
    AbstractControlChannel* actualMeasurementFilter;

    // Controller Transfer Function Block: filter for reference signal
    // as specified by the client
    AbstractControlChannel* referenceInputFilter;

    // Control block to use as the filter for reference signal if
    // it has not otherwise been specified by the client
    PassThruBlock defaultReferenceInputFilter;

    // Actual filter for reference signal currently being used
    AbstractControlChannel* actualReferenceInputFilter;

    // Controller Transfer Function Block: filter for feedforward signal
    // as specified by the client
    AbstractControlChannel* feedforwardFilter;

    // Control block to use as the filter for feedforward signal if
    // it has not otherwise been specified by the client
    PassThruBlock defaultFeedforwardFilter;

    // Actual filter for feedforward signal currently being used
    AbstractControlChannel* actualFeedforwardFilter;

    // Controller Transfer Function Block: implementation of the control law
    // as speified by the client
    AbstractControlChannel* compensator;

    // Control block to use as the Compensator block if it has
    // not otherwise been specified by the client.
    // Note: It is currently not valid for the client not to specify
    // a compensator.  This defaultCompensator is a convenience used
    // to simplify the internal configuration.
    // It is the client's responsibility to determine if the
    // Controller is properly configured by calling the isConfigured()
    // member function.
    PassThruBlock defaultCompensator;

    // Actual compensator block currently being used
    AbstractControlChannel* actualCompensator;

    // Control block used to limit the maximum and minimum actuator command
    // values if those limits have been set by the client
    LimitBlock actuatorLimitBlock;

    // Control block to use as the limit on the maximum and minimum actuator command
    // values if no limits have been set by the client
    PassThruBlock defaultActuatorLimitBlock;

    // Actual limit block currently being used
    AbstractControlChannel* actualActuatorLimitBlock;

    // Indication of whether or not actuator command limits have been set
    bool commandLimitsSet;

    // Actuator Command Limits
    Real highCommandLimit;
    Real lowCommandLimit;

    // ControlChannelBlock for calculating the difference between
    // Reference Signal and Measurement (Filtered Reference Signal - Filtered Measurement)
    DifferenceBlock diffBlock;

    // ControlChannelBlock for adding the compensator output to the
    // feedforward filter output (Compensator Ouput + Filtered Feedforward Signal)
    AdderBlock adderBlock;

    // ControlChannelSuperBlock used to implement control function
    ControlChannelSuperBlock superBlock;

    // Recovery action associated with the instability condition
    RecoveryAction* instabilityRecoveryAction;

    // Indication of whether or not the feedback loop is closed
    // for this controller;
    bool feedbackLoopClosed;

    // perform local reset operations
    // These member functions are "helpers" for implementation of the Resettable
    // interface.  All resetting functionality for instances of this class should
    // be in these member functions.  Thus the public member functions reset() and
    // resetConfiguration() will simply call superclass member functions and
    // call these "local" member functions.
    void localReset();
    void localResetConfiguration();

    // perform internal configuration of Controller
    void setupInternalConfiguration();

    // helper functions for performing internal configuration
    void determineActualControlBlocks();
    void makeInternalLinks();
    void linkBlocksIntoSuperBlock();
    void embedBlocksIntoSuperBlock();
    void linkExternalInputsToSuperBlock();

public :

    // create a Controller
    Controller();

    // methods from the Resettable interface
    virtual void reset();
    virtual void resetConfiguration();

    // methods from the Configurable interface
    //virtual void resetConfiguration(); // shared method with Resettable interface
    virtual bool isConfigured();

    // methods from the Telemeterable interface
    virtual int getTelemetryImageLength();
    virtual void writeToTelemetry(TelemetryStream* tmStream);

    // set the control channel which implements the compensator transfer function
    // for this Controller
    // @param   c   control channel to be used as compensator transfer function
    //              for this Controller
    virtual void setCompensator(AbstractControlChannel* c);

    // @return  the control channel currently set to be used as the compensator
    //          transfer function for this Controller
    virtual AbstractControlChannel* getCompensator();

    // set the control channel which implements the measurement filter
    // transfer function for this Controller
    // @param   c   control channel to be used as measurement filter
    //              for this Controller
    virtual void setMeasurementFilter(AbstractControlChannel* c);

    // @return  the control channel currently set to be used as the measurement filter
    virtual AbstractControlChannel* getMeasurementFilter();

    // set the control channel which implements the feedforward filter
    // transfer function for this Controller
    // @param   c   control channel to be used as the feedforward filter
    virtual void setFeedforwardFilter(AbstractControlChannel* c);

    // @return  the control channel currently set to be used as the feedforward filter
    virtual AbstractControlChannel* getFeedforwardFilter();

    // set the control channel which implements the reference signal filter
    // transfer function for this Controller
    // @param   c   control channel to be used as the reference signal filter
    virtual void setReferenceSignalFilter(AbstractControlChannel* c);

    // @return  the control channel currently set to be used as the reference signal filter
    virtual AbstractControlChannel* getReferenceSignalFilter();

    // link the specified DataItem to the Input Reference Signal
    virtual void linkReferenceSignal(DataItemRead d);

    // link the specified DataItem to the Input Sensor Measurement
    virtual void linkSensorMeasurement(DataItemRead d);

    // link the specified DataItem to the Input Feedforward Signal
    virtual void linkFeedforwardSignal(DataItemRead d);

    // link the specified DataItem to the Output Actuator Command
    virtual void linkActuatorCommand(DataItemWrite d);

    // link the specified DataItem to the Compensator Output
    virtual void linkCompensatorOutput(DataItemWrite d);

    // link the specified DataItem to the Output Control Error
    virtual void linkControllerError(DataItemWrite d);

    // set maximum and minimum limits on actuator command output
    // @param   high    upper limit on actuator command
    // @param   low     lower limit on actuator command
    virtual void setCommandLimits(Real high, Real low);

    // remove maximum and minimum limits on actuator command output
    virtual void unSetCommandLimits();

    // A call to this method directs the controller object to perform an
    // internal stability check.
    // @return true if the check indicates that the controller is stable
    //         false otherwise
    virtual bool isStable();

    // Setter and getter methods for the recovery action associated to the
    // controller instability condition
    virtual void setInstabilityRecoveryAction(RecoveryAction* r);
    virtual RecoveryAction* getInstabilityRecoveryAction();

    // Methods to switch between open and closed loop operational mode
    virtual void setOpenLoop();
    virtual void setClosedLoop();

    // Put the controller in "hold" mode: inputs no longer propagated
    // and internal state and external outputs are not updated.
    virtual void hold();

    // Bring the controller back to its normal operation mode (from a hold)
    virtual void resume();

    // A call to this method causes the current measurements to be
    // propagated through the control system and the controller outputs
    // to be accordingly updated.
    virtual void doControl();
};

// -------------------------------------------------------------------
#endif // ControllerH
