// TestCaseController_3.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseController_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/AocsEvent.h"
#include "../../SequentialDataProcessing/P_Block.h"
#include "../../SequentialDataProcessing/D_Block.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/AbstractControlChannel.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ControllerManagement/Controller.h"

// -------------------------------------------------------------------
TestCaseController_3::TestCaseController_3():
	TestCaseGenericSetUp(ID_CONTROLLERMANAGER*10+3,
									"TestCaseController_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseController_3::runTestCase() {

    // Create a Controller
    Controller myController;

    // Create and load dummy instability recovery action
    NullRecoveryAction nra;
    myController.setInstabilityRecoveryAction(&nra);

    // Link measurement input
    Real sensorInputData;
    DataItemRead sensorInput(&sensorInputData);
    myController.linkSensorMeasurement(sensorInput);

    // Link the Reference signal input
    Real referenceSignalData;
    DataItemRead referenceSignal(&referenceSignalData);
    myController.linkReferenceSignal(referenceSignal);

    // Link the Feedforward signal input
    Real feedforwardSignalData;
    DataItemRead feedforwardSignal(&feedforwardSignalData);
    myController.linkFeedforwardSignal(feedforwardSignal);

    // Link actuator command output
    Real actuatorCommandOutputData;
    AocsTime actuatorOutputTime;
    DataItemWrite actuatorCommandOutput(&actuatorOutputTime, &actuatorCommandOutputData);
    myController.linkActuatorCommand(actuatorCommandOutput);

    // Link the compensator output
    Real compensatorOutputData;
    AocsTime compensatorOutputTime;
    DataItemWrite compensatorOutput(&compensatorOutputTime, &compensatorOutputData);
    myController.linkCompensatorOutput(compensatorOutput);

    // Link the control error output
    Real controlErrorOutputData;
    AocsTime controlErrorOutputTime;
    DataItemWrite controlErrorOutput(&controlErrorOutputTime, &controlErrorOutputData);
    myController.linkControllerError(controlErrorOutput);

    // Set Measurement Filter
    PassThruBlock measurementFilter;
    myController.setMeasurementFilter(&measurementFilter);

    // Set Reference Signal Filter
    D_Block referenceFilter;
    referenceFilter.setGain(1.0);
    referenceFilter.setMaximumDerivative(1000.0);
    myController.setReferenceSignalFilter(&referenceFilter);

    // Set Feedforward Filter
    PassThruBlock feedforwardFilter;
    myController.setFeedforwardFilter(&feedforwardFilter);

    // Set the compensator transfer function
    PassThruBlock compensatorFilter;
    myController.setCompensator(&compensatorFilter);

    // setup input values
    sensorInputData = 0.0;
    referenceSignalData = -21.0;
    feedforwardSignalData = 0.0;

    // Ask the controller to do its job

    // Note: running the dummy clock moves it forward
    // by dummyPeriod cycles (as #define'd in DummyAocsClock.h.
    // So the time from creation of the controller (which
    // is set as the last propagation time) to the time
    // of this doControl call will be 10 units.
    // Therefore DT in the D_Block will be 10.
    // Previous state will be initialized to 0,
    // so the expected effect of the referenceFilter
    // D_Block when propagated is to
    //
    //   subtract 0 from the input value (see referenceSignalData
    //   value assigned above) [result so far = -21.0]
    //
    //   multiply that result by the gain value (see setGain call
    //   above) [result so far = -21.0]
    //
    //   divide that result by 10
    //   [result = -2.1]

    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != -2.1)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != -2.1)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != -2.1)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // Do not change the inputs

    // Now the D_Block filter will give a 0
    // value for the reference signal resulting
    // in all zeros for the outputs
    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 0.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 0.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 0.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // Change the Reference Signal value by 10.0
    // This should result in all the outputs being
    // equal to 1.0
    referenceSignalData += 10.0;

    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 1.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 1.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 1.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}


    // Change the Reference Signal value by 20.0 again.
    // This would normally result in all the outputs being
    // equal to 2.0
    referenceSignalData += 20.0;

    // But this time we're going to put the whole
    // controller block on hold, so all the outputs
    // should remain fixed.

    // However, since hold is not implemented in the framework
    // prototype for the ControlChannleSuperBlock class, this
    // hold does not work as of yet, and the output values
    // are updated.  When hold is implemented for ControlChannelSuperBlock,
    // the output values checked here should be set to 1.0, and
    // a resume check should be added.

    myController.hold();

    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 2.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 2.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 2.0)
	{	setTestResult(TEST_FAILURE, "Controller test 3 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}

