// TestCaseController_4.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseController_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/DataItemRead.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/PassThruBlock.h"
#include "../../SequentialDataProcessing/D_Block.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/AbstractControlChannel.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ControllerManagement/Controller.h"
#include "../../ControllerManagement/ControllerManager.h"
#include "../../ControllerManagement/FollowerControllerModeManager.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseController_4::TestCaseController_4():
	TestCaseGenericSetUp(ID_CONTROLLERMANAGER*10+4,
									"TestCaseController_4") {
		return;
}

// -------------------------------------------------------------------

void TestCaseController_4::runTestCase() {

    // ----------------------------------------
    // Create some simple controllers
    // ----------------------------------------

    // Create an instability recovery action
    NullRecoveryAction nra;

    // controller0: pure passthru

    Controller controller0;
    PassThruBlock compensator0;
    controller0.setCompensator(&compensator0);
    controller0.setInstabilityRecoveryAction(&nra);

    Real sensorInputData0;
    DataItemRead sensorInput0(&sensorInputData0);
    controller0.linkSensorMeasurement(sensorInput0);

    Real actuatorOutputData0;
    AocsTime actuatorOutputTime0;
    DataItemWrite actuatorOutput0(&actuatorOutputTime0, &actuatorOutputData0);
    controller0.linkActuatorCommand(actuatorOutput0);

    // controller1: pure passthru

    Controller controller1;
    PassThruBlock compensator1;
    controller1.setCompensator(&compensator1);
    controller1.setInstabilityRecoveryAction(&nra);

    Real sensorInputData1;
    DataItemRead sensorInput1(&sensorInputData1);
    controller1.linkSensorMeasurement(sensorInput1);

    Real actuatorOutputData1;
    AocsTime actuatorOutputTime1;
    DataItemWrite actuatorOutput1(&actuatorOutputTime1, &actuatorOutputData1);
    controller1.linkActuatorCommand(actuatorOutput1);

    // controller2: pure passthru

    Controller controller2;
    PassThruBlock compensator2;
    controller2.setCompensator(&compensator2);
    controller2.setInstabilityRecoveryAction(&nra);

    Real sensorInputData2;
    DataItemRead sensorInput2(&sensorInputData2);
    controller2.linkSensorMeasurement(sensorInput2);

    Real actuatorOutputData2;
    AocsTime actuatorOutputTime2;
    DataItemWrite actuatorOutput2(&actuatorOutputTime2, &actuatorOutputData2);
    controller2.linkActuatorCommand(actuatorOutput2);

    // controller3: pure passthru

    Controller controller3;
    PassThruBlock compensator3;
    controller3.setCompensator(&compensator3);
    controller3.setInstabilityRecoveryAction(&nra);

    Real sensorInputData3;
    DataItemRead sensorInput3(&sensorInputData3);
    controller3.linkSensorMeasurement(sensorInput3);

    Real actuatorOutputData3;
    AocsTime actuatorOutputTime3;
    DataItemWrite actuatorOutput3(&actuatorOutputTime3, &actuatorOutputData3);
    controller3.linkActuatorCommand(actuatorOutput3);

    // ----------------------------------------
    // Create controller lists, mode managers,
    // etc. and build a ControllerManager
    // ----------------------------------------

    // Create controller list 0
    ObjectListTemplate<Controller> cList0;
    cList0.allocateMemory(10);
    cList0.addObject(&controller0);
    cList0.addObject(&controller2);

    // Create controller list 1
    ObjectListTemplate<Controller> cList1;
    cList1.allocateMemory(10);
    cList1.addObject(&controller1);
    cList1.addObject(&controller2);

    // cList0: controller0, controller2
    // cList1: controller1, controller2


    // Create and configure an AOCS mission mode manager
    NullModeChangeAction mca1;
    AocsMissionModeManager aocsMissionModeManager(2, 0, 5);
    aocsMissionModeManager.setModeChangeAction(&mca1);

    // Create and configure a controller mode manager
    FollowerControllerModeManager fcmm(2, // int numberOfModes
                                       5, // int maxNumberOfMonitors
                                       &aocsMissionModeManager); // ModeManager* masterModeManager

    fcmm.loadControllerList(0, &cList0);
    fcmm.loadControllerList(1, &cList1);

    // Create and load the mode change actions in the FD mode manager
    NullModeChangeAction mca2;
    fcmm.setModeChangeAction(&mca2);

    // Create and configure ControllerManager
    ControllerManager cManager;
    cManager.setModeManager(&fcmm);
    if (cManager.getModeManager() != &fcmm)
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}

    // ----------------------------------------
    // Test 0: Setup input, run controller
    //         in mode 0, and test output
    // ----------------------------------------

    // controller0: pure passthru
    sensorInputData0 = 121.0;
    actuatorOutputData0 = 0.0;
    // after cManager is run, actuatorOutputData0 should be -121.0
    // because controller0 is in cList0

    // controller1: pure passthru
    sensorInputData1 = -30.0;
    actuatorOutputData1 = 0.0;
    // after cManager is run, actuatorOutputData1 should be unchanged
    // because controller1 is not in cList0

    // controller2: pure passthru
    sensorInputData2 = 1.0;
    actuatorOutputData2 = 0.0;
    // after cManager is run, actuatorOutputData2 should be -1.0
    // controller2 is in both cList0 and cList1

    // controller3: pure passthru
    sensorInputData3 = 100.0;
    actuatorOutputData3 = 2000.0;
    // after cManager is run, actuatorOutputData3 should be unchanged
    // because controller3 is not in cList0 or cList1




    // Run the controller Manager
    aocsClock.run(0);
    cManager.run(0);

    // Check output

    if (actuatorOutputData0 != -121.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData1 != 0.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData2 != -1.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData3 != 2000.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}

    // ----------------------------------------
    // Test 1: Setup input, run controller
    //         in mode 1, and test output
    // ----------------------------------------

    // controller0: pure passthru
    sensorInputData0 = 120.0;
    actuatorOutputData0 = 1.0;
    // after cManager is run, actuatorOutputData0 should be unchanged
    // because controller0 is not in cList1

    // controller1: pure passthru
    sensorInputData1 = -29.0;
    actuatorOutputData1 = 0.0;
    // after cManager is run, actuatorOutputData1 should be 29.0
    // because controller1 is in cList1

    // controller2: pure passthru
    sensorInputData2 = -17.5;
    actuatorOutputData2 = 0.0;
    // after cManager is run, actuatorOutputData2 should be 17.5
    // because controller2 is in cList0 and cList1

    // controller3: pure passthru
    sensorInputData3 = 100.0;
    actuatorOutputData3 = 2000.0;
    // after cManager is run, actuatorOutputData3 should be unchanged
    // because controller3 is not in cList0 or cList1


    // Change the mission manager mode
    // This should cause a change in the controller manager mode
    aocsMissionModeManager.setMode(1);

    // Run the controller Manager
    aocsClock.run(0);
    cManager.run(0);


    // Check output

    if (actuatorOutputData0 != 1.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData1 != 29.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData2 != 17.5)
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData3 != 2000.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}



    // ----------------------------------------
    // Test 2: Change cList0, setup input,
    //         run controller in mode 1,
    //         and test output
    // ----------------------------------------

    cList0.addObject(&controller3);

    // cList0: controller0, controller2, controller3
    // cList1: controller1, controller2

    // controller0: pure passthru
    sensorInputData0 = 12.0;
    actuatorOutputData0 = 1.0;
    // after cManager is run, actuatorOutputData0 should be -12.0
    // because controller0 is in cList0

    // controller1: pure passthru
    sensorInputData1 = -12345.0;
    actuatorOutputData1 = 0.0;
    // after cManager is run, actuatorOutputData1 should be unchanged
    // because controller1 is not in cList0

    // controller2: pure passthru
    sensorInputData2 = -54321.0;
    actuatorOutputData2 = 0.0;
    // after cManager is run, actuatorOutputData2 should be 54321.0
    // because controller2 is in cList0 and cList1

    // controller3: pure passthru
    sensorInputData3 = 100.0;
    actuatorOutputData3 = 2000.0;
    // after cManager is run, actuatorOutputData3 should be -100.0
    // because controller3 is now in cList0


    // Change the mission manager mode
    // This should cause a change in the controller manager mode
    aocsMissionModeManager.setMode(0);

    // Run the controller Manager
    aocsClock.run(0);
    cManager.run(0);

    // Check output

    if (actuatorOutputData0 != -12.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData1 != 0.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData2 != 54321.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}
    if (actuatorOutputData3 != -100.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 4 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


