// TestCaseFailureDetection_2.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureDetection_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../FailureDetection/FailureDetectionManager.h"
#include "../../FailureDetection/FollowerFailureDetectionModeManager.h"
#include "../../FailureDetection/MonitoringCheck.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ObjectMonitoring/Property.h"
#include "../../ObjectMonitoring/SimpleChange.h"

// -------------------------------------------------------------------
TestCaseFailureDetection_2::TestCaseFailureDetection_2():
	TestCaseGenericSetUp(ID_FAILUREDETECTIONMANAGER*10+2,
									"TestCaseFailureDetection_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseFailureDetection_2::runTestCase() {

	// Create the scalar and the twoEulerAngles data
	TwoEulerAngles euler;
	Scalar scalar;

	// Set the values of the scalar and twoEulerAngles data
	scalar.getDataItemWrite(0).set(1);
	euler.getDataItemWrite(0).set(2);
	euler.getDataItemWrite(1).set(3);

	// Create and configure the AOCS mission mode manager
	AocsMissionModeManager aocsMissionModeManager(2,0,5);
	NullModeChangeAction mca;
	aocsMissionModeManager.setModeChangeAction(&mca);

	// Create the three properties to be subjected to monitoring
	Property p1=scalar.getProperty(0);
	Property p2=euler.getProperty(1);
	Property p3=aocsMissionModeManager.getModeProperty();

	// Create the change objects to monitor the property values
	SimpleChange sc1(1);		// 1=value of scalar datum
	SimpleChange sc2(3);		// 3=value of 2nd element of twoEulerAngles	
	SimpleChange sc3(0);		// 0=aocs Mission Manager mode

	// Create the monitoring check objects
	MonitoringCheck mc1(p1,&sc1,NULL);
	MonitoringCheck mc2(p2,&sc2,NULL);
	MonitoringCheck mc3(p3,&sc3,NULL);

	// Create and allocate memory for the list of consistency 
	// checkable objects (kept empty for this test)
	ObjectListTemplate<ConsistencyCheckable> ccList;
	ccList.allocateMemory(10);

	// Create and allocate memory for the list of monitoring 
	// check objects
	ObjectListTemplate<MonitoringCheck> mcList;
	mcList.allocateMemory(10);

	// Load the list of monitoring check objects
	mcList.addObject(&mc1);
	mcList.addObject(&mc2);
	mcList.addObject(&mc3);

	// Create the failure detection mode manager
	FollowerFailureDetectionModeManager fdmm(2,5,&aocsMissionModeManager);

	// Load the consistency checkable and monitoring check lists
	// in the failure detection mode manager
	fdmm.loadConsistencyCheckableList(0,&ccList);
	fdmm.loadConsistencyCheckableList(1,&ccList);
	fdmm.loadMonitoringCheckList(0,&mcList);
	fdmm.loadMonitoringCheckList(1,&mcList);

	// Load the mode change actions in the FD mode manager
	fdmm.setModeChangeAction(&mca);

	// Create the failure detection manager
	FailureDetectionManager fdm;

	// Plug the mode manager in the failure detection manager
	fdm.setFailureDetectionModeManager(&fdmm);
	fdm.setChangeEventRepository(ModeManager::getChangeEventRepository());

	// Retrieve the number of failure events
	failureEvtN = AocsObject::getFailureEventRepository()->getCounter();

	// Run the failure detection manager 
	fdm.run(0);

	// Check that no failure events were generated (all objects 
	// loaded in the consistency checkable list were consistent)
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// Force a change in two of the monitored properties
	scalar.getDataItemWrite(0).set(4);
	aocsMissionModeManager.setMode(1);

	// Retrieve the number of change events
	int changeEvtN = ModeManager::getChangeEventRepository()->getCounter();

	// Run the failure detection manager 
	fdm.run(0);

	// Check that two failure events and two change events were generated 
	// (corresponding to the two properties that changed value)
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+2)
	{	setTestResult(TEST_FAILURE, "Wrong number of failure event");
		return;
	}
	if (ModeManager::getChangeEventRepository()->getCounter()!=changeEvtN+2)
	{	setTestResult(TEST_FAILURE, "Wrong number of change event");
		return;
	}

	// Check that the last created failure event is of the correct type
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->getEventType()!=
			MONITORING_CHECK_FAILURE)
	{	setTestResult(TEST_FAILURE, "Wrong failure event");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
