// TestCaseFailureDetection_3.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureDetection_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../FailureDetection/FailureDetectionManager.h"
#include "../../FailureDetection/FollowerFailureDetectionModeManager.h"
#include "../../FailureDetection/MonitoringCheck.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ObjectMonitoring/Property.h"
#include "../../ObjectMonitoring/SimpleChange.h"
#include "../../ObjectMonitoring/DeltaChange.h"

// -------------------------------------------------------------------
TestCaseFailureDetection_3::TestCaseFailureDetection_3():
	TestCaseGenericSetUp(ID_FAILUREDETECTIONMANAGER*10+3,
									"TestCaseFailureDetection_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseFailureDetection_3::runTestCase() {

	// Create a scalar datum
	Scalar scalar;

	// Set the initial value of the scalar datum
	scalar.getDataItemWrite(0).set(1);

	// Retrieve the property encapsulating the scalar datum value
	Property p1=scalar.getProperty(0);

	// Create the change objects to monitor the scalar 
	SimpleChange sc1(1);		// 1=value of scalar datum
	DeltaChange dc1(10);
	
	// Create the monitoring check objects
	MonitoringCheck mc1(p1,&sc1,NULL);
	MonitoringCheck mc2(p1,&dc1,NULL);

	// Create and allocate memory for the list of consistency 
	// checkable objects 
	ObjectListTemplate<ConsistencyCheckable> ccList;
	ccList.allocateMemory(10);

	// Load the consistency checkable list
	ccList.addObject(&scalar);

	// Create and allocate memory for the two lists of monitoring 
	// check objects
	ObjectListTemplate<MonitoringCheck> mcList1;
	ObjectListTemplate<MonitoringCheck> mcList2;
	mcList1.allocateMemory(10);
	mcList2.allocateMemory(10);

	// Load the two lists of monitoring check objects
	mcList1.addObject(&mc1);
	mcList2.addObject(&mc2);

	// Create and configure the AOCS mission mode manager
	AocsMissionModeManager aocsMissionModeManager(2,0,5);
	NullModeChangeAction mca;
	aocsMissionModeManager.setModeChangeAction(&mca);

	// Create the failure detection mode manager
	FollowerFailureDetectionModeManager fdmm(2,5,&aocsMissionModeManager);

	// Load the consistency checkable and monitoring check lists
	// in the failure detection mode manager
	fdmm.loadConsistencyCheckableList(0,&ccList);
	fdmm.loadConsistencyCheckableList(1,&ccList);
	fdmm.loadMonitoringCheckList(0,&mcList1);
	fdmm.loadMonitoringCheckList(1,&mcList2);

	// Load the mode change actions in the FD mode manager
	fdmm.setModeChangeAction(&mca);

	// Create the failure detection manager
	FailureDetectionManager fdm;

	// Configure the mode manager in the failure detection manager
	fdm.setFailureDetectionModeManager(&fdmm);
	fdm.setChangeEventRepository(ModeManager::getChangeEventRepository());

	// Retrieve the number of failure events
	failureEvtN = AocsObject::getFailureEventRepository()->getCounter();

	// Run the failure detection manager 
	fdm.run(0);

	// Check that no failure events were generated 
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// Cause a mode switch in the FD manager (the FD mode manager follows
	// the AOCS mission manager)
	aocsMissionModeManager.setMode(1);

	// Run the failure detection manager 
	fdm.run(0);

	// Change the scalar value by less than the delta change threshold
	scalar.getDataItemWrite(0).set(1.1);

	// Run the failure detection manager 
	fdm.run(0);

	// Check that no failure events were generated 
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// Change the scalar value by more than the delta change threshold
	scalar.getDataItemWrite(0).set(1.1+10.1);

	// Run the failure detection manager 
	fdm.run(0);

	// Check that one failure event was generated (the delta change monitoring
	// check should have failed)
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
	{	setTestResult(TEST_FAILURE, "Wrong number of failure events");
		return;
	}

	// Go back to mode 0
	aocsMissionModeManager.setMode(0);

	// Bring back the scalar to its original value
	scalar.getDataItemWrite(0).set(1);

	// Run the failure detection manager 
	fdm.run(0);

	// Check that no failure events were generated 
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// Go to mode 1 
	aocsMissionModeManager.setMode(1);

	// Run the failure detection manager 
	fdm.run(1);

	// Check that no failure events were generated (the delta change object
	// should have been reset upon switching to mode 1
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
