// TestCaseFailureRecovery_1.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureRecovery_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../FailureRecovery/ObjectReset.h"
#include "../../FailureRecovery/ModeChange.h"
#include "../../FailureRecovery/SystemReset.h"

// -------------------------------------------------------------------
TestCaseFailureRecovery_1::TestCaseFailureRecovery_1():
	TestCaseGenericSetUp(ID_FAILURERECOVERYMANAGER*10+1,
									"TestCaseFailureRecovery_1") {
		return;
}

// -------------------------------------------------------------------
// Check the recovery actions
// The following three recovery actions are tested:
//	- orRecAct	(object reset)
//  - mcRecAct	(mode change)
//	- srRecAct	(system reset)

void TestCaseFailureRecovery_1::runTestCase() {

	// Create a scalar to serve as dummy object to be reset
	// (the scalar is made is its own data owner)
	Scalar scalar;
	scalar.getDataItemWrite(0).set(2);

	// Create an object reset recovery action
	ObjectReset orRecAct(&scalar);

	// Disable recovery action
	orRecAct.disable();

	// Check disabled status
	if (orRecAct.isEnabled())
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Trigger object reset action
	orRecAct.doRecovery();

	// Verify that the recovery action was not executed
	if (scalar.getDataItemRead(0).get() != 2.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Enable and re-execute the recovery action
	orRecAct.enable();
	orRecAct.doRecovery();

	// Verify that the recovery action was executed
	if (scalar.getDataItemRead(0).get() != 0.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Set the scalar again to a non-zero value
	scalar.getDataItemWrite(0).set(2);

	// create and configure an AocsMissionManager 
	AocsMissionModeManager ammm(3,1,5);
	NullModeChangeAction mca;
	ammm.setModeChangeAction(&mca);

	// Create a mode change recovery action associated to the AocsMissionManager
	ModeChange mcRecAct(&ammm,2);

	// Trigger the mode change recovery action
	mcRecAct.doRecovery();

	// Check that the mode change took place
	if (ammm.getModeProperty().get() != 2)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Create a SystemReset recovery action
	SystemReset srRecAct;

	// Trigger the system event recovery action
	srRecAct.doRecovery();

	// Check that the system reset was performed
	if (AocsObject::getSystemManager()->getSystemEventRepository()->
			latestSystemEvent()->getEventType()!=SYSTEM_RESET)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Check that the scalar (among others) was reset
	if (scalar.getDataItemRead(0).get() != 0.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Chain the three recovery actions 
	srRecAct.setNextRecoveryAction(&mcRecAct);
	mcRecAct.setNextRecoveryAction(&orRecAct);

	// Reload the scalar
	scalar.getDataItemWrite(0).set(2);

	// Trigger the first recovery action in the chain and check that
	// last recovery action in the chain was executer
	srRecAct.doRecovery();
	if (scalar.getDataItemRead(0).get() != 0.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// Unload the system manager from AocsObject
	AocsObject::setSystemManager(NULL);

	// Check that no failure events were generated
	if (AocsObject::getFailureEventRepository()->getCounter() != failureEvtN)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 1 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
