// TestCaseFailureRecovery_2.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureRecovery_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/RecoveryEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../FailureRecovery/SystemResetOnTooManyFailures.h"
#include "../../FailureRecovery/LocalRecoveryActions.h"
#include "../../FailureRecovery/ObjectReset.h"

// -------------------------------------------------------------------
TestCaseFailureRecovery_2::TestCaseFailureRecovery_2():
	TestCaseGenericSetUp(ID_FAILURERECOVERYMANAGER*10+2,
									"TestCaseFailureRecovery_2") {
		return;
}

// -------------------------------------------------------------------
// Check the recovery strategies
// The following two recovery strategies are tested:
//	- srRecStr (system rest on too many failures)
//  - lraRecStr (local recovery actions)

void TestCaseFailureRecovery_2::runTestCase() {

	// Create a scalar to serve as dummy AocsObject and as dummy object 
	// to be reset (the scalar is made its own data owner)
	Scalar scalar;
	scalar.getDataItemWrite(0).set(2);

	// Create a null recovery action
	NullRecoveryAction nRecAct;

	// Create two dummy failure events
	FailureEventRepository* fer=AocsObject::getFailureEventRepository();
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Create and configure a SystemResetOnTooManyFailures recovery strategy
	SystemResetOnTooManyFailures srRecStr;
	srRecStr.setFailureEventThreshold(3);

	// Trigger the recovery strategy 
	srRecStr.doRecovery();

	// verify that no system reset was performed (the number of failure events
	// is smaller than the failure event threshold)
	if (AocsObject::getSystemManager()->getSystemEventRepository()->
			getCounter()!=0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Create four dummy failure events
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Trigger the recovery strategy 
	srRecStr.doRecovery();

	// Verify that the system reset was performed (the number of failure events
	// is greater than the failure event threshold)
	if (AocsObject::getSystemManager()->getSystemEventRepository()->
			latestSystemEvent()->getEventType()!=SYSTEM_RESET)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Verify that the scalar was reset
	if (scalar.getDataItemRead(0).get() != 0.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Reload the scalar
	scalar.getDataItemWrite(0).set(2);

	// Verify that the correct recovery event was generated
	if (RecoveryStrategy::getRecoveryEventRepository()->
			latestRecoveryEvent()->getEventType()!=SYSTEM_RESET_RECOVERY)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Create a LocalRecoveryAction recovery strategy
	LocalRecoveryActions lraRecStr;

	// Create a recovery action to reset the scalar
	ObjectReset orRecAct(&scalar);

	// Add two dummy failure events including one associated to the
	// object reset recovery action
	fer->create(&scalar,DUMMY_EVENT,&scalar,&orRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Recover the number of recovery events
	int recEvtN=RecoveryStrategy::getRecoveryEventRepository()->getCounter();

	// Execute the LocalRecoveryAction recovery strategy
	lraRecStr.doRecovery();

	// Verify that the scalar was reset
	if (scalar.getDataItemRead(0).get() != 0.0)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Check that 4 recovery events were generated (corresponding to the 
	// 4 failure events processed by the recovery strategy - NB the failure
	// event repository has size 4)
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN+2)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Check that the last recovery event has the correct type
	if (RecoveryStrategy::getRecoveryEventRepository()->latestRecoveryEvent()->
			getEventType()!=LOCAL_RECOVERY_ACTION)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// Unload the system manager from AocsObject
	AocsObject::setSystemManager(NULL);

	// Check that no hidden failure events were generated
	if (AocsObject::getFailureEventRepository()->getCounter() != failureEvtN+8)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 2 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
