// AttitudeSlew.cpp
//
// Version  1.0
// Date     29.11.2000
// Author   T. Brown
//
// -------------------------------------------------------------------

#include <math.h>   // for fabs() function

#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../ObjectList/Include.h"
#include "../AocsData/DataItemRead.h"
#include "../AocsData/DataItemWrite.h"
#include "../ObjectMonitoring/Include.h"
#include "../AocsEvent/Include.h"
#include "Include.h"

// -------------------------------------------------------------------
// static attributes

// Sending data to telemetry is not implemented in f/w prototype,
// so all Telemetry Image Lengths are 0.
int AttitudeSlew::tmImageLength[NTMFORMATS] = {0, 0, 0, 0};
Real AttitudeSlew::dummyTarget = 0;
AocsTime AttitudeSlew::dummyTargetTimeTag = 0;

// -------------------------------------------------------------------
AttitudeSlew::AttitudeSlew() : Manoeuvre()
{
    setClassId(ID_ATTITUDESLEW);

	configure(0,0,0,DataItemWrite(&dummyTargetTimeTag,&dummyTarget),0);
}

// -------------------------------------------------------------------
AttitudeSlew::AttitudeSlew(AocsTime startTime, AocsTime endTime,
                           Real goalValue, DataItemWrite attitude,
                           Real errorThreshold) : Manoeuvre()
{
    setClassId(ID_ATTITUDESLEW);

    configure(startTime, endTime, goalValue, attitude, errorThreshold);
}

// -------------------------------------------------------------------
void AttitudeSlew::configure(AocsTime startTime, AocsTime endTime,
                             Real goalValue, DataItemWrite attitude,
                             Real errorThreshold)
{
    execTime = startTime;   // Note: execTime is an inherited protected
                            // member of the Manoeuvre class

    tf = endTime;
    att_end = goalValue;
    itemToSet = attitude;
    allowedDifference = errorThreshold;
    return;
}

// -------------------------------------------------------------------
bool AttitudeSlew::isConfigured()
{
    if (!Manoeuvre::isConfigured()) return false;

    if (!itemToSet.isDataItemWriteConfigured()) return false;

    return true;
}

// -------------------------------------------------------------------
void AttitudeSlew::localResetConfiguration()
{
    execTime = 0;
    tf = 0;
    att_end = 0;
    allowedDifference = 0;
	itemToSet = DataItemWrite(&dummyTargetTimeTag,&dummyTarget);
    return;
}

// -------------------------------------------------------------------
void AttitudeSlew::resetConfiguration()
{
    Manoeuvre::resetConfiguration();
    localResetConfiguration();
    return;
}

// -------------------------------------------------------------------
bool AttitudeSlew::canStart()
{
    if (isConfigured()) return Manoeuvre::canStart();

    return false;
}

// -------------------------------------------------------------------
void AttitudeSlew::initialize()
{
    t0 = getTime();
    att_start = itemToSet.get();

    // Calculate the slope of the linear slew.
    //
    // Note that the slope is calculated based on the actual starting time (t0)
    // as opposed to the desired starting time, Manoeuvre::execTime.  One implication
    // of this is that starting the manoeuvre prior to its designated start time
    // will NOT cause the manoeuvre to finish earlier than its designated end time.
    // It will simply change the slope of the slew.  Assuming that there is no
    // other factor altering the same value that the manoeuvre is to alter, then
    // the slope of the slew will be lowered, causing a more gradual slew to the
    // desired value.  Similarly, starting the manoeuvre later than its desired
    // starting time, will not cause the manoeuvre to finish later than the
    // designated end time.  It will simply cause the slew to have a higher
    // slope and thus a more sudden change.  This implementation works under
    // the assumption that being at the desired location at the designated time
    // is more important than performing the slew at the slope implied by the
    // start time, end time, and desired location specified.
    //
    // An alternate implementation (or perhaps an implementation for a different
    // subclass of Manoeuvre) would be to calculate the slope based upon the
    // desired start time.  This would mean that the slope would remain the
    // same regardless of when the manoeuvre is actually started and thus the
    // point at which the goal value is reached would adjust earlier or later
    // in time as the actual start time is adjusted earlier or later in time.
    //
    // The doContinue() method in this class is written with the assumption
    // that if it is called at any point later than the specified end time
    // of this manoeuvre, it should ignore the linear change of value
    // formula and simply establish the new set point to be the goal value.
    // This means that changing the implementation of the slope calculation
    // to the alternate implementation described above would require a
    // rewrite of the doContinue() method.
    slope = (att_end - att_start) / (tf - t0);

    Manoeuvre::initialize();
}

// -------------------------------------------------------------------
Real AttitudeSlew::calculateNextValue()
{
    AocsTime curTime = getTime();
    if (curTime >= tf)
    {
        return att_end;
    }
    else
    {
        return att_start + ((getTime() - t0) * slope);
    }
}

// -------------------------------------------------------------------
bool AttitudeSlew::isOkayAsNewValue(Real newValue)
{
    Real curValue = itemToSet.get();
    Real absDiff = fabs(newValue - curValue);
    if (absDiff > allowedDifference) return false;

    return true;
}

// -------------------------------------------------------------------
bool AttitudeSlew::canContinue()
{
    Real nextValue = calculateNextValue();
    if (!isOkayAsNewValue(nextValue)) return false;

    return Manoeuvre::canContinue();
}

// -------------------------------------------------------------------
void AttitudeSlew::doContinue()
{
    // determine what the next value for the
    // the itemToSet should be
    Real nextValue = calculateNextValue();

    // if the next value is not okay, abort
    if (!isOkayAsNewValue(nextValue))
    {
        // unnatural termination
        abort();
    }

    // set the output to the appropriate next value
    itemToSet.set(nextValue);

    // if we are beyond the end time of this manoeuvre,
    // terminate naturally
    if (getTime() >= tf)
    {
        currentState = finished;
        notifyMonitors(currentState);
    }

    return;
}

// -------------------------------------------------------------------
int AttitudeSlew::getTelemetryImageLength()
{
    return (tmImageLength[getTelemetryFormat()]
                                +Manoeuvre::getTelemetryImageLength());
}

// -------------------------------------------------------------------
void AttitudeSlew::writeToTelemetry(TelemetryStream* tmStream)
{
    Manoeuvre::writeToTelemetry(tmStream);     // write super object's state

    // TBD : not implemented in framework prototype
}
