// TestCaseManoeuvre_4.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseManoeuvre_4.h"
#include "ManoeuvreManagementTestSuite.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/AttitudeSlew.h"

// -------------------------------------------------------------------
TestCaseManoeuvre_4::TestCaseManoeuvre_4():
	TestCaseGenericSetUp(ID_MANOEUVREMANAGER*10+4,
									"TestCaseManoeuvre_4") {
		return;
}

// -------------------------------------------------------------------

void TestCaseManoeuvre_4::runTestCase() {

    // Create and configure a ManoeuvreManager
    ManoeuvreManager mnManager;

    // Initialize the ManoeuvreManager's manoeuvre list.
    mnManager.initializeManoeuvreList(20);

    // Get access to the current clock
    DummyAocsClock* myClock = (DummyAocsClock*) AocsObject::getAocsClock();
    AocsTime currentTime = myClock->getTime();
    int clockPeriod = myClock->getClockPeriod();

    // Create some AttitudeSlew manoeuvres

    // ------------------------------------------------------------------------------
    // slew0: an AttitudeSlew manoeuvre which should start pretty soon
    AocsTime slew0StartTime = currentTime + clockPeriod;
    AocsTime slew0ActualStartTime = slew0StartTime;
    AocsTime slew0EndTime = slew0StartTime + 20 * clockPeriod;
    AocsTime slew0OutputTime;
    Real slew0Value = 170.3;
    DataItemWrite slew0DataItem(&slew0OutputTime, &slew0Value);

    AttitudeSlew slew0(slew0StartTime, slew0EndTime, 15.0, slew0DataItem, 100.0);
    slew0.setMonitorListSize(2);
    mnManager.addManoeuvre(&slew0);

    // Testing information about slew0
    //
    // Once this manoeuvre is initialized, it should have the following
    // attribute values
    //
    //   t0 = currentTime + clockPeriod
    //      = 1030 in the current implementation
    //        If previous tests change, so may this value
    //   tf = t0 + 20 * clockPeriod
    //      = 1230 in the current implementation
    //        The current clock cycle is 10.
    //        If previous tests change, so may this value.
    //   att_start = 170.30
    //   att_end   = 15.0
    //   slope = (15.0 - 170.3) / (1230 - 1030)
    //         = -0.7765
    //
    // Therefore, change in the value for each clock cycle should be
    //      (clock cycle * slope) = -7.765


    // ------------------------------------------------------------------------------
    //
    // slew1: an AttitudeSlew manoeuvre which should have started a while ago
    // Note: This will not work if the current time is less than two clock periods
    //       This is because AocsTime values are currently unsigned and therefore
    //       cannot be negative.  The subtraction in the following statement thus
    //       will cause the start time for this slew to be a very high positive
    //       number if the current time is less than two clock periods.
    AocsTime slew1StartTime = currentTime - (2 * clockPeriod);
    AocsTime slew1ActualStartTime = currentTime;
    AocsTime slew1EndTime = slew1StartTime + 10 * clockPeriod;
    AocsTime slew1OutputTime;
    Real slew1Value = -60.0;
    DataItemWrite slew1DataItem(&slew1OutputTime, &slew1Value);

    AttitudeSlew slew1(slew1StartTime, slew1EndTime, 27.76, slew1DataItem, 100.0);
    slew1.setMonitorListSize(2);
    mnManager.addManoeuvre(&slew1);

    // Testing information about slew1
    //
    // Once this manoeuvre is initialized, it should have the following
    // attribute values
    //
    //   t0 = currentTime - 2 * clockPeriod
    //      = 1000 in the current implementation
    //        If previous tests change, so may this value
    //   tf = t0 + 10 * clockPeriod
    //      = 1100 in the current implementation
    //        The current clock cycle is 10.
    //        If previous tests change, so may this value.
    //   att_start = -60.0
    //   att_end   = 27.76
    //   slope = (27.76 - (-60.0) / (1100 - 1020)  (Note 1020 is actual start time
    //                                              not requested start time.)
    //         = 1.097
    //
    // Therefore, change in the value for each clock cycle should be
    //      (clock cycle * slope) = 10.97

    // move the clock forward and run the manoeuvre manager a "few" times

    bool status;
    for (int runCount=0; runCount<100; runCount++) {
        // run the manoeuvre manager
        mnManager.run(0);

        status = checkSlew(myClock->getTime(), slew0ActualStartTime, slew0EndTime,
                           slew0Value, 170.3, 15.0,
                           -7.765, clockPeriod);
        if (status != true) 
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        status = checkSlew(myClock->getTime(), slew1ActualStartTime, slew1EndTime,
                           slew1Value, -60.0, 27.76,
                           10.97, clockPeriod);
        if (status != true) 
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // advance the time and keep on going
        myClock->run(0);
    }

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


