// ObjectList.cpp
//
// Version	1.0
// Date		10.03.00
// Author	A. Pasetti/T. Brown
//
// -------------------------------------------------------------------

// Include framework header files
#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../OperatingSystemObjects/AocsClock.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../AocsEvent/Include.h"
#include "../ObjectList/Include.h"

// -------------------------------------------------------------------
// Define static variables
int ObjectList::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// NB The length of the debugTm telemetry image is set only after 
// allocateMemory is called because it depends on the list size
 
ObjectList::ObjectList()
{
	setClassId(ID_OBJECTLIST);	

	size =0;
	list = NULL;
	listWasInitialized = false;

	tmImageLength[shortTm]  = 0;	// not implemented in fw prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;
	
	localReset();						

	localResetConfiguration();			
}

// -------------------------------------------------------------------
// Reset the list by resetting any on-going iteration

void ObjectList::localReset()
{
	// reset on-going iteration (if any)
	endOfIteration = false;
	iterationCounter = 0;
}

// -------------------------------------------------------------------

void ObjectList::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------
// Reset the list configuration by clearing all list entries 

void ObjectList::localResetConfiguration()
{
	// reset list entries
	for (int i=0; i<size; i++)
		list[i]=NULL;

	numberOfItems=0;
}

// -------------------------------------------------------------------

void ObjectList::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ObjectList::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (!listWasInitialized)
		return false;

	return true;
}

// -------------------------------------------------------------------
// This method allocates the memory for the list of object references.
// The method also defines the length of the telemetry image in debug mode
// when all the references are sent to the telemetry stream.
// This method should be called during the initialization phase.
// Attempts to call the method more than once will result in a configuration
// error.

void ObjectList::allocateMemory(int listSize)
{
	if (listWasInitialized)			// method was already called!
	{	reportConfigurationError(this,MEMORY_ALREADY_ALLOCATED,this);
		return;
	}

	listWasInitialized = true;

	size = listSize;

	list = new RootObject*[size];

	// set list entries to NULL
	for (int i=0; i<size; i++)
		list[i]=NULL;

	numberOfItems=0;

	localReset();
}

// -------------------------------------------------------------------

int ObjectList::addObject(RootObject* newObject)
{
	// Scan list until a free slot is found
	int i=0;
	while ((list[i]!=NULL) && (i<size))
		i++;

	if (i==size)		// list is already full
	{	reportConfigurationError(this,LIST_IS_FULL,this);
		return -1;
	}
	else					// insert object in free slot
	{	list[i]=newObject;
		numberOfItems++;
		return i;
	}
}

// -------------------------------------------------------------------

int ObjectList::removeObject(RootObject* object)
{
	int location = locateObjectInList(object);
	if (location==-1)  // object was not present in the list
	{
		reportConfigurationError(this, OBJECT_NOT_IN_LIST, this);
		return -1;
	}	
	else
	{
		list[location]=NULL;
		numberOfItems--;
		return location;
	}
}


// -------------------------------------------------------------------
// return the specified object's location in the list array
// or -1 if the object is not in the list
int ObjectList::locateObjectInList(RootObject* object)
{
	// Scan list until object is found
	int i=0;
	while ((list[i]!=object) && (i<size))
		i++;

	if (i==size)		// object was not present in the list
	{
		return -1;
	}
	else
	{
		return i;
	}
}

// -------------------------------------------------------------------

bool ObjectList::isObjectInList(RootObject* object)
{
	int location=locateObjectInList(object);
	if (location==-1)  // object was not present in the list
	{
		return false;
	}	
	else
	{
		return true;
	}
}

// -------------------------------------------------------------------

RootObject* ObjectList::first()
{
	endOfIteration = false;
	iterationCounter = 0;

	while ( (iterationCounter<size) && (list[iterationCounter]==NULL) )
		iterationCounter++;

	if (iterationCounter==size)
	{	endOfIteration=true;
		return NULL;
	}
	else
		return list[iterationCounter];
}

// -------------------------------------------------------------------
// Attempts to call next() when the end of the list has already been
// reached result in a null pointer being returned

RootObject* ObjectList::next()
{
	if (isLast())
		return NULL;

	iterationCounter++;

	while ( (iterationCounter<size) && (list[iterationCounter]==NULL) )
		iterationCounter++;

	if (iterationCounter==size)
	{	endOfIteration=true;
		return NULL;
	}
	else
		return list[iterationCounter];
}

// -------------------------------------------------------------------

bool ObjectList::isLast()
{
	return (endOfIteration);
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ObjectList::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void ObjectList::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		

	// Not implemented in framework prototype
}