// ObjectList.h
//
// Version	1.0
// Date		09.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Base class for object lists
// -------------------------------------------------------------------

#ifndef ObjectListH
#define ObjectListH

// -------------------------------------------------------------------

#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../BasicObjects/AocsObject.h"

// -------------------------------------------------------------------

class ObjectList: public AocsObject {

	static int tmImageLength[NTMFORMATS];

	int size;				// number of references in the object list
	int numberOfItems;		// number if items in the object list
	int iterationCounter;	// counter used when iterating through list items
	bool endOfIteration;	// true when list iteration has been completed

	RootObject** list;

	// True after memory allocation methods has been called
	bool listWasInitialized;

	// Resetting the list clears all its entries
	void localReset();

	// Resetting the list configuration clears the recovery actions
	void localResetConfiguration();

	// locate a specified object in the list
	int locateObjectInList(RootObject* object);

public :

	ObjectList();
	
	// Configuration methods to allocate the memory for the list. This
	// method should only be called once when the list is configured.
	// Attempts to call it a second time will result in a configuration
	// error being generated.
	void allocateMemory(int listSize);

	// Methods to add and remove items from the list.
	// There is no guarantee as to where in the list new objects are 
	// added in the list.
	// Attempts to add objects to a list that is full will result in a
	// configuration error being generated. Likewise for attempts to retrieve 
	// non-existent objects from the list.
	// The return value is the position in the list where the object was
	// added/removed. A return value of -1 indicates that the object 
	// could not be added/removed (list was full or object was not found
	// in the list)
	int addObject(RootObject* newObject);
	int removeObject(RootObject* object);
	
	// Check whether the list is full
	bool isListFull()
		{ return (numberOfItems==size);}

	// Check whether an object is in the list
	bool isObjectInList(RootObject* object);

	// Methods to iterate through the objects in the list.
	// The iteration methods return all the non-null references but
	// there is no guarantee as to the order in which they are returned.
	RootObject* first();
	RootObject* next();
	bool isLast();

	// During an iteration, return the index corresponding to the last 
	// list object returned by the iteration operations
	int getPosition()
		{ return iterationCounter; }
	
	// Return and remove elements by position
	RootObject* getObjectByPosition(int i)
		{ return list[i]; }
	void removeObjectByPosition(int i)
		{ list[i] = 0; }

	// Resetting the list clears all its entries
	virtual void reset();

	// Resetting list configuration clears all entries in the list
	virtual void resetConfiguration();

	// List is configured if its memory allocation method was called
	virtual bool isConfigured();

	// Telemetry methods 
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward accessor methods
	int getListSize()
		{ return size; }

	int getNumberOfItems()
		{ return numberOfItems; }
};

// -------------------------------------------------------------------
#endif
