// TestCaseReconfiguration_1.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseReconfiguration_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/ReconfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../AocsEvent/ReconfigurationEventRepository.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ReconfigurationManagement/ReconfigurerHelper.h"

// -------------------------------------------------------------------
TestCaseReconfiguration_1::TestCaseReconfiguration_1():
	TestCaseGenericSetUp(ID_BASICUNITRECONFIGURER*10+1,
									"TestCaseReconfiguration_1") {
		return;
}

// -------------------------------------------------------------------

void TestCaseReconfiguration_1::runTestCase() {

	// Retrieve the number of various types of relevant events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();
	int ren = ReconfigurerHelper::getReconfigurationEventRepository()->getCounter();
	int hen = ReconfigurerHelper::getChangeEventRepository()->getCounter();
	
	// Create and configure a reconfigurer helper
	// Set the rank to decrease from configuration 0 to configuration 3
	AocsObject ao;
	NullRecoveryAction nra;
	ReconfigurerHelper rh(4,2,3,&ao);
	rh.setReconfigurationErrorRecoveryAction(&nra);
	rh.rankConfiguration(0,9);
	rh.rankConfiguration(1,8);
	rh.rankConfiguration(2,7);
	rh.rankConfiguration(3,6);

	// Check that the reconfigurer helper is configured
	if (!rh.isConfigured())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Check that the initial configuration is the default configuration 
	// (passed as parameter to the constructor)
	if (rh.getConfiguration() != 3)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Command a reconfiguration and verify the correct configuration
	// and the generation of the relevant events
	if (!rh.canReconfigure())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	rh.reconfigure();
	if (rh.getConfiguration() != 0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (rh.isConfigurationHealthy(3))
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Disable reconfigurations and verify that calls to reconfigure() have
	// no effect
	rh.disableReconfiguration();
	if (rh.canReconfigure())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	rh.reconfigure();
	if (rh.getConfiguration() != 0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	rh.enableReconfiguration();

	// Disable configuration 1 and check that a command to reconfigure has
	// the correct effects
	rh.markConfiguration(1,false);
	if (!rh.canReconfigure())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	rh.reconfigure();
	if (rh.getConfiguration() != 2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Check the correct value of the configuration state
	if (rh.getConfigurationState().getMemoryPad() != 0x00040002)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// All configurations should be exhausted 
	if (rh.canReconfigure())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	rh.reconfigure();
	if (rh.getConfiguration() != 2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+3)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Attempt to perform an illegal operation and very generation of 
	// configuration error event
	rh.markConfiguration(10,true);
	if (AocsObject::getConfigurationEventRepository()->getCounter() != cen+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Reset the reconfigurer helper and check correct new state
	rh.reset();
	if (rh.getConfigurationState().getMemoryPad() != 0x000F0003)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+4)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// Check that no unexpected events were generated
	if (AocsObject::getConfigurationEventRepository()->getCounter() != cen+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}
	if (ReconfigurerHelper::getChangeEventRepository()->getCounter() != hen)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 1 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}



