// TestCaseReconfiguration_2.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseReconfiguration_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/ReconfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../AocsEvent/ReconfigurationEventRepository.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/FssPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ReconfigurationManagement/BasicUnitReconfigurer.h"

// -------------------------------------------------------------------
TestCaseReconfiguration_2::TestCaseReconfiguration_2():
	TestCaseGenericSetUp(ID_BASICUNITRECONFIGURER*10+2,
									"TestCaseReconfiguration_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseReconfiguration_2::runTestCase() {

	// Retrieve the number of various types of relevant events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();
	int ren = ReconfigurerHelper::getReconfigurationEventRepository()->getCounter();
	int hen = ReconfigurerHelper::getChangeEventRepository()->getCounter();
	
	// Create and configure a MACS TC object
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Default recovery action for various objects
	NullRecoveryAction nra;

	// Create and configure three fine sun sensors
	FssPrototype fss1(1,11,12);
	FssPrototype fss2(2,11,12);
	FssPrototype fss3(3,11,12);

	Real fss_x, fss_y;
	AocsTime fss_x_tag, fss_y_tag;
	DataItemWrite fss_x_diw(&fss_x_tag, &fss_x);
	DataItemWrite fss_y_diw(&fss_y_tag, &fss_y);

	RawBusDataConverterIn convIn(2,0.1);

	fss1.setAocsUnitHardware(&macsTcCont);
	fss1.setFunctionalInConverter(&convIn);
	fss1.setFunctionalInLink(fss_x_diw,0);
	fss1.setFunctionalInLink(fss_y_diw,1);
	fss1.setHwFailureRecoveryAction(&nra);

	fss2.setAocsUnitHardware(&macsTcCont);
	fss2.setFunctionalInConverter(&convIn);
	fss2.setFunctionalInLink(fss_x_diw,0);
	fss2.setFunctionalInLink(fss_y_diw,1);
	fss2.setHwFailureRecoveryAction(&nra);

	fss3.setAocsUnitHardware(&macsTcCont);
	fss3.setFunctionalInConverter(&convIn);
	fss3.setFunctionalInLink(fss_x_diw,0);
	fss3.setFunctionalInLink(fss_y_diw,1);
	fss3.setHwFailureRecoveryAction(&nra);

	// Create and configure a basic unit reconfigurer
	BasicUnitReconfigurer bur(3,1,1);
	bur.setReconfigurationErrorRecoveryAction(&nra);
	bur.setUnit(&fss1,0);
	bur.setUnit(&fss2,1);
	bur.setUnit(&fss3,2);

	// Check that the unit reconfigurer is correctly configured
	if (!bur.isConfigured())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	if (bur.getConfiguration()!=1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Command a reconfiguration and check result. 
	if (!bur.canReconfigure())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	bur.reconfigure();
	if (bur.getConfiguration()!=0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (bur.isConfigurationHealthy(1))
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Unit reconfigurations should trigger an initialization of the new unit 
	// and should therefore result in an RC MACS instruction being put on the bus.
	// Since there is no MACS bus, a MACS failure should be generated
	if (macsInstr!=0x010E)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Force a configuration and check result
	bur.setConfiguration(2);
	if (bur.canReconfigure())		// already 2 reconfigurations done
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (bur.getConfiguration()!=2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (bur.isConfigurationHealthy(0))
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (ReconfigurerHelper::getReconfigurationEventRepository()->getCounter() != ren+2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Check generation of failure corresponding to MACS initialization 
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+2)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Command data acquisition and check correct instruction is in MACS register
	bur.acquireFunctionalData();
	if (macsInstr!=-29666)			//  -29666 = 0x8C1E
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+4)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// Simulate acquisition of x and y channel data
	fss3.setFcHwInpBuff(100,0);
	fss3.setFcHwInpBuff(200,1);
	bur.refreshFunctionalIn();
	if (fss_x != -10.0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}
	if (fss_y != -20.0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 2 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}



