// TestCaseReconfiguration_4.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseReconfiguration_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/NVector.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/ReconfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../AocsEvent/ReconfigurationEventRepository.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/RwPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ReconfigurationManagement/RwSet.h"

// -------------------------------------------------------------------
TestCaseReconfiguration_4::TestCaseReconfiguration_4():
	TestCaseGenericSetUp(ID_BASICUNITRECONFIGURER*10+4,
									"TestCaseReconfiguration_4") {
		return;
}

// -------------------------------------------------------------------
// Check the rw reconfiguration manager.
// The rw reconfiguration manager manages a set of four wheels.

void TestCaseReconfiguration_4::runTestCase() {

	// Retrieve the number of various types of relevant events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();
	int ren = ReconfigurerHelper::getReconfigurationEventRepository()->getCounter();
	int hen = ReconfigurerHelper::getChangeEventRepository()->getCounter();
	
	// Create and configure a MACS TC object
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Default recovery action for various objects
	NullRecoveryAction nra;

	// Create and configure vectors holding the wheel speed, the wheel 
	// momenta, and the wheel torque requests (both along wheel axes 
	// and in sc reference frame). A dummy owner is assigned to their
	// data items 
	NVector rwSpeed(4);
	NVector rwAngMom(3);
	NVector rwTorque(4);
	NVector scTorque(3);

	// Create and configure four reaction wheels
	RawBusDataConverterIn convIn1(1,0.1);
	RawBusDataConverterOut convOut1(1,10);
	RawBusDataConverterIn convIn2(1,0.1);
	RawBusDataConverterOut convOut2(1,10);
	RawBusDataConverterIn convIn3(1,0.1);
	RawBusDataConverterOut convOut3(1,10);
	RawBusDataConverterIn convIn4(1,0.1);
	RawBusDataConverterOut convOut4(1,10);
	RawBusDataConverterIn* rbdci[]={&convIn1, &convIn2, &convIn3, &convIn4};
	RawBusDataConverterOut* rbdco[]={&convOut1, &convOut2, &convOut3, &convOut4};
	RwPrototype rw1(1,11,12);
	RwPrototype rw2(2,11,12);
	RwPrototype rw3(3,11,12);
	RwPrototype rw4(4,11,12);
	RwPrototype* rw[]={&rw1, &rw2, &rw3, &rw4};
	for (int i=0; i<4; i++)
	{	rw[i]->setAocsUnitHardware(&macsTcCont);
		rw[i]->setFunctionalInConverter(rbdci[i]);
		rw[i]->setFunctionalInLink(rwSpeed.getDataItemWrite(i),0);
		rw[i]->setFunctionalOutConverter(rbdco[i]);
		rw[i]->setFunctionalOutLink(rwTorque.getDataItemWrite(i),0);
		rw[i]->setHwFailureRecoveryAction(&nra);
	}

	// Create the wheel reconfiguration manager
	Real rwMatrix[3][4];
	rwMatrix[0][0] = 1;		// direction of RW1
	rwMatrix[1][0] = 0;
	rwMatrix[2][0] = 0;
	rwMatrix[0][1] = 0;		// direction of RW2
	rwMatrix[1][1] = 1;
	rwMatrix[2][1] = 0;
	rwMatrix[0][2] = 0;		// direction of RW3
	rwMatrix[1][2] = 0;
	rwMatrix[2][2] = 1;
	rwMatrix[0][3] = 0.58;	// direction of RW4
	rwMatrix[1][3] = 0.58;
	rwMatrix[2][3] = 0.58;

	Real rwInertia = 0.1;

	RwSet rwSet(0,rwMatrix,rwInertia);

	for (i=0; i<4; i++)
		rwSet.setRw(rw[i],i);

	rwSet.setFunctionalOutLink(&scTorque);
	rwSet.setFunctionalInLink(&rwAngMom);

	rwSet.setReconfigurationErrorRecoveryAction(&nra);

	// Check that the rw reconfiguration manager is properly configured
	if (!rwSet.isConfigured())
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}

	// Load a spacecraft torque and check transmission to the rw units
	for (i=0; i<3; i++)
		scTorque.getDataItemWrite(i).set(i);	
	rwSet.refreshFunctionalOut();
	rwSet.sendFunctionalData();
	if (rw[0]->getFcHwOutBuff(0) != 0)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if (rw[1]->getFcHwOutBuff(0) != 32768+10)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if (rw[2]->getFcHwOutBuff(0) != 32768+20)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+3)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}

	// Load wheel speed values and check acquisition
	for (i=0; i<4; i++)
		rw[i]->setFcHwInpBuff(32768+100*i,0);
	rwSet.acquireFunctionalData();
	rwSet.refreshFunctionalIn();
	for (i=0; i<3; i++)
		if (rwAngMom.getDataItemRead(i).get() != i)
		{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
			return;
		}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+6)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}

	// Force a reconfiguration to use wheels 2 to 4
	rwSet.setConfiguration(0);

	// Advance time
	aocsClock.run(0);

	// Load wheel speed values and check acquisition
	for (i=0; i<4; i++)
		rw[i]->setFcHwInpBuff(32768+1000*i,0);
	rwSet.acquireFunctionalData();
	rwSet.refreshFunctionalIn();
	if ((rwAngMom.getDataItemRead(0).get() < rwInertia*100*3*0.58-0.00001) ||
		(rwAngMom.getDataItemRead(0).get() > rwInertia*100*3*0.58+0.00001)) 
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if ((rwAngMom.getDataItemRead(1).get() < rwInertia*(100*1+100*3*0.58)-0.00001) ||
		(rwAngMom.getDataItemRead(1).get() > rwInertia*(100*1+100*3*0.58)+0.00001))
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if ((rwAngMom.getDataItemRead(2).get() < rwInertia*(100*2+100*3*0.58)-0.00001) ||
		(rwAngMom.getDataItemRead(2).get() > rwInertia*(100*2+100*3*0.58)+0.00001))
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+9+1)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 4 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}





bool reconfigurationManagementTest_4(DummyAocsClock* aocsClock)
{
		return false;

	// If this point is reached, the test was successful
	return true;
}








