// ReconfigurerHelper.cpp
//
// Version	1.0
// Date		27.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../ReconfigurationManagement/ReconfigurerHelper.h"
#include "../AocsUnit/AocsUnit.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"	
#include "../AocsEvent/ReconfigurationEventRepository.h"	
#include "../AocsEvent/ChangeEventRepository.h"	
#include "../SystemManagement/SystemManager.h"
#include "../ObjectMonitoring/ChangeObject.h"

// -------------------------------------------------------------------
// Define static variables
int ReconfigurerHelper::tmImageLength[NTMFORMATS];
ReconfigurationEventRepository*	ReconfigurerHelper::reconfigurationEventRepository=NULL;
ChangeEventRepository*			ReconfigurerHelper::changeEventRepository=NULL;

// -------------------------------------------------------------------

ReconfigurerHelper::ReconfigurerHelper(int numberOfConfig, 
										int numberOfMon,
										 ConfigurationId defaultConfig,
										  AocsObject* reconfigManager)
{
	setClassId(ID_RECONFIGURERHELPER);				

	tmImageLength[shortTm]  = 0;		// Not implemented in fw prototype
	tmImageLength[normalTm] = 0;	
	tmImageLength[longTm]   = tmImageLength[normalTm];
	tmImageLength[debugTm]  = tmImageLength[longTm];

	numberOfConfigurations = numberOfConfig;
	numberOfMonitors = numberOfMon; 
	defaultConfiguration = defaultConfig;
	activeConfiguration = defaultConfiguration;
	reconfigurationManager = reconfigManager;

	// Allocate memory
	healthy = new bool[numberOfConfig];
	enabled = new bool[numberOfConfig];
	rank = new ConfigurationRank[numberOfConfig];

	monitorList.allocateMemory(numberOfMonitors);
	changeObjectList.allocateMemory(numberOfMonitors);

	localReset();
	localResetConfiguration();

	AocsObject::getSystemManager()->addReconfigurable(this);
}

// -------------------------------------------------------------------

void ReconfigurerHelper::localReset()
{
	for (int i=0; i<numberOfConfigurations; i++)
		enabled[i] = true;

	for (i=0; i<numberOfConfigurations; i++)
		healthy[i] = true;

	for (i=0; i<numberOfConfigurations; i++)
		rank[i] = 1;

	reconfigurationEnabled = true;

	monitorList.reset();
	changeObjectList.reset();

	setConfiguration(defaultConfiguration);

	// The call to setConfiguration marks the starting configuration
	// as 'unhealthy'. Hence, make all configurations healthy again
	for (i=0; i<numberOfConfigurations; i++)
		healthy[i] = true;

	// After changing the health status, the system manager must be
	// notified to allow it to update its copy of the configuration
	// state
	AocsObject::getSystemManager()->configurationStateChange(this);
}

// -------------------------------------------------------------------

void ReconfigurerHelper::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void ReconfigurerHelper::localResetConfiguration()
{
	reconfigErrRecAct = NULL;
	monitorList.resetConfiguration();
	changeObjectList.resetConfiguration();
}

// -------------------------------------------------------------------

void ReconfigurerHelper::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void ReconfigurerHelper::resetStaticConfiguration()
{
	reconfigurationEventRepository = NULL;
}

// -------------------------------------------------------------------

bool ReconfigurerHelper::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (!monitorList.isConfigured())
		return false;

	if (!changeObjectList.isConfigured())
		return false;

	if (reconfigurationEventRepository==NULL)
		return false;

	if (changeEventRepository==NULL)
		return false;

	if (reconfigErrRecAct==NULL)
		return false;

	return true;
}

// -------------------------------------------------------------------

void ReconfigurerHelper::markConfiguration(ConfigurationId n, bool mark)
{ 
	if (isConfigurationLegal(n)) 
		healthy[n]=mark; 

	// A change in health status is a change in configuration state
	// hence, notify the system manager
	AocsObject::getSystemManager()->configurationStateChange(this);
}

// -------------------------------------------------------------------
// The configuration state only consists of the health status of each
// configuration and of the currently active configuration.
// It is assumed that there are at most 16 different configurations
//
// The configuration state is encapsulated in an integer with the
// following format:
//	 bits 0 -16: bit i is set to 1 if the i-th configuration is healthy
//	 bits 17-28: unused
//	 bits 29-31: currently active configuration

void ReconfigurerHelper::setConfigurationState(ConfigurationState s)
{
	int state = s.getMemoryPad();
	int mask = 1;

	activeConfiguration = state & 0x0000000F;
	state = state>>16;

	for (int i=0; i<numberOfConfigurations; i++)
	{	if (state & mask)
			healthy[i] = true;
		else
			healthy[i] = false;
		mask = mask<<1;
	}
}

// -------------------------------------------------------------------
// The configuration state only consists of the health status of each
// configuration and of the currently active configuration.
// It is assumed that there are at most 16 different configurations
//
// The configuration state is encapsulated in an integer with the
// following format:
//	 bits 0 -16: bit i is set to 1 if the i-th configuration is healthy
//	 bits 17-28: unused
//	 bits 29-31: currently active configuration

ConfigurationState ReconfigurerHelper::getConfigurationState()
{
	int state = 0;
	int mask = 1;

	for (int i=0; i<numberOfConfigurations; i++)
	{	if (healthy[i])
			state = state | mask;
		mask = mask<<1;
	}

	state = state<<16;

	state = state + activeConfiguration;

	return ConfigurationState(getInstanceId(), state);
}

// -------------------------------------------------------------------

void ReconfigurerHelper::addMonitor(PropertyMonitor* m, ChangeObject* c)
{
	monitorList.addObject(m);
	changeObjectList.addObject(c);
}

// -------------------------------------------------------------------

void ReconfigurerHelper::removeMonitor(PropertyMonitor* m)
{
	int position=monitorList.addObject(m);
	if (position>0)
		changeObjectList.removeObjectByPosition(position);
}

// -------------------------------------------------------------------

bool ReconfigurerHelper::isConfigurationLegal(ConfigurationId n)
{
	if ( (n<numberOfConfigurations) && (n>=0))
		return true;

	reportConfigurationError(this,ILLEGAL_CONFIGURATION,this);
	return false;
}

// -------------------------------------------------------------------

bool ReconfigurerHelper::canReconfigure()
{
	if (!reconfigurationEnabled)
		return false;

	for (int i=0; i<numberOfConfigurations; i++)
		if ( healthy[i] && enabled[i] && (i!=activeConfiguration) )
			return true;

	return false;
}

// -------------------------------------------------------------------

void ReconfigurerHelper::reconfigure()
{
	if (!reconfigurationEnabled)
		return;

	// Identify highest-ranking free configuration
	int maxRank=-1;
	ConfigurationId maxRankConfig=0;
	for (int i=0; i<numberOfConfigurations; i++)
		if ( healthy[i] && enabled[i] && (i!=activeConfiguration) )
			if (maxRank<rank[i])
			{	maxRank=rank[i];
				maxRankConfig=i;
			}

	if (maxRank>-1)
		setConfiguration(maxRankConfig);
	else
	{	reconfigurationEventRepository->create(this,
												RECONFIGURATION_FAILURE,
											     0,0,getInstanceId());
		reportFailure(this,NO_MORE_CONFIGURATION,this,reconfigErrRecAct);
	}
}
	
// -------------------------------------------------------------------

void ReconfigurerHelper::setConfiguration(ConfigurationId newConfiguration)
{
	// Check that target configuration is legal
	if (!isConfigurationLegal(newConfiguration))
		return;
	
	// Do change only if target config. is different from current config.
	if (newConfiguration == activeConfiguration)
		return;

	// Create event describing configuration change
	reconfigurationEventRepository->create(this,
											RECONFIGURATION,
											 activeConfiguration,
											  newConfiguration,
											   reconfigurationManager->getInstanceId());

	// Mark the current configuration as unhealthy and then change to
	// the new configuration
	healthy[activeConfiguration]=false;
	activeConfiguration = newConfiguration;

	// Notify all configuration monitors
	PropertyMonitor* monitor=monitorList.first(); 
	ChangeObject* changeObject=changeObjectList.first();
	while (!monitorList.isLast())
	{	if (changeObject->checkValue(newConfiguration))
		{	changeEventRepository->create(this,
										   MONITORED_PROPERTY_CHANGE,
											getConfigurationPropertyId(),
											 changeObject,
											  getConfigurationProperty().get());
			monitor->propertyChange(this,getConfigurationProperty(),changeObject);
		}
		monitor=monitorList.next();
		changeObject=changeObjectList.next();
	}

	// Notify the system manager of the change in configuration state
	AocsObject::getSystemManager()->configurationStateChange(this);
}

// -------------------------------------------------------------------

bool ReconfigurerHelper::isConfigurationHealthy(ConfigurationId n)
{
	if (isConfigurationLegal(n)) 
		return healthy[n]; 
	else
		return false;
}

// -------------------------------------------------------------------

ConfigurationRank ReconfigurerHelper::getConfigurationRank(ConfigurationId n)
{
	if (isConfigurationLegal(n)) 
		return rank[n]; 
	else
		return NULL;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ReconfigurerHelper::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void ReconfigurerHelper::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
