// ReconfigurerHelper.h
//
// Version	1.0
// Date		27.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Helper class to manage reconfigurations. It encapsulates all
// reconfiguration operations that are common to all reconfiguration 
// managers.
// Concrete reconfiguration managers delegate implementation of the
// operations they inherit from Configurable to the reconfigurer 
// helper.
// -------------------------------------------------------------------

#ifndef ReconfigurerHelperH
#define ReconfigurerHelperH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../ReconfigurationManagement/Reconfigurable.h"
#include "../ObjectList/ObjectListTemplate.h"

// -------------------------------------------------------------------

class ReconfigurerHelper : public AocsObject, 
								public Reconfigurable {

	static int tmImageLength[NTMFORMATS];
	static ReconfigurationEventRepository* reconfigurationEventRepository;
	static ChangeEventRepository* changeEventRepository;

	// Number of configurations in the reconfiguration group
	ConfigurationId numberOfConfigurations;

	// Default configuration
	ConfigurationId defaultConfiguration;

	// healthy[i] contains health status of i-th configuration
	bool* healthy;

	// i-th configuration is enabled if enabled[i] is true
	bool* enabled;

	// rank[i] holds the rank of the i-th configuration
	ConfigurationRank* rank;

	// Reconfigurations are enabled if reconfigurationEnabled is true
	bool reconfigurationEnabled;

	// List of registered monitors (to be notified in case of configuration 
	// changes) and of their associated change objects
	int numberOfMonitors;
	ObjectListTemplate<PropertyMonitor> monitorList;
	ObjectListTemplate<ChangeObject> changeObjectList;

	// Recovery action associated to a failed reconfiguration request
	RecoveryAction* reconfigErrRecAct;

	// Current configuration indicator
	ConfigurationId activeConfiguration;

	// Configuration manager to which this helper is attached
	AocsObject* reconfigurationManager;

	// Local reset methods
	void localResetConfiguration();		
	void localReset();

	// Return true if argument is a valid configuration indicator
	bool isConfigurationLegal(ConfigurationId n);

public :

	ReconfigurerHelper(int numberOfConfigurations, 
						int numberOfMonitors,
						 ConfigurationId defaultConfiguration,
						  AocsObject* configurationManager);

	// Configuration property management
	virtual PropertyId getConfigurationPropertyId()
		{ return (PropertyId)getInstanceId(); }
	virtual Property getConfigurationProperty()
		{ return Property(&activeConfiguration,getConfigurationPropertyId()); }

	// Return current configuration
	virtual ConfigurationId getConfiguration()
		{ return activeConfiguration; }

	// Force the current configuration to n
	virtual void setConfiguration(ConfigurationId n);

	// Mark configuration n healthy (true) or unhealthy (false)
	virtual void markConfiguration(ConfigurationId n, bool mark);

	// Set the rank of configuration n to r
	virtual void rankConfiguration(ConfigurationId n, ConfigurationRank r)
		{ if (isConfigurationLegal(n)) rank[n]=r; }

	// Enable and disable a particular configuration
	virtual void enableConfiguration(ConfigurationId n)
		{ if (isConfigurationLegal(n)) enabled[n]=true; }
	virtual void disableConfiguration(ConfigurationId n)
		{ if (isConfigurationLegal(n)) enabled[n]=false; }

	// Enable and disable all reconfigurations
	virtual void enableReconfiguration()
		{ reconfigurationEnabled = true; }
	virtual void disableReconfiguration()
		{ reconfigurationEnabled = false; }

	// Command a reconfiguration
	virtual void reconfigure();

	// Return true if a reconfiguration is possible
	virtual bool canReconfigure();

	// Return true if configuration n is healthy
	virtual bool isConfigurationHealthy(ConfigurationId n);

	// Return the reconfiguration order
	virtual int getReconfigurationOrder()
		{ return numberOfConfigurations; }

	// Return the rank of configuration n
	virtual ConfigurationRank getConfigurationRank(ConfigurationId n);

	// Setter and getter methods for the reconfiguration state
	virtual void setConfigurationState(ConfigurationState s);
	virtual ConfigurationState getConfigurationState();

	// Add and remove a configuration monitor
	void addMonitor(PropertyMonitor* monitor, ChangeObject* changeObject);
	void removeMonitor(PropertyMonitor* monitor);

	// Setter and getter method for recovery action associated to 
	// failed reconfiguration request  
	void setReconfigurationErrorRecoveryAction(RecoveryAction* r)
		{ reconfigErrRecAct = r; }
	RecoveryAction* getReconfigurationErrorRecoveryAction()
		{ return reconfigErrRecAct; }

	// Setter and getter methods for the event repositories
	static void setReconfigurationEventRepository(ReconfigurationEventRepository* c)
		{ reconfigurationEventRepository = c; }
	static ReconfigurationEventRepository* getReconfigurationEventRepository()
		{ return reconfigurationEventRepository; }
	static void setChangeEventRepository(ChangeEventRepository* c)
		{ changeEventRepository = c; }
	static ChangeEventRepository* getChangeEventRepository()
		{ return changeEventRepository; }

	// Methods inherited from interface Configurable
	virtual void resetConfiguration();	
	virtual bool isConfigured();		

	// Methods inherited from interface Resettable
	virtual void reset();					

	// Reset the static part of the configuration state
	static void resetStaticConfiguration();

	// Methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

};

// -------------------------------------------------------------------
#endif