// RwSet.h
//
// Version	1.0
// Date		05.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Reconfiguration manager to manage a group of 4 identical reaction 
// wheels. 
// The reaction wheels are treated as instances of class AocsUnit.
// There are four possible configurations in this reconfiguration 
// group with the i-th configuration corresponding to the situation
// where the i-th reaction wheel is inactive. 
// The default configuration is configuration 4 (ie. wheels 1 to 3
// are active).
// The incoming data for the rwSet unit is the vector representing
// the wheel angular momenta in sc reference frame.
// The outgoing data for the rwSet unit is the vector representing
// the wheel torques in sc reference frame.
//
// -------------------------------------------------------------------

#ifndef RwSetH
#define RwSetH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../ReconfigurationManagement/Reconfigurable.h"
#include "../ReconfigurationManagement/ReconfigurerHelper.h"

// -------------------------------------------------------------------

class RwSet : public AocsObject, 
				public Reconfigurable,
				  public AocsUnitFunctional {

	static int tmImageLength[NTMFORMATS];

	ReconfigurerHelper* helper;

	// Array of reaction wheels to  be reconfigured
	AocsUnit* rw[4];

	// RwMatrix is a 3x4 matrix.
	// RwMatrix[*,i] is a unitary vector identifying the direction 
	// of the i-th reaction wheel in spacecrat reference frame
	Real rwMatrix[3][4];

	// RwMatrixInv[i] = int(rwMatrix_i * rwInertia)
	// where rwMatrix_i is the 3x3 matrix obtained from RwMatrix after 
	// deleting the i-th column and rwInertia is the reaction wheel
	// inertia (assumed identical for all four wheels).
	Real rwMatrixInv[4][3][3];

	// Reaction wheel inertia (identical for all four wheels)
	Real rwInertia;

	// Vector of current reaction wheel speed (the speed of the 
	// inactive wheel is set to 0)
	DataItemRead rwSpeed[4];

	// Vector of current reaction wheel torques
	DataItemWrite rwTorque[4];

	// Vector of torque requests in sc reference frame
	DataItemWrite scTorque[3];

	// Vector of wheel angular momenta in sc reference frame
	DataItemWrite scAngMomenta[3];

	// Local reset methods
	void localReset();
	void localResetConfiguration();

public :

	RwSet(int numberOfMonitors, 
			Real rwMatrix[3][4],
			  Real rwInertia);

	// Configuration property management
	virtual PropertyId getConfigurationPropertyId()
		{ return helper->getConfigurationPropertyId(); }
	virtual Property getConfigurationProperty()
		{ return helper->getConfigurationProperty(); }

	// Return current configuration
	virtual ConfigurationId getConfiguration()
		{ return helper->getConfiguration(); }

	// Force the current configuration to n
	virtual void setConfiguration(ConfigurationId n);

	// Mark configuration n healthy (true) or unhealthy (false)
	virtual void markConfiguration(ConfigurationId n, bool mark)
		{ helper->markConfiguration(n,mark); }

	// Set the rank of configuration n to r
	virtual void rankConfiguration(ConfigurationId n, ConfigurationRank r)
		{ return; }

	// Enable and disable a particular configuration
	virtual void enableConfiguration(ConfigurationId n)
		{ helper->enableConfiguration(n); }
	virtual void disableConfiguration(ConfigurationId n)
		{ helper->disableConfiguration(n); }

	// Enable and disable all reconfigurations
	virtual void enableReconfiguration()
		{ helper->enableReconfiguration(); }
	virtual void disableReconfiguration()
		{ helper->disableReconfiguration(); }

	// Command a reconfiguration
	virtual void reconfigure();

	// Return true if a reconfiguration is possible
	virtual bool canReconfigure()
		{ return helper->canReconfigure(); }

	// Return true if configuration n is healthy
	virtual bool isConfigurationHealthy(ConfigurationId n)
		{ return helper->isConfigurationHealthy(n); }

	// Return the reconfiguration order
	virtual int getReconfigurationOrder()
		{ return helper->getReconfigurationOrder(); }

	// Return the rank of configuration n
	virtual ConfigurationRank getConfigurationRank(ConfigurationId n)
		{ return helper->getConfigurationRank(n); }

	// Setter and getter methods for the reconfiguration state
	virtual void setConfigurationState(ConfigurationState s)
		{ helper->setConfigurationState(s); }
	virtual ConfigurationState getConfigurationState()
		{ return helper->getConfigurationState(); }

	// Add and remove a configuration monitor
	void addMonitor(PropertyMonitor* monitor, ChangeObject* changeObject)
		{ helper->addMonitor(monitor,changeObject); }
	void removeMonitor(PropertyMonitor* monitor)
		{ helper->removeMonitor(monitor); }

	// Setter and getter method for recovery action associated to 
	// failed reconfiguration request
	void setReconfigurationErrorRecoveryAction(RecoveryAction* r)
		{ helper->setReconfigurationErrorRecoveryAction(r);}
	RecoveryAction* getReconfigurationErrorRecoveryAction()
		{ return helper->getReconfigurationErrorRecoveryAction(); }

	// Methods inherited from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();

	// Methods inherited from interface Resettable
	virtual void reset();
		
	// Methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Methods inherited from AocsUnitFunctional interface 
	virtual void acquireFunctionalData();
	virtual void sendFunctionalData();
	virtual bool isAcquireTransactionFinished();
	virtual bool isSendTransactionFinished();
	virtual void refreshFunctionalIn();
	virtual void refreshFunctionalOut();
	virtual void synchronizeFunctional();
	virtual bool isSynchronizeFunctionalFinished();
	virtual void setFunctionalInLink(DataItemWrite d, int i);
	virtual void setFunctionalOutLink(DataItemWrite d, int i);
	virtual void setFunctionalInLink(AocsData* d);
	virtual void setFunctionalOutLink(AocsData* d);
	virtual DataItemWrite getFunctionalInLink(int i);
	virtual DataItemWrite getFunctionalOutLink(int i);

	virtual void setFunctionalInConverter(AbstractControlChannel* c)
		{ return; }
	virtual void setFunctionalOutConverter(AbstractControlChannel* c)
		{ return; }

	// Setter and getter methods for the individual rw units
	void setRw(AocsUnit* u, int n);

};

// -------------------------------------------------------------------
#endif