// AbstractControlChannel.h
//
// Version	1.0
// Date		16.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Abstract base class for all control channel classes
// -------------------------------------------------------------------

#ifndef AbstractControlChannelH
#define AbstractControlChannelH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsData/DataItemRead.h"

// -------------------------------------------------------------------

class AbstractControlChannel : public AocsObject {

	int n;			// number of inputs
	int m;			// number of outputs

	void localReset();
	void localResetConfiguration();

	int propagationToken;

	// Recovery action used when propagateTime is called with illegal target time
	RecoveryAction* recoveryAction;		

	// Reference to enclosing control channel
	AbstractControlChannel* enclosingControlChannel;

protected :

	AbstractControlChannel(); 

	// i-th element contains channel linked to i-th input of this channel
	AbstractControlChannel** ccLink; 

	// Time to which state was last propagated
	AocsTime lastPropagatedTime;	

	// Setter methods for input and output order
	void setInputOrder(int i)
		{ n = i; }
	void setOutputOrder(int i)
		{ m = i; }

public :

	virtual void propagate(AocsTime t)=0;

	virtual void hold()=0;
	virtual void release()=0;

	// link DataItemRead d to j-th input
	virtual void linkInput(DataItemRead d, int j)=0;

	// link j-th input to i-th output of control channel c
	virtual void linkInput(AbstractControlChannel* c, int i, int j)=0;

	virtual DataItemRead getOutput(int j)=0;

	// Setter method for the enclosing control channel
	void setEnclosingControlChannel(AbstractControlChannel* c)
		{ enclosingControlChannel = c; }

	// Methods to propagate operations to source blocks
	void reset(PropagationFlag flag, int token);
	bool isConfigured(PropagationFlag flag, int token);

	// Pure virtual method from interface Resettable
	virtual void reset();				

	// Pure virtual method from interface Configurable
	virtual void resetConfiguration();				
	virtual bool isConfigured()=0;

	// Straightforward getter and setter methods
	AocsTime getLastPropagatedTime()
		{ return lastPropagatedTime; }

	int getOutputOrder()
		{ return m; }

	int getInputOrder()
		{ return n; }

	void setIllegalTimeRecoveryAction(RecoveryAction* r)
		{ recoveryAction = r; }

	RecoveryAction* getIllegalTimeRecoveryAction()
		{ return recoveryAction; }

	AbstractControlChannel* getEnclosingControlChannel()
		{ return enclosingControlChannel; }

	int getPropagationToken()
		{ return propagationToken; }

	void setPropagationToken(int newToken)
		{ propagationToken=newToken; }

};

// -------------------------------------------------------------------
#endif
