// ControlChannelBlock.cpp
//
// Version	1.0
// Date		17.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include required for use of sqrt function
#include <stdio.h>

// Include framework header files
#include "../SequentialDataProcessing/ControlChannelBlock.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Default constructor.
// NB:	Unlike other constructors in the framework, this constructor does 
//		not call the "localReset" and "localResetConfiguration" methods because 
//		these methods initialize arrays whose size is only defined in subclasses
//		(this size depends on the input, output and state order of the block).
//		These methods are called by method memoryAllocate which completes the
//		object's construction by allocating memory to input, output and state
//		arrays. Method memoryAllocate is in turn called by the constructor of
//		the derived classes which are the ones who know how many inputs, outputs
//		and states pertain to each class of control channel blocks.

ControlChannelBlock::ControlChannelBlock()				
{
	p = 0;

	setClassId(ID_CONTROLCHANNELBLOCK);			// assign class identifier
}

// -------------------------------------------------------------------

void ControlChannelBlock::localReset()
{
	int i;
	
	validityTime = getTime();

	onHold = false;
	initFlag = true;	

	for (i=0; i<getInputOrder(); i++)
		input[i]=0;

	for (i=0; i<getOutputOrder(); i++)
	{	output[i]=0;
		outputBuffer[i]=0;
	}

	for (i=0; i<getStateOrder(); i++)
		state[i]=0;
}

// -------------------------------------------------------------------

void ControlChannelBlock::reset()
{
	AbstractControlChannel::reset();	// reset super object state
	localReset();
}

// -------------------------------------------------------------------

void ControlChannelBlock::localResetConfiguration()
{
	DataItemRead emptyDataItem(NULL);

	for (int i=0; i<getInputOrder(); i++)
	{	ccLink[i]=NULL;
		inputLink[i]=emptyDataItem;
	}
}

// -------------------------------------------------------------------

void ControlChannelBlock::resetConfiguration()
{
	AbstractControlChannel::resetConfiguration();	// reset super object's configuration
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ControlChannelBlock::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (getOutputOrder()==0)		// NB: Blocks can have zero inputs!
		return false;

	for (int i=0; i<getInputOrder(); i++)
	{	if ( !inputLink[i].isDataItemReadConfigured() )
			return false;
	}

	return true;
}

// -------------------------------------------------------------------

void ControlChannelBlock::allocateMemory(int i, int j, int k)
{
	// Check whether the memory was already allocated
	if ( (getInputOrder()>0) || (getOutputOrder()>0) || (getStateOrder()>0) )
	{	reportConfigurationError(this, MEMORY_ALREADY_ALLOCATED, this);
		return;
	}

	setInputOrder(i);
	input = new Real[getInputOrder()];
	inputLink = new DataItemRead[getInputOrder()];
	ccLink = new AbstractControlChannel*[getInputOrder()];
	
	setOutputOrder(j);
	output = new Real[getOutputOrder()];
	outputBuffer = new Real[getOutputOrder()];
	
	setStateOrder(k);
	state = new Real[getStateOrder()];

	// Reset the local object's state (see also comment attached to constructor)
	// NB This method is called by the constructor of derived classes, hence it is
	//	  necessary to specify that the version of localReset and localResetConfiguration
	//	  to be called is the one of class ControlChannelBlock
	ControlChannelBlock::localReset();		
	ControlChannelBlock::localResetConfiguration();
}

// -------------------------------------------------------------------

void ControlChannelBlock::updateInputs(AocsTime t)
{
	int i;
	
	// If this block takes its inputs from other control channels, then make
	// sure that the outputs of the supplying control channels are up-to-date.
	// If they are not, call propagate on them to force the updating of their outputs
	for (i=0; i<getInputOrder(); i++)
		if ( ccLink[i] != NULL )
//			if ( t > (ccLink[i]->getLastPropagatedTime()) )  removed on 31.05.00
				ccLink[i]->propagate(t);

	// Now load the input buffer
	for (i=0; i<getInputOrder(); i++)
		input[i] = inputLink[i].get();
}

// -------------------------------------------------------------------

void ControlChannelBlock::propagate(AocsTime t)
{
	if (t<lastPropagatedTime)
	{	reportFailure(this, ILLEGAL_PROPAGATION_TIME, 
						this, getIllegalTimeRecoveryAction());
		return;
	}

	updateInputs(t);
	if  ( (t>lastPropagatedTime) || (initFlag) )
	{	if ( !isOnHold() )
		{	propagationAlgorithm(t);
			updateOutputBuffer(t);
		}
		lastPropagatedTime = t;
		initFlag = false;
	}
}

// -------------------------------------------------------------------

void ControlChannelBlock::updateOutputBuffer(AocsTime t)
{
	for (int i=0; i<getOutputOrder(); i++)
		outputBuffer[i] = output[i];

	validityTime = t;
}
// -------------------------------------------------------------------

void ControlChannelBlock::linkInput(AbstractControlChannel* c, int i, int j)
{
	if ( j >= getInputOrder() )
	{	reportConfigurationError(this, NON_EXISTENT_INPUT, this);
		return;
	}

	inputLink[j] = c->getOutput(i);
	ccLink[j] = c;
}

// -------------------------------------------------------------------

void ControlChannelBlock::linkInput(DataItemRead d, int j)
{
	if ( j >= getInputOrder() )
	{	reportConfigurationError(this, NON_EXISTENT_INPUT, this);
		return;
	}

	inputLink[j] = d;
}

// -------------------------------------------------------------------

DataItemRead ControlChannelBlock::getOutput(int j)
{ 
	if ( j >= getOutputOrder() )
	{	reportConfigurationError(this, NON_EXISTENT_OUTPUT, this);
		return DataItemRead((Real*)NULL);
	}

	return DataItemRead(&outputBuffer[j]); 
}
