// TestCaseControlChannel_8.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseControlChannel_8.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/D_Block.h"
#include "../../SequentialDataProcessing/XmathUcbPidBlock.h"

// -------------------------------------------------------------------
TestCaseControlChannel_8::TestCaseControlChannel_8():
	TestCaseGenericSetUp(ID_ABSTRACTCONTROLCHANNELBLOCK*10+8,
									"TestControlChannel_8") {
		return;
}

// -------------------------------------------------------------------

void TestCaseControlChannel_8::runTestCase() {

	// Create the PID Xmath block
	AocsTime samplingTime=1;
	XmathUcbPidBlock pid(samplingTime);

	// Reset the block
	pid.reset();

	// Create a dummy data to serve as input for the super block
	Scalar inputSource;

	// Create a dummy object to act as owner of the data items in the input source
	AocsObject dataOwner;

	// Initialize the input to 0
	inputSource.getDataItemWrite(0).set(0);

	// Link the input source to the PID block
	pid.linkInput(inputSource.getDataItemRead(0),0);

	// Check that the PID block is properly configured
	if (!pid.isConfigured())
	{	setTestResult(TEST_FAILURE, "PID is not configured");
		return;
	}

	// Get the default gains from the PID
	Real localKp=pid.getKp();
	Real localKd=pid.getKd();
	Real localKi=pid.getKi();

	// Run the PID with the default gains and with a linear input
	int n=10;
	Real P_output, I_output, D_output;
	AocsTime timeStep=samplingTime;
	Real slope=2;
	AocsTime t0=aocsClock.getTime();
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	inputSource.getDataItemWrite(0).set(slope*(t-t0));	
		Real output=pid.getOutput(0).get();	// facilitates debugging
		pid.propagate(t);

		// Compute the expected PID output
		P_output=localKp*slope*(t-t0);
		I_output=localKi*0.5*slope*(t-t0)*((t-t0)+samplingTime);
		if (i==0)
			D_output=0.0;
		else
			D_output=localKd*slope;

		// Check correct value of PID output
		output=pid.getOutput(0).get();	
		Real expectedOuput = (P_output+D_output+I_output);
		if ( pid.getOutput(0).get() != (P_output+D_output+I_output) )
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}

		// Increment time
		t+=timeStep;
	}

	// Set all the integral and proportional gains to zero 
	pid.setKi(0);
	pid.setKp(0);

	// Run the PID for a few more cycles with a constant input. Its output
	// should now be equal to zero .
	for (i=0; i<n; i++)
	{	Real output=pid.getOutput(0).get();	// facilitates debugging
		pid.propagate(t);
		t+=timeStep;
		if ( pid.getOutput(0).get() != 0.0 )
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}
	}

	// Check that no failure or configuration events have been created
	if ( failureEvtRep.getCounter() != failureEvtN )
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}
	if ( configEvtRep.getCounter() != configEvtN )
	{	setTestResult(TEST_FAILURE, "Configuration event was generated");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
