// DmaTelecommandLoader.cpp
//
// Version	1.0
// Date		08.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../TelecommandManagement/DmaTelecommandLoader.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelecommandManagement/Telecommand.h"
#include "../TelecommandManagement/TransactionTelecommand.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Define static variables
int DmaTelecommandLoader::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

DmaTelecommandLoader::DmaTelecommandLoader(int* tcBufferStart)
{
	setClassId(ID_DMATELECOMMANDLOADER);
	
	tmImageLength[shortTm]  = 0;	// not implemented in fw prototype			
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	tcBuffer = (char*)tcBufferStart;
	ptr = 0;

	localResetConfiguration();

	localReset();
}

// -------------------------------------------------------------------

void DmaTelecommandLoader::localReset()
{
	// Clear all memory buffers
	int i, j;
	for (i=0; i<N_DATA_BLOCKS; i++)
	{	dataMemory[i].isFree = true;
		dataMemory[i].tcId = 0;
		for (j=0; j<DATA_BLOCK_SIZE; j++)
			dataMemory[i].memory[j] = 0;
	}

	for (i=0; i<N_VTABLE_BLOCKS; i++)
	{	vtableMemory[i].isFree = true;
		vtableMemory[i].tcId= 0;
		for (j=0; j<VTABLE_BLOCK_SIZE; j++)
			vtableMemory[i].memory[j] = 0;
	}

	for (i=0; i<N_CODE_BLOCKS; i++)
	{	codeMemory[i].isFree = true;
		codeMemory[i].tcId = 0;
		for (j=0; j<CODE_BLOCK_SIZE; j++)
			codeMemory[i].memory[j] = 0;
	}
}

// -------------------------------------------------------------------

void DmaTelecommandLoader::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void DmaTelecommandLoader::localResetConfiguration()
{
	tcLoaderRecoveryAction = NULL;
	tcManager = NULL;
}

// -------------------------------------------------------------------

void DmaTelecommandLoader::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool DmaTelecommandLoader::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (tcLoaderRecoveryAction==NULL)
		return false;

	if (tcManager==NULL)
		return false;

	return true;
}

// -------------------------------------------------------------------
// Process the TC packet. If the packet contains only a single block,
// then it contains only a simple telecommand. If the packet contains
// more than one block, then it contains a telecommands transaction
// with each block containing a transaction telecommand.
// When this method is entered ptr should point to the first byte in
// the TC packet.

void DmaTelecommandLoader::processTelecommandPacket()
{
	unsigned char numberOfTcBlocks = tcBuffer[ptr];	
	ptr++;

	if (numberOfTcBlocks == 1)
		processSimpleTelecommand();
	else
		processTelecommandTransaction(numberOfTcBlocks);
}

// -------------------------------------------------------------------
// Extract the simple TC from the TC block and load it into the 
// TC manager.

void DmaTelecommandLoader::processSimpleTelecommand()
{
	Telecommand* tc = makeSimpleTelecommand();
	tcManager->loadTelecommand(tc);
}

// -------------------------------------------------------------------
// Extract the transaction telecommands, link them together into a 
// single TC transaction and load it into the TC manager.
// numberOfTcBlocks is the number of TC's in the TC transaction.

void DmaTelecommandLoader::processTelecommandTransaction(int numberOfTcBlocks)
{
	AocsTime timeTag;
	TransactionTelecommand* nextTtc;

	TransactionTelecommand* firstTtc = makeTransactionTelecommand(true,NULL,&timeTag);
	TelecommandId tcId = firstTtc->getTelecommandId();
	
	TransactionTelecommand* ttc = firstTtc;

	for (unsigned char i=1; i<numberOfTcBlocks; i++)
	{	nextTtc = makeTransactionTelecommand(false, tcId, &timeTag);
		ttc->setNextTransactionTelecommand(nextTtc);
		ttc = nextTtc;
	}

	tcManager->loadTelecommand(firstTtc);
}

// -------------------------------------------------------------------
// Scan the data blocks and return the address of the first 
// unallocated data block. 

int* DmaTelecommandLoader::newData(InstanceId tcId)
{
	for (int i=0; i<N_DATA_BLOCKS; i++)
		if (dataMemory[i].isFree)
		{
			dataMemory[i].isFree = false;
			dataMemory[i].tcId = tcId;
			return &dataMemory[i].memory[0];
		}

	// If this point is reached, the memory could not be alllocated
	reportFailure(this,TC_OUT_OF_MEMORY,this,tcLoaderRecoveryAction);

	return NULL;
}

// -------------------------------------------------------------------
// Scan the vtable blocks and return the address of the first 
// unallocated vtable block. 

int* DmaTelecommandLoader::newVTable(InstanceId tcId)
{
	for (int i=0; i<N_VTABLE_BLOCKS; i++)
		if (vtableMemory[i].isFree)
		{
			vtableMemory[i].isFree = false;
			dataMemory[i].tcId = tcId;
			return &vtableMemory[i].memory[0];
		}

	// If this point is reached, the memory could not be alllocated
	reportFailure(this,TC_OUT_OF_MEMORY,this,tcLoaderRecoveryAction);

	return NULL;
}

// -------------------------------------------------------------------
// Scan the code blocks and return the address of the first 
// unallocated code block. 

int* DmaTelecommandLoader::newCode(InstanceId tcId)
{
	for (int i=0; i<N_CODE_BLOCKS; i++)
		if (codeMemory[i].isFree)
		{
			codeMemory[i].isFree = false;
			dataMemory[i].tcId = tcId;
			return &codeMemory[i].memory[0];
		}

	// If this point is reached, the memory could not be alllocated
	reportFailure(this,TC_OUT_OF_MEMORY,this,tcLoaderRecoveryAction);

	return NULL;
}

// -------------------------------------------------------------------
// Scan the data blocks, identify the one allocated to the argument
// telecommand and release it by marking it as 'free'. 

void DmaTelecommandLoader::releaseData(TelecommandId tcId)
{
	for (int i=0; i<N_DATA_BLOCKS; i++)
		if (dataMemory[i].tcId == tcId)
			dataMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Scan the VTable blocks, identify the one allocated to the argument
// telecommand and release it by marking it as 'free'. 

void DmaTelecommandLoader::releaseVTable(TelecommandId tcId)
{
	for (int i=0; i<N_VTABLE_BLOCKS; i++)
		if (vtableMemory[i].tcId == tcId)
			vtableMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Scan the code blocks, identify those allocated to the argument
// telecommand and release them by marking them as 'free'. 

void DmaTelecommandLoader::releaseCode(TelecommandId tcId)
{
	for (int i=0; i<N_CODE_BLOCKS; i++)
		if (codeMemory[i].tcId == tcId)
			codeMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Release all data blocks. 

void DmaTelecommandLoader::releaseDataAll()
{
	for (int i=0; i<N_DATA_BLOCKS; i++)
		dataMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Release all VTable blocks. 

void DmaTelecommandLoader::releaseVTableAll()
{
	for (int i=0; i<N_VTABLE_BLOCKS; i++)
		vtableMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Release all code blocks. 

void DmaTelecommandLoader::releaseCodeAll()
{
	for (int i=0; i<N_CODE_BLOCKS; i++)
		codeMemory[i].isFree = true;
}

// -------------------------------------------------------------------
// Free the memory allocated to the argument telecommand.

void DmaTelecommandLoader::release(TelecommandId tcId)
{
	releaseData(tcId);
	releaseVTable(tcId);
	releaseCode(tcId);
}

// -------------------------------------------------------------------
// Free all memory allocated to telecommands.

void DmaTelecommandLoader::releaseAll()
{
	releaseDataAll();
	releaseVTableAll();
	releaseCodeAll();
}

// -------------------------------------------------------------------

void DmaTelecommandLoader::run(AocsTime t)
{
	ptr = 1;								// start of first TC packet
	for (int i=0; i<tcBuffer[0]; i++)
		processTelecommandPacket();

	// Reset the telecommand buffer
	tcBuffer[0] = 0;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int DmaTelecommandLoader::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void DmaTelecommandLoader::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}

