// DmaTelecommandLoader.h
//
// Version	1.0
// Date		08.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Base class for DMA-based telecommand loaders. 
//
// The TC loader assumes that the TC data have been copied (probably
// by an ISR triggered by the arrival of the TC data) to a TC Buffer.
//
// A TC buffer contains one or more TC packets. Its structure is:
//	- byte 0: number of packets in the TC buffer
//	- ...	: TC packets in sequence
//
// A TC packet contains either a single simple telecommand or a
// telecommand transaction made up of a sequence of simple 
// telecommands.
//
// A TC packet has the following structure:
//	- byte 0: number of TC blocks in the packets
//	- . . . : TC blocks in sequence
//
// A TC block contains an individual telecommand.
// If there is only one TC block in the TC packet, then the packet
// contains a simple telecommand. If there are more than one TC 
// blocks, then the packet contains a TC transaction.
//
// A TC block has the following structure:
//	- byte 0: length in bytes of TC data section
//	- byte 1: length in longwords of TC VTable section
//	- . . .
//
// If byte 1 is 0, then the the TC uses a VTable section already
// present in memory and there is no code section.
//
// A data section has the following  structure:
//	- bytes 0-3: VTable pointer (only present if VTable is not 
//				 part of the telecommand)
//	- bytes 4-7: time tag (only in the first TC of a transaction)
//	- byte 8:	 TC class identifier
//	- ...		 TC data 
//
// TC's in a telecommand trasanction all have the same time-tag.
//
// A VTable section has the following structure:
//	- bytes 0-3: method presence longword
//	- bytes 4-7: first method pointer
//	- ...	   : other method pointers
//
// A code section consists of one or more method sections. Its 
// structure is as follows:
//	- byte 0: number of method sections
//	- ...	: method sections in sequence
//
// A method section contains the code implementing one method. Its 
// structure is as follows:
//	- byte 0: length of method section
//	- ...	: method data
//
// The TC loader maintains three memory areas from which memory can
// be allocated for storing the data, VTable and code sections of
// incoming telecommands. In order to make the memory allocation
// process predictable, memory is allocated in chuncks of fixed size.
//
// The TC buffer is processed sequentially by methods in this class.
// Variable ptr identifies the beginning of the next unit to be
// processed. Methods that process units in the TC buffer should
// update variable ptr. TC buffer units are: TC packets, TC blocks, 
// TC data sections, TC VTable section, TC code section.
// -------------------------------------------------------------------

#ifndef DmaTelecommandLoaderH
#define DmaTelecommandLoaderH

// -------------------------------------------------------------------

#include "../TelecommandManagement/TelecommandLoader.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"

// -------------------------------------------------------------------

#define TC_BUFFER_SIZE			100
#define DATA_BLOCK_SIZE			10
#define VTABLE_BLOCK_SIZE		20
#define CODE_BLOCK_SIZE			20
#define N_DATA_BLOCKS			10
#define N_VTABLE_BLOCKS			10
#define N_CODE_BLOCKS			10

// -------------------------------------------------------------------

class DmaTelecommandLoader : public TelecommandLoader, 
							  public AocsObject {

	static int tmImageLength[NTMFORMATS];

	TelecommandManager* tcManager;

	// A data block is the unit of memory allocation for data sections
	struct DataBlock {
		bool isFree;
		InstanceId tcId;
		int memory[DATA_BLOCK_SIZE];
	};

	// A VTable block is the unit of memory allocation for VTable sections
	struct VTableBlock {
		bool isFree;
		InstanceId tcId;
		int memory[VTABLE_BLOCK_SIZE];
	};

	// A code block is the unit of memory allocation for code sections
	struct CodeBlock {
		bool isFree;
		InstanceId tcId;
		int memory[CODE_BLOCK_SIZE];
	};

	// Arrays from which memory is allocated to telecommands
	DataBlock dataMemory[N_DATA_BLOCKS];
	VTableBlock vtableMemory[N_VTABLE_BLOCKS];
	CodeBlock codeMemory[N_CODE_BLOCKS];

	// Recovery action for internal errors
	RecoveryAction* tcLoaderRecoveryAction;

	// Local reset methods
	void localReset();
	void localResetConfiguration();

protected:

	// TC buffer and its pointer
	char* tcBuffer;		
	int ptr;

	// Process a TC packer
	void processTelecommandPacket();

	// Process a TC packet containing one single simple TC
	void processSimpleTelecommand();

	// Process a TC packet containing a TC transaction made up of 
	// numberOfTcBlocks transaction TC's
	void processTelecommandTransaction(int numberOfTcBlocks);

	// Construct a simple TC from the raw data in the TC buffer
	virtual Telecommand* makeSimpleTelecommand()=0;

	// Construct a transaction TC from the raw data in the TC buffer.
	// Flag isFirst is set to true when the TC to be constructed is
	// the first in the TC transaction.
	// When isFirst is not true, parameter tTcId is the TC id of the
	// first TC in the transaction.
	// Parameter timeTag is an output parameter when isFirst is true
	// (the time tag is read from the TC buffer) and an input parameter
	// when isFirst is false (TC's in a TC transaction all receive the
	// same time tag as the first TC in the transaction).
	virtual TransactionTelecommand* 
			makeTransactionTelecommand(bool isFirst, TelecommandId tTcId, AocsTime* timeTag)=0;

	// Memory allocation and deallocation methods
	int* newData(TelecommandId tcId);
	int* newVTable(TelecommandId tcId);
	int* newCode(TelecommandId tcId);
	void releaseData(TelecommandId tcId);
	void releaseVTable(TelecommandId tcId);
	void releaseCode(TelecommandId tcId);
	void releaseDataAll();
	void releaseVTableAll();
	void releaseCodeAll();

public:

	DmaTelecommandLoader(int* tcBuffer);

	void run(AocsTime t);

	// Release the memory allocated to a telecommand
	virtual void release(TelecommandId TcId);

    // Release all memory allocated to TC's
    virtual void releaseAll();

	// Virtual methods inherited from Resettable.
	virtual void reset();		

	// Virtual methods inherited from Configurable
	virtual void resetConfiguration();		
	virtual bool isConfigured();		
	
	// Virtual methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	void setTelecommandLoaderRecoveryAction(RecoveryAction* r)
		{ tcLoaderRecoveryAction = r; }
	RecoveryAction* getTelecommandLoaderRecoveryAction()
		{ return tcLoaderRecoveryAction; }

	void setTelecommandManager(TelecommandManager* m)
		{ tcManager = m; }
	TelecommandManager* getTelecommandManager()
		{ return tcManager; }

	char* getTelecommandBuffer()
		{ return tcBuffer; }
};

// -------------------------------------------------------------------
#endif