// TestCaseTelecommand_6.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseTelecommand_6.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../GeneralInclude/CompilerSwitches.h"
#include "../../AocsEvent/TelecommandEvent.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ManoeuvreEvent.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../TelecommandManagement/ManoeuvreTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTelecommand.h"
#include "../../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTransactionTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTransactionTelecommand.h"
#include "../../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../../TelecommandManagement/Erc32DmaTelecommandLoader.h"
#include "../../TelecommandManagement/TelecommandManager.h"
#include "../../TelecommandManagement/TestTelecommandLoader.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/DummyManoeuvre.h"

// -------------------------------------------------------------------
TestCaseTelecommand_6::TestCaseTelecommand_6():
	TestCaseGenericSetUp(ID_TELECOMMANDMANAGER*10+6,
									"TestCaseTelecommand_6") {
		return;
}

// -------------------------------------------------------------------
// Test the telecommand loader with manoeuvre TC.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The buffer consists of a single packet containing a single TC to
// load a manoeuvre in the manoeuvre manager.

void TestCaseTelecommand_6::runTestCase() {

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
	{	setTestResult(TEST_FAILURE, "Telecommand test 6 failed");
		return;
	}

	// Create and configure a manoeuvre manager where the TC will load
	// the test manoeuvre
	ManoeuvreManager mnManager;
	mnManager.initializeManoeuvreList(20);

	// Create the dummy manoeuvre that the TC will load
	AocsTime dmStartTime = 0;	// manoeuvre to start immediately
	int dmStepCount = 25;
	DummyManoeuvre dm(dmStartTime, dmStepCount);
	dm.setMonitorListSize(3);

	// Configure the static part of the manoeuvre telecommand
	ManoeuvreTelecommand::setManoeuvreManager(&mnManager);

	// Load the dummy manoeuvre address in a 4-byte array
	int dmAddressInt = (int)&dm;
	int* dmAddressPtr = &dmAddressInt;
	char* dmAddress = (char*)dmAddressPtr;

	// Create a dummy manoeuvre TC. It is needed to recover the VTable 
	// pointer for class ManoeuvreTelecommand.
	ManoeuvreTelecommand mntc(0,&dm);

	// Load the VTable pointer for class ManoeuvreTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mntcVTableAddress = *((int*)&mntc);
#else
	int mntcVTableAddress = *(((int*)&mntc+3));
#endif
	int* mntcVTablePtrInt = &mntcVTableAddress;
	char* mntcVTablePtr = (char*)mntcVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = 0;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. The TC buffer consists of a single 
	// TC packet containing the manoeuvre TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=13;	// start of TC block -- 13 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mntcVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mntcVTablePtr[1];
	tcBuffer[6]=mntcVTablePtr[2];
	tcBuffer[7]=mntcVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mntc.getClassId();	// class identifier

	tcBuffer[13]=dmAddress[0];		// manoeuvre address
	tcBuffer[14]=dmAddress[1];
	tcBuffer[15]=dmAddress[2];
	tcBuffer[16]=dmAddress[3];

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 6 failed");
		return;
	}

	// Run the TC manager -- the manoeuvre should be loaded in the 
	// manoeuvre manager
	tcm.run(0);

	// Run the manoeuvre manager -- the manoeuvre should be executed
	mnManager.run(0);

	// Check that two TC events were geenrated (TC was loaded and TC
	// was executed)
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 6 failed");
		return;
	}

	// Check that the manoeuvre was loaded in the manoeuvre manager
	if (mnManager.getManoeuvreEventRepository()->latestManoeuvreEvent()->
			getEventType()!=MANOEUVRE_STARTED)
	{	setTestResult(TEST_FAILURE, "Telecommand test 6 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
