// TelemetryManager.cpp
//
// Version	1.0
// Date		05.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/TelecommandEventRepository.h"
#include "../TelecommandManagement/TelecommandLoader.h"
#include "../TelecommandManagement/Telecommand.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Define static variables
int TelecommandManager::tmImageLength[NTMFORMATS];
TelecommandEventRepository*	TelecommandManager::tcEventRepository=NULL;

// -------------------------------------------------------------------

TelecommandManager::TelecommandManager(int maxNumberPendingTc)
{
	setClassId(ID_TELECOMMANDMANAGER);		

	tmImageLength[shortTm]  = 0;	// not implemented in fw prototype			
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	tcList.allocateMemory(maxNumberPendingTc);

	localResetConfiguration();

	localReset();
}

// -------------------------------------------------------------------

void TelecommandManager::localResetConfiguration()
{
	tcLoader = NULL;
	tcListFullRecoveryAction = NULL;
	tcStackErrorRecoveryAction = NULL;
	tcList.resetConfiguration();
}

// -------------------------------------------------------------------

void TelecommandManager::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void TelecommandManager::resetStaticConfiguration()
{
	tcEventRepository = NULL;
}

// -------------------------------------------------------------------

void TelecommandManager::localReset()
{
	// Reset TC data stack pointers
	intStackPointer = 0;
	realStackPointer = 0;

	// Reset pending TC list 
	tcList.reset();

	// Remove all pending telecommands from tcList, this requires the
	// configuration of tcList to be reset and the resources allocated
	// to the telecommands to  be released
	tcList.resetConfiguration();
	if (tcLoader != NULL)
		tcLoader->releaseAll();
}

// -------------------------------------------------------------------

void TelecommandManager::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

bool TelecommandManager::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (tcEventRepository==NULL)
		return false;

	if (tcLoader==NULL)
		return false;

	if (!tcList.isConfigured())
		return false;

	if (tcListFullRecoveryAction==NULL)
		return false;

	if (tcStackErrorRecoveryAction==NULL)
		return false;

	return true;
}

// -------------------------------------------------------------------

void TelecommandManager::pushInt(int value)
{
	if (intStackPointer==TC_INT_STACK_SIZE)
		reportFailure(this,TC_STACK_OVERFLOW,this,tcStackErrorRecoveryAction);
	else
	{	intStack[intStackPointer]=value;
		intStackPointer++;
	}
}

// -------------------------------------------------------------------

int TelecommandManager::popInt()
{
	if (intStackPointer==0)
	{	reportFailure(this,TC_STACK_UNDERFLOW,this,tcStackErrorRecoveryAction);
		return 0;
	}
	else
	{	intStackPointer--;
		return intStack[intStackPointer];
	}
}

// -------------------------------------------------------------------

void TelecommandManager::pushReal(Real value)
{
	if (realStackPointer==TC_REAL_STACK_SIZE)
		reportFailure(this,TC_STACK_OVERFLOW,this,tcStackErrorRecoveryAction);
	else
	{	realStack[realStackPointer]=value;
		realStackPointer++;
	}
}

// -------------------------------------------------------------------

Real TelecommandManager::popReal()
{
	if (realStackPointer==0)
	{	reportFailure(this,TC_STACK_UNDERFLOW,this,tcStackErrorRecoveryAction);
		return 0;
	}
	else
	{	realStackPointer--;
		return realStack[realStackPointer];
	}
}

// -------------------------------------------------------------------

int TelecommandManager::getNumberOfTelecommands()
{
	int n=0;
	for (Telecommand* t=tcList.first(); 
			!tcList.isLast(); 
				t=tcList.next())
		n++;

	return n;
}

// -------------------------------------------------------------------

void TelecommandManager::loadTelecommand(Telecommand* telecommand)
{
	if (tcList.isListFull())
		reportFailure(this,TC_LIST_FULL,this,tcListFullRecoveryAction);
	else
	{	tcList.addObject(telecommand);
		tcEventRepository->create(this,TC_LOADED,telecommand->getTelecommandId());
	}
}

// -------------------------------------------------------------------

void TelecommandManager::run(AocsTime deadline)
{
	for (Telecommand* t=tcList.first(); 
			!tcList.isLast(); 
				t=tcList.next())
	{
		if (t->getTimeTag()<=getTime())
		{	
			if (t->isEnabled() && t->canExecute())
			{
				// Reset TC data stack pointers
				intStackPointer = 0;
				realStackPointer = 0;

				// Execute TC - return code indicates execution status 
				EventType e=(EventType)t->execute();

				// Create TC event
				tcEventRepository->create(this,e,t->getTelecommandId());

				// Unload telecommand 
				tcList.removeObject(t);
			}
			else 
			{	
				// Unload telecommand 
				tcList.removeObject(t);
				tcEventRepository->create(this,TC_UNLOADED,t->getTelecommandId());
			}

			// Release memory used by telecommand in the TC loader
			tcLoader->release(t->getTelecommandId());
		}
	}
}								

// -------------------------------------------------------------------
// From Telemeterable interface

int TelecommandManager::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void TelecommandManager::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
