// VsDmaTelecommandLoader.cpp
//
// Version	1.0
// Date		09.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelecommandManagement/Telecommand.h"
#include "../TelecommandManagement/TransactionTelecommand.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------

VsDmaTelecommandLoader::VsDmaTelecommandLoader(int* tcBufferStart):
								DmaTelecommandLoader(tcBufferStart)
{
	setClassId(ID_VSDMATELECOMMANDLOADER);
	
	instanceIdCounter = TC_INSTANCE_ID_BASE;
}

// -------------------------------------------------------------------
// When this method is called, ptr should be pointing to the first 
// byte in a TC block.

Telecommand* VsDmaTelecommandLoader::makeSimpleTelecommand()
{
	int lengthDataSection = tcBuffer[ptr];
	int lengthVTableSection = tcBuffer[ptr+1];
	ptr +=2;

	if (lengthVTableSection==0)
		return makeDataTelecommand(lengthDataSection);
	else
		return makeCodeTelecommand(lengthDataSection, lengthVTableSection);
}

// -------------------------------------------------------------------
// Construct a simple TC whose code is already present in memory.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.

Telecommand* VsDmaTelecommandLoader::makeDataTelecommand(int lengthDataSection)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	char* tcStart = (char*)newData(instanceId);

	tcStart[0] = tcBuffer[ptr];		// copy TC VTable pointer
	tcStart[1] = tcBuffer[ptr+1];		
	tcStart[2] = tcBuffer[ptr+2];		
	tcStart[3] = tcBuffer[ptr+3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[4] = instanceIdPtr[0];	// copy instance ID
	tcStart[5] = instanceIdPtr[1];

	tcStart[6] = tcBuffer[ptr+8];	// copy class ID		
	tcStart[7] = 10;				// unused byte		

	tcStart[8] = 1;					// initialize 'enabled' to 'true'
	tcStart[9] = 11;				// unused byte	
	tcStart[10] = 12;				// unused byte
	tcStart[11] = 13;				// unused byte

	tcStart[12] = tcBuffer[ptr+4];	// copy time tag
	tcStart[13] = tcBuffer[ptr+5];		
	tcStart[14] = tcBuffer[ptr+6];		
	tcStart[15] = tcBuffer[ptr+7];		

	// Copy remaining data
	for (int i=0; i<(lengthDataSection-9); i++)
		tcStart[16+i] = tcBuffer[ptr+9+i];

	// Update the tcBuffer pointer
	ptr += lengthDataSection;

	return (Telecommand*)tcStart;
}

// -------------------------------------------------------------------
// Construct a simple TC whose code is partly in the TC block.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.
// Caution: this method was never tested!

Telecommand* VsDmaTelecommandLoader::makeCodeTelecommand(
						int lengthDataSection, int lengthVTableSection)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	// Allocate memory for the VTable
	int* vtable = newVTable(instanceId);

	// Allocate memory for the telecommand object
	char* tcStart = (char*)newData(instanceId);

	char* vtableChar = (char*)vtable;
	tcStart[0] = vtableChar[0];		// set up VTable pointer
	tcStart[1] = vtableChar[1];		
	tcStart[2] = vtableChar[2];		
	tcStart[3] = vtableChar[3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[4] = instanceIdPtr[0];	// copy instance ID
	tcStart[5] = instanceIdPtr[1];

	tcStart[6] = tcBuffer[ptr+4];	// copy class ID		
	tcStart[7] = 10;				// unused byte		

	tcStart[8] = 1;					// initialize 'enabled' to 'true'
	tcStart[9] = 11;				// unused byte	
	tcStart[10] = 12;				// unused byte
	tcStart[11] = 13;				// unused byte

	tcStart[12] = tcBuffer[ptr];	// copy time tag
	tcStart[13] = tcBuffer[ptr+1];		
	tcStart[14] = tcBuffer[ptr+2];		
	tcStart[15] = tcBuffer[ptr+3];		

	// Copy remaining data 
	for (int i=0; i<(lengthDataSection-5); i++)
		tcStart[16+i] = tcBuffer[ptr+5+i];

	// Update the tcBuffer pointer - ptr should then point to the beginning
	// of the VTable section
	ptr += lengthDataSection;

	// Retrieve the method presence longword
	int methodPresenceLongword;
	char* methodPresenceLongwordChar = (char*)&methodPresenceLongword;
	methodPresenceLongwordChar[0] = tcBuffer[ptr];
	methodPresenceLongwordChar[1] = tcBuffer[ptr+1];
	methodPresenceLongwordChar[2] = tcBuffer[ptr+2];
	methodPresenceLongwordChar[3] = tcBuffer[ptr+3];

	// Process telecommand methods in sequence.
	// The length of the VTable section is equal to the number of methods
	// of the telecommand + 1 (the method presence word)

	int methodCounter = 0;
	int methodSectionStart = ptr+lengthVTableSection;

	for (i=0; i<lengthVTableSection-1; i++)
	{	
		if (methodPresenceLongword & 0x00000001)
		{
			vtable[4*i] = tcBuffer[ptr+4+4*methodCounter];
			vtable[4*i+1] = tcBuffer[ptr+4+4*methodCounter+1];
			vtable[4*i+2] = tcBuffer[ptr+4+4*methodCounter+2];
			vtable[4*i+3] = tcBuffer[ptr+4+4*methodCounter+3];
			methodCounter++;
		}
		else
		{
			char* methodStart = makeMethodSection(methodSectionStart, instanceId);
			vtable[4*i] = methodStart[0];
			vtable[4*i+1] = methodStart[1];
			vtable[4*i+2] = methodStart[2];
			vtable[4*i+3] = methodStart[3];
			methodSectionStart = methodSectionStart+tcBuffer[methodSectionStart]; 
		}			
		methodPresenceLongword = methodPresenceLongword>>1;
	}

	// Update tcBuffer ptr - it should then point to the first byte following 
	// the end of the TC block
	ptr = methodSectionStart;

	return (Telecommand*)tcStart;
}

// -------------------------------------------------------------------
// Construct a method section.
// The method section code is in tcBuffer starting at position 
// methodSectionStart.
// The method returns the address of the method.

char* VsDmaTelecommandLoader::makeMethodSection(
							int methodSectionStart, TelecommandId tcId)
{
	char* methodCodeStart = (char*)newCode(tcId);

	int methodCodeLength = tcBuffer[methodSectionStart];

	for (int i=0; i<4*methodCodeLength; i++)
		methodCodeStart[i] = tcBuffer[methodSectionStart+1+i];

	return methodCodeStart;
}

// -------------------------------------------------------------------
// When this method is called, ptr should be pointing to the first 
// byte in a TC block.
// If 'isFirst' is 'true', then this TC block contains the first TC in 
// a TC transaction and the method returns the time tag of the TC 
// transaction in 'timeTag'.
// If 'isFirst' is 'false', then 'timeTag' is an input parameter and
// it contains the time tag to be assigned to the TC.

TransactionTelecommand* VsDmaTelecommandLoader::makeTransactionTelecommand(
							bool isFirst, TelecommandId tcId, AocsTime* timeTag)
{
	int lengthDataSection = tcBuffer[ptr];
	int lengthVTableSection = tcBuffer[ptr+1];
	ptr +=2;

	if (lengthVTableSection==0)
		return makeDataTransactionTelecommand(
					lengthDataSection, isFirst, tcId, timeTag);
	else
		return makeCodeTransactionTelecommand(
			lengthDataSection, lengthVTableSection, isFirst, tcId, timeTag);
}

// -------------------------------------------------------------------
// Construct a transaction TC whose code is already present in memory.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.

TransactionTelecommand* VsDmaTelecommandLoader::makeDataTransactionTelecommand(
			int lengthDataSection, bool isFirst, TelecommandId tcId, AocsTime* timeTag)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	// Allocate memory for the new TC
	char* tcStart;
	if (isFirst)
		tcStart = (char*)newData((TelecommandId)instanceId);
	else
		tcStart = (char*)newData(tcId);

	tcStart[0] = tcBuffer[ptr];			// copy TC VTable pointer
	tcStart[1] = tcBuffer[ptr+1];		
	tcStart[2] = tcBuffer[ptr+2];		
	tcStart[3] = tcBuffer[ptr+3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[4] = instanceIdPtr[0];		// copy instance ID
	tcStart[5] = instanceIdPtr[1];

	tcStart[8] = 1;						// initialize 'enabled' to 'true'
	tcStart[9] = 10;					// unused byte
	tcStart[10] = 11;					// unused byte
	tcStart[11] = 12;					// unused byte

	if (isFirst)
	{	
		tcStart[12] = tcBuffer[ptr+4];	// copy time tag
		tcStart[13] = tcBuffer[ptr+5];		
		tcStart[14] = tcBuffer[ptr+6];		
		tcStart[15] = tcBuffer[ptr+7];		
		AocsTime* timeTagDummy = (AocsTime*)&tcStart[12];
		*timeTag = *timeTagDummy;

		tcStart[6] = tcBuffer[ptr+8];	// copy class ID
		tcStart[7] = 13;				// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data 
		for (int i=0; i<(lengthDataSection-9); i++)
			tcStart[24+i] = tcBuffer[ptr+9+i];
	}
	else
	{
		char* timeTagChar = (char*)timeTag;
		tcStart[12] = timeTagChar[0];	// copy time tag
		tcStart[13] = timeTagChar[1];		
		tcStart[14] = timeTagChar[2];		
		tcStart[15] = timeTagChar[3];		

		tcStart[6] = tcBuffer[ptr+4];	// copy class ID		
		tcStart[7] = 13;				// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data
		for (int i=0; i<(lengthDataSection-5); i++)
			tcStart[24+i] = tcBuffer[ptr+5+i];
	}

	// Update the tcBuffer pointer
	ptr += lengthDataSection;

	return (TransactionTelecommand*)tcStart;
}

// -------------------------------------------------------------------
// Construct a transaction TC whose code is partly in the TC block.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.
// Caution: this method was never tested!

TransactionTelecommand* VsDmaTelecommandLoader::makeCodeTransactionTelecommand(
 	int lengthDataSection, int lengthVTableSection, bool isFirst, 
		TelecommandId tcId, AocsTime* timeTag)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	// Allocate memory for the VTable
	int* vtable;
	if (isFirst)
		vtable = newVTable((TelecommandId)instanceId);
	else
		vtable = newVTable(tcId);

	// Allocate memory for the telecommand object
	char* tcStart;
	if (isFirst)
		tcStart = (char*)newData((TelecommandId)instanceId);
	else
		tcStart = (char*)newData(tcId);

	char* vtableChar = (char*)vtable;
	tcStart[0] = vtableChar[0];		// set up VTable pointer
	tcStart[1] = vtableChar[1];		
	tcStart[2] = vtableChar[2];		
	tcStart[3] = vtableChar[3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[4] = instanceIdPtr[0];	// copy instance ID
	tcStart[5] = instanceIdPtr[1];

	tcStart[8] = 1;					// initialize 'enabled' to 'true'
	tcStart[9] = 10;				// unused byte
	tcStart[10] = 11;				// unused byte
	tcStart[11] = 12;				// unused byte

	if (isFirst)
	{	
		tcStart[12] = tcBuffer[ptr+4];	// copy time tag
		tcStart[13] = tcBuffer[ptr+5];		
		tcStart[14] = tcBuffer[ptr+6];		
		tcStart[15] = tcBuffer[ptr+7];		
		AocsTime* timeTagDummy = (AocsTime*)&tcStart[12];
		*timeTag = *timeTagDummy;

		tcStart[6] = tcBuffer[ptr+8];	// copy class ID
		tcStart[7] = 13;				// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data
		for (int i=0; i<(lengthDataSection-9); i++)
			tcStart[24+i] = tcBuffer[ptr+9+i];
	}
	else
	{
		char* timeTagChar = (char*)timeTag;
		tcStart[12] = timeTagChar[0];	// copy time tag
		tcStart[13] = timeTagChar[0];		
		tcStart[14] = timeTagChar[0];		
		tcStart[15] = timeTagChar[0];		

		tcStart[6] = tcBuffer[ptr+4];	// copy class ID		
		tcStart[7] = 13;				// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data
		for (int i=0; i<(lengthDataSection-5); i++)
			tcStart[24+i] = tcBuffer[ptr+5+i];
	}

	// Update the tcBuffer pointer - ptr should then point to the beginning
	// of the VTable section
	ptr += lengthDataSection;

	// Retrieve the method presence longword
	int methodPresenceLongword;
	char* methodPresenceLongwordChar = (char*)&methodPresenceLongword;
	methodPresenceLongwordChar[0] = tcBuffer[ptr];
	methodPresenceLongwordChar[1] = tcBuffer[ptr+1];
	methodPresenceLongwordChar[2] = tcBuffer[ptr+2];
	methodPresenceLongwordChar[3] = tcBuffer[ptr+3];

	// Process telecommand methods in sequence.
	// The length of the VTable section is equal to the number of methods
	// of the telecommand + 1 (the method presence word)

	int methodCounter = 0;
	int methodSectionStart = ptr+lengthVTableSection;

	for (int i=0; i<lengthVTableSection-1; i++)
	{	
		if (methodPresenceLongword & 0x00000001)
		{
			vtable[4*i] = tcBuffer[ptr+4+4*methodCounter];
			vtable[4*i+1] = tcBuffer[ptr+4+4*methodCounter+1];
			vtable[4*i+2] = tcBuffer[ptr+4+4*methodCounter+2];
			vtable[4*i+3] = tcBuffer[ptr+4+4*methodCounter+3];
			methodCounter++;
		}
		else
		{
			char* methodStart;
			if (isFirst)
				methodStart = 
					(char*)makeMethodSection(methodSectionStart, instanceId);
			else
				methodStart = 
					(char*)makeMethodSection(methodSectionStart, tcId);

			vtable[4*i] = methodStart[0];
			vtable[4*i+1] = methodStart[1];
			vtable[4*i+2] = methodStart[2];
			vtable[4*i+3] = methodStart[3];
			methodSectionStart = methodSectionStart+tcBuffer[methodSectionStart]; 
		}			
		methodPresenceLongword = methodPresenceLongword>>1;
	}

	// Update tcBuffer ptr - it should then point to the first byte following 
	// the end of the TC block
	ptr = methodSectionStart;

	return (TransactionTelecommand*)tcStart;
}

